@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: amenu.c,v 1.14 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: amenu.c,v 1.37 2004/07/26 10:02:44 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -43,89 +42,41 @@
 /*#include	"fight.h"*/
 /*#include	"fx.h"*/
 /*#include	"trap.h"*/
-/*#include	"draw.h"*/
+#include	"draw.h"
 /*#include	"curs.h"*/
 #include	"menu.h"
+#include	"amenu.h"
+/*#include	"request.h"*/
+/*#include	"gfile.h"*/
 /*#include	"msg.h"*/
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-/*#include	"turn_prot.h"*/
-#include	"dun_prot.h"
-/*#include	"town_prot.h"*/
-#include	"item_prot.h"
-/*#include	"spell_prot.h"*/
-#include	"chr_prot.h"
-#include	"party_prot.h"
-#include	"mnstr_prot.h"
-/*#include	"pet_prot.h"*/
-/*#include	"fight_prot.h"*/
-/*#include	"fx_prot.h"*/
-/*#include	"trap_prot.h"*/
-#include	"draw_prot.h"
-/*#include	"curs_prot.h"*/
-#include	"menu_prot.h"
-/*#include	"tmenu_prot.h"*/
-#include	"amenu_prot.h"
-/*#include	"gfile_prot.h"*/
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+/*#include	"turn-prot.h"*/
+#include	"dun-prot.h"
+/*#include	"town-prot.h"*/
+#include	"item-prot.h"
+/*#include	"spell-prot.h"*/
+#include	"chr-prot.h"
+#include	"party-prot.h"
+#include	"mnstr-prot.h"
+/*#include	"pet-prot.h"*/
+/*#include	"fight-prot.h"*/
+/*#include	"fx-prot.h"*/
+/*#include	"trap-prot.h"*/
+#include	"draw-prot.h"
+/*#include	"curs-prot.h"*/
+#include	"menu-prot.h"
+/*#include	"tmenu-prot.h"*/
+#include	"amenu-prot.h"
+/*#include	"request-prot.h"*/
+/*#include	"gfile-prot.h"*/
+/*#include	"msg-prot.h"*/
 
 /**/
 
 /* ưޡ */
 
-#define	AUTO_MARK_MAX_N	12
-
-#define	FLG_AUTO_MARK_EDIT_CONT	(1 << 0)
-#define	FLG_AUTO_MARK_EDIT_PRIORITY	(1 << 1)
-
-typedef enum {
-	AUTO_MARK_TRGT_MNSTR,
-	AUTO_MARK_TRGT_ITEM,
-	AUTO_MARK_TRGT_DOOR,
-	AUTO_MARK_TRGT_TRAP,
-	AUTO_MARK_TRGT_MBR,
-	AUTO_MARK_TRGT_UNCHANGE,
-	AUTO_MARK_TRGT_CLEAR,
-} auto_mark_trgt_t;
-
-typedef enum {
-	AUTO_MARK_DIST_NEAR,
-	AUTO_MARK_DIST_MEDIUM,
-	AUTO_MARK_DIST_FAR,
-} auto_mark_dist_t;
-
-typedef enum {
-	AUTO_MARK_ACT_FIGHT,
-	AUTO_MARK_ACT_THROW,
-	AUTO_MARK_ACT_TAIL,
-	AUTO_MARK_ACT_PICK_UP,
-	AUTO_MARK_ACT_BREAK,
-	AUTO_MARK_ACT_OPEN,
-	AUTO_MARK_ACT_CLOSE,
-	AUTO_MARK_ACT_JAM,
-	AUTO_MARK_ACT_DISARM,
-	AUTO_MARK_ACT_PEEP,
-	AUTO_MARK_ACT_SRCH,
-} auto_mark_act_t;
-
-typedef enum {
-	AUTO_MARK_COND_NEAREST,
-	AUTO_MARK_COND_DISPERSE,
-	AUTO_MARK_COND_STAGGER,
-	AUTO_MARK_COND_OWN,
-} auto_mark_cond_t;
-
-typedef struct {
-	auto_mark_trgt_t	trgt;
-	auto_mark_dist_t	dist;
-	auto_mark_act_t	act;
-	auto_mark_cond_t	cond;
-	unsigned long	flg;
-} auto_mark_t;
-
-/**/
-
 static auto_mark_t	ls_auto_mark[LS_MBR_MAX_N][AUTO_MARK_MAX_N];
 static char	msg_ttl_auto_mark[AUTO_MARK_MAX_N]
 		[MSG_TTL_AUTO_MARK_MAX_LEN + 1];
@@ -133,6 +84,14 @@ static long	auto_mark_cur_n;
 static mbr_t	*g_sel_mbr;
 
 static auto_mark_t	dflt_auto_mark_fight = {
+	AUTO_MARK_TRGT_MNSTR,
+	AUTO_MARK_DIST_NEAR,
+	AUTO_MARK_ACT_FIGHT,
+	AUTO_MARK_COND_DISPERSE,
+	FLG_AUTO_MARK_EDIT_CONT,
+};
+
+static auto_mark_t	dflt_auto_mark_fight_pri = {
 	AUTO_MARK_TRGT_MNSTR,
 	AUTO_MARK_DIST_NEAR,
 	AUTO_MARK_ACT_FIGHT,
@@ -141,6 +100,14 @@ static auto_mark_t	dflt_auto_mark_fight = {
 };
 
 static auto_mark_t	dflt_auto_mark_throw = {
+	AUTO_MARK_TRGT_MNSTR,
+	AUTO_MARK_DIST_MEDIUM,
+	AUTO_MARK_ACT_THROW,
+	AUTO_MARK_COND_DISPERSE,
+	FLG_NULL,
+};
+
+static auto_mark_t	dflt_auto_mark_throw_pri = {
 	AUTO_MARK_TRGT_MNSTR,
 	AUTO_MARK_DIST_MEDIUM,
 	AUTO_MARK_ACT_THROW,
@@ -149,6 +116,14 @@ static auto_mark_t	dflt_auto_mark_throw = {
 };
 
 static auto_mark_t	dflt_auto_mark_fight_far = {
+	AUTO_MARK_TRGT_MNSTR,
+	AUTO_MARK_DIST_FAR,
+	AUTO_MARK_ACT_FIGHT,
+	AUTO_MARK_COND_DISPERSE,
+	FLG_NULL,
+};
+
+static auto_mark_t	dflt_auto_mark_fight_far_pri = {
 	AUTO_MARK_TRGT_MNSTR,
 	AUTO_MARK_DIST_FAR,
 	AUTO_MARK_ACT_FIGHT,
@@ -157,6 +132,14 @@ static auto_mark_t	dflt_auto_mark_fight_far = {
 };
 
 static auto_mark_t	dflt_auto_mark_fight_all = {
+	AUTO_MARK_TRGT_MNSTR,
+	AUTO_MARK_DIST_NEAR,
+	AUTO_MARK_ACT_FIGHT,
+	AUTO_MARK_COND_DISPERSE,
+	FLG_NULL,
+};
+
+static auto_mark_t	dflt_auto_mark_fight_all_pri = {
 	AUTO_MARK_TRGT_MNSTR,
 	AUTO_MARK_DIST_NEAR,
 	AUTO_MARK_ACT_FIGHT,
@@ -194,7 +177,20 @@ static auto_mark_appoint_item_t	auto_mark_appoint_item
 
 static auto_mark_appoint_item_t	auto_mark_appoint_item_dflt
 		[AUTO_MARK_APPOINT_ITEM_MAX_N] = {
-	{ FACE_MJR_WEAPON, FACE_MNR_SWORD, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_KNIFE, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_SHORT_SWORD, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_LONG_SWORD, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_SABRE, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_RAPIER, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_HAND_AXE, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_BATTLE_AXE, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_SPEAR, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_LANCE, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_FLAIL, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_MACE, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_HAMMER, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_GLAIVE, TRUE },
+	{ FACE_MJR_WEAPON, FACE_MNR_KNUCKLE, TRUE },
 	{ FACE_MJR_WEAPON, FACE_MNR_BOW, TRUE },
 	{ FACE_MJR_WEAPON, FACE_MNR_ARW, TRUE },
 	{ FACE_MJR_WEAPON, FACE_MNR_STONE, TRUE },
@@ -227,10 +223,13 @@ static auto_mark_appoint_item_t	auto_mark_appoint_item_dflt
 
 	{ FACE_MJR_GEM, FACE_MNR_NULL, TRUE },
 
+	{ FACE_MJR_MISC, FACE_MNR_INST, TRUE },
+	{ FACE_MJR_MISC, FACE_MNR_ANTHOLOGY, TRUE },
 	{ FACE_MJR_MISC, FACE_MNR_LIGHT, TRUE },
 	{ FACE_MJR_MISC, FACE_MNR_SPIKE, TRUE },
 	{ FACE_MJR_MISC, FACE_MNR_TOBACCO, TRUE },
 	{ FACE_MJR_MISC, FACE_MNR_PIPE, TRUE },
+	{ FACE_MJR_MISC, FACE_MNR_RARE_GOODS, TRUE },
 	{ FACE_MJR_MISC, FACE_MNR_JUNK, FALSE },
 
 	{ FACE_MJR_NULL, FACE_MNR_NULL, FALSE },
@@ -238,10 +237,14 @@ static auto_mark_appoint_item_t	auto_mark_appoint_item_dflt
 
 /**/
 
-#include	"amenu_tab.h"
+#include	"amenu-tab.h"
 
 /**/
 
+/*****************************************************************
+* 
+*****************************************************************/
+
 void	init_auto_menu( void )
 {
 	long	i, j, k;
@@ -262,7 +265,9 @@ void	init_auto_menu( void )
 	init_auto_mark_all_mbr();
 }
 
-/**/
+/*****************************************************************
+* ٤ƤΥСν
+*****************************************************************/
 
 void	init_auto_mark_all_mbr( void )
 {
@@ -272,13 +277,28 @@ void	init_auto_mark_all_mbr( void )
 		init_auto_mark_mbr( i );
 }
 
-/**/
+/*****************************************************************
+* Сν
+* long ls_mbr_n : СΥꥹֹ
+*****************************************************************/
 
 void	init_auto_mark_mbr( long ls_mbr_n )
 {
 	long	j;
 
+	/*  */
+
+	for( j = 0; j < AUTO_MARK_MAX_N; j++ ){
+		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_clear;
+		strncpy( msg_ttl_auto_mark[j],
+				MSG_NAME_DFLT_AUTO_MARK_CLEAR,
+				MSG_TTL_AUTO_MARK_MAX_LEN );
+	}
+
+	/* Ʈ(ɲ) */
+
 	j = 0;
+
 	switch( ls_mbr_n ){
 	default:
 	case 0:
@@ -298,6 +318,31 @@ void	init_auto_mark_mbr( long ls_mbr_n )
 			MSG_NAME_DFLT_AUTO_MARK_FIGHT,
 			MSG_TTL_AUTO_MARK_MAX_LEN );
 	j++;
+
+	/* Ʈ() */
+
+	switch( ls_mbr_n ){
+	default:
+	case 0:
+	case 1:
+	case 3:
+		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_fight_pri;
+		break;
+	case 2:
+		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_throw_pri;
+		break;
+	case 4:
+	case 5:
+		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_fight_far_pri;
+		break;
+	}
+	strncpy( msg_ttl_auto_mark[j],
+			MSG_NAME_DFLT_AUTO_MARK_FIGHT_PRI,
+			MSG_TTL_AUTO_MARK_MAX_LEN );
+	j++;
+
+	/* ܶƮ(ɲ) */
+
 	if( j < AUTO_MARK_MAX_N ){
 		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_fight_all;
 		strncpy( msg_ttl_auto_mark[j],
@@ -305,6 +350,21 @@ void	init_auto_mark_mbr( long ls_mbr_n )
 				MSG_TTL_AUTO_MARK_MAX_LEN );
 	}
 	j++;
+
+	/* ܶƮ() */
+
+	if( j < AUTO_MARK_MAX_N ){
+		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_fight_all_pri;
+		strncpy( msg_ttl_auto_mark[j],
+				MSG_NAME_DFLT_AUTO_MARK_FIGHT_ALL_PRI,
+				MSG_TTL_AUTO_MARK_MAX_LEN );
+	}
+	j++;
+
+	/* ƤΥƥ */
+
+	j += 2;
+
 	if( j < AUTO_MARK_MAX_N ){
 		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_item_all;
 		strncpy( msg_ttl_auto_mark[j],
@@ -312,6 +372,9 @@ void	init_auto_mark_mbr( long ls_mbr_n )
 				MSG_TTL_AUTO_MARK_MAX_LEN );
 	}
 	j++;
+
+	/* äƤ륢ƥ */
+
 	if( j < AUTO_MARK_MAX_N ){
 		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_item_own;
 		strncpy( msg_ttl_auto_mark[j],
@@ -319,12 +382,7 @@ void	init_auto_mark_mbr( long ls_mbr_n )
 				MSG_TTL_AUTO_MARK_MAX_LEN );
 	}
 	j++;
-	for( ; j < AUTO_MARK_MAX_N; j++ ){
-		ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_clear;
-		strncpy( msg_ttl_auto_mark[j],
-				MSG_NAME_DFLT_AUTO_MARK_CLEAR,
-				MSG_TTL_AUTO_MARK_MAX_LEN );
-	}
+
 	j = AUTO_MARK_MAX_N - 1;
 	ls_auto_mark[ls_mbr_n][j] = dflt_auto_mark_clear;
 	strncpy( msg_ttl_auto_mark[j],
@@ -332,7 +390,12 @@ void	init_auto_mark_mbr( long ls_mbr_n )
 			MSG_TTL_AUTO_MARK_MAX_LEN );
 }
 
-/**/
+/*****************************************************************
+* ˥塼
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*menu_auto_mark( menu_t **pp, menu_t **dflt )
 {
@@ -345,7 +408,10 @@ char	*menu_auto_mark( menu_t **pp, menu_t **dflt )
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* ˥塼ΥꥹȤ
+* menu_t **dflt : ˥塼Υǥե
+*****************************************************************/
 
 void	set_menu_auto_mark( menu_t **dflt )
 {
@@ -369,7 +435,7 @@ void	set_menu_auto_mark( menu_t **dflt )
 	strncpy( p[k].kind, "<Title>", MENU_KIND_MAX_LEN );
 	p[k].key = '\0';
 	p[k].accel_key[0] = '\0';
-	strncpy( p[k].msg, MSG_MENU_AUTO_MARK_TITLE,
+	strncpy( p[k].msg, MSG_TTL_MENU_AUTO_MARK,
 			MENU_MSG_MAX_LEN );
 	p[k].call_back = NULL;
 	p[k].arg = NULL;
@@ -386,7 +452,7 @@ void	set_menu_auto_mark( menu_t **dflt )
 
 		sprintf( p[k].path, "/Auto mark/%ld", i );
 		strncpy( p[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
-		p[k].key = 'A' + i;
+		p[k].key = (char)('A' + i);
 		p[k].accel_key[0] = '\0';
 		strncpy( p[k].msg, get_msg_ttl_auto_mark( i ),
 				MENU_MSG_MAX_LEN );
@@ -431,7 +497,12 @@ void	set_menu_auto_mark( menu_t **dflt )
 	p[k].path[0] = '\0';
 }
 
-/**/
+/*****************************************************************
+* ˥塼Ǽ¹Ԥ򤵤줿˸ƤФ
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_exec( menu_t **pp, menu_t **dflt )
 {
@@ -454,22 +525,27 @@ char	*cb_menu_auto_mark_exec( menu_t **pp, menu_t **dflt )
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Ū˥ޡʤʤϢ³ޡ
+* mbr_t *mbr : С
+*****************************************************************/
 
 void	set_auto_mark_cont( mbr_t *mbr )
 {
 	if( mbr == NULL )
 		return;
 
-	if( !chk_flg( ls_auto_mark[mbr->ls_mbr_n][auto_mark_cur_n].flg,
+	if( chk_flg( ls_auto_mark[mbr->ls_mbr_n][auto_mark_cur_n].flg,
 			FLG_AUTO_MARK_EDIT_PRIORITY ) ){
-		return;
+		mbr->flg_chr &= ~(FLG_CHR_AUTO_MARK_CONT);
 	}
-
-	mbr->flg_chr &= ~(FLG_CHR_AUTO_MARK_CONT);
 }
 
-/**/
+/*****************************************************************
+* ˥塼¹Ԥ
+* mbr_t *mbr : С
+* return : Ԥ
+*****************************************************************/
 
 bool_t	exec_menu_auto_mark( mbr_t *mbr )
 {
@@ -526,7 +602,11 @@ bool_t	exec_menu_auto_mark( mbr_t *mbr )
 	return TRUE;
 }
 
-/**/
+/*****************************************************************
+* 󥹥оݤ˥˥塼¹Ԥ
+* mbr_t *mbr : С
+* return : Ԥ
+*****************************************************************/
 
 bool_t	exec_menu_auto_mark_mnstr( mbr_t *mbr )
 {
@@ -544,6 +624,9 @@ bool_t	exec_menu_auto_mark_mnstr( mbr_t *mbr )
 	case AUTO_MARK_ACT_THROW:
 		act_kind = ACT_KIND_MNSTR_THROW;
 		break;
+	case AUTO_MARK_ACT_STEAL:
+		act_kind = ACT_KIND_MNSTR_STEAL;
+		break;
 	case AUTO_MARK_ACT_TAIL:
 		act_kind = ACT_KIND_MNSTR_TAIL;
 		break;
@@ -554,7 +637,7 @@ bool_t	exec_menu_auto_mark_mnstr( mbr_t *mbr )
 	case AUTO_MARK_ACT_DISARM:
 	case AUTO_MARK_ACT_BREAK:
 	case AUTO_MARK_ACT_PEEP:
-	case AUTO_MARK_ACT_SRCH:
+	case AUTO_MARK_ACT_CHK:
 	default:
 		return FALSE;
 	}
@@ -603,7 +686,11 @@ bool_t	exec_menu_auto_mark_mnstr( mbr_t *mbr )
 	return TRUE;
 }
 
-/**/
+/*****************************************************************
+* ƥоݤ˥˥塼¹Ԥ
+* mbr_t *mbr : С
+* return : Ԥ
+*****************************************************************/
 
 bool_t	exec_menu_auto_mark_item( mbr_t *mbr )
 {
@@ -617,6 +704,7 @@ bool_t	exec_menu_auto_mark_item( mbr_t *mbr )
 	switch( am->act ){
 	case AUTO_MARK_ACT_FIGHT:
 	case AUTO_MARK_ACT_THROW:
+	case AUTO_MARK_ACT_STEAL:
 	case AUTO_MARK_ACT_TAIL:
 		return FALSE;
 	case AUTO_MARK_ACT_PICK_UP:
@@ -636,8 +724,8 @@ bool_t	exec_menu_auto_mark_item( mbr_t *mbr )
 		break;
 	case AUTO_MARK_ACT_PEEP:
 		return FALSE;
-	case AUTO_MARK_ACT_SRCH:
-		act_kind = ACT_KIND_ITEM_SRCH;
+	case AUTO_MARK_ACT_CHK:
+		act_kind = ACT_KIND_ITEM_CHK;
 		break;
 	default:
 		return FALSE;
@@ -689,7 +777,11 @@ bool_t	exec_menu_auto_mark_item( mbr_t *mbr )
 	return TRUE;
 }
 
-/**/
+/*****************************************************************
+* ɥоݤ˥˥塼¹Ԥ
+* mbr_t *mbr : С
+* return : Ԥ
+*****************************************************************/
 
 bool_t	exec_menu_auto_mark_door( mbr_t *mbr )
 {
@@ -703,6 +795,7 @@ bool_t	exec_menu_auto_mark_door( mbr_t *mbr )
 	switch( am->act ){
 	case AUTO_MARK_ACT_FIGHT:
 	case AUTO_MARK_ACT_THROW:
+	case AUTO_MARK_ACT_STEAL:
 	case AUTO_MARK_ACT_TAIL:
 	case AUTO_MARK_ACT_PICK_UP:
 		return FALSE;
@@ -724,8 +817,8 @@ bool_t	exec_menu_auto_mark_door( mbr_t *mbr )
 	case AUTO_MARK_ACT_PEEP:
 		act_kind = ACT_KIND_DOOR_PEEP;
 		break;
-	case AUTO_MARK_ACT_SRCH:
-		act_kind = ACT_KIND_DOOR_SRCH;
+	case AUTO_MARK_ACT_CHK:
+		act_kind = ACT_KIND_DOOR_CHK;
 		break;
 	default:
 		return FALSE;
@@ -775,7 +868,11 @@ bool_t	exec_menu_auto_mark_door( mbr_t *mbr )
 	return TRUE;
 }
 
-/**/
+/*****************************************************************
+* Соݤ˥˥塼¹Ԥ
+* mbr_t *mbr : С
+* return : Ԥ
+*****************************************************************/
 
 bool_t	exec_menu_auto_mark_mbr( mbr_t *mbr )
 {
@@ -828,6 +925,9 @@ bool_t	exec_menu_auto_mark_mbr( mbr_t *mbr )
 	case AUTO_MARK_ACT_THROW:
 		act_kind = ACT_KIND_MBR_THROW;
 		break;
+	case AUTO_MARK_ACT_STEAL:
+		act_kind = ACT_KIND_MBR_STEAL;
+		break;
 	case AUTO_MARK_ACT_TAIL:
 		act_kind = ACT_KIND_MBR_TAIL;
 		break;
@@ -838,7 +938,7 @@ bool_t	exec_menu_auto_mark_mbr( mbr_t *mbr )
 	case AUTO_MARK_ACT_DISARM:
 	case AUTO_MARK_ACT_BREAK:
 	case AUTO_MARK_ACT_PEEP:
-	case AUTO_MARK_ACT_SRCH:
+	case AUTO_MARK_ACT_CHK:
 		return FALSE;
 	default:
 		return FALSE;
@@ -855,7 +955,12 @@ bool_t	exec_menu_auto_mark_mbr( mbr_t *mbr )
 	return TRUE;
 }
 
-/**/
+/*****************************************************************
+* ˥塼Խ򤵤줿˸ƤФ
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_edit( menu_t **pp, menu_t **dflt )
 {
@@ -864,7 +969,7 @@ char	*cb_menu_auto_mark_edit( menu_t **pp, menu_t **dflt )
 	auto_mark_cur_n = (long)((*pp)->arg);
 
 	while( 1 ){
-		str_goto = exec_menu_sel_mbr( &g_sel_mbr, NULL, TRUE );
+		str_goto = exec_menu_sel_mbr( &g_sel_mbr, NULL, TRUE, FALSE );
 		if( str_goto != NULL )
 			return str_goto;
 
@@ -884,7 +989,12 @@ char	*cb_menu_auto_mark_edit( menu_t **pp, menu_t **dflt )
 	return STR_MENU_ERR;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_edit_init(
 	menu_t **pp, menu_t **dflt
@@ -955,7 +1065,12 @@ char	*cb_menu_auto_mark_edit_init(
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼оݤ
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_trgt_init(
 	menu_t **pp, menu_t **dflt
@@ -1001,7 +1116,12 @@ char	*cb_menu_auto_mark_trgt_init(
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼оݤ򤷤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_trgt( menu_t **pp, menu_t **dflt )
 {
@@ -1039,7 +1159,12 @@ char	*cb_menu_auto_mark_trgt( menu_t **pp, menu_t **dflt )
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼δֹ礤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_dist_init(
 	menu_t **pp, menu_t **dflt
@@ -1082,7 +1207,12 @@ char	*cb_menu_auto_mark_dist_init(
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼δֹ礤򤷤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_dist( menu_t **pp, menu_t **dflt )
 {
@@ -1092,7 +1222,12 @@ char	*cb_menu_auto_mark_dist( menu_t **pp, menu_t **dflt )
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼ιư
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_act_init( menu_t **pp, menu_t **dflt )
 {
@@ -1125,6 +1260,9 @@ char	*cb_menu_auto_mark_act_init( menu_t **pp, menu_t **dflt )
 	case AUTO_MARK_ACT_THROW:
 		s = "Throw";
 		break;
+	case AUTO_MARK_ACT_STEAL:
+		s = "Steal";
+		break;
 	case AUTO_MARK_ACT_TAIL:
 		s = "Tail";
 		break;
@@ -1149,8 +1287,8 @@ char	*cb_menu_auto_mark_act_init( menu_t **pp, menu_t **dflt )
 	case AUTO_MARK_ACT_PEEP:
 		s = "Peep";
 		break;
-	case AUTO_MARK_ACT_SRCH:
-		s = "Search";
+	case AUTO_MARK_ACT_CHK:
+		s = "Check";
 		break;
 	}
 
@@ -1172,6 +1310,7 @@ char	*cb_menu_auto_mark_act_init( menu_t **pp, menu_t **dflt )
 	case AUTO_MARK_TRGT_ITEM:
 		set_flg_menu( *pp, "Fight", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Throw", FLG_MENU_HIDDEN );
+		set_flg_menu( *pp, "Steal", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Tail", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Close", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Jam", FLG_MENU_HIDDEN );
@@ -1180,12 +1319,14 @@ char	*cb_menu_auto_mark_act_init( menu_t **pp, menu_t **dflt )
 	case AUTO_MARK_TRGT_DOOR:
 		set_flg_menu( *pp, "Fight", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Throw", FLG_MENU_HIDDEN );
+		set_flg_menu( *pp, "Steal", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Tail", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Pick up", FLG_MENU_HIDDEN );
 		break;
 	case AUTO_MARK_TRGT_TRAP:
 		set_flg_menu( *pp, "Fight", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Throw", FLG_MENU_HIDDEN );
+		set_flg_menu( *pp, "Steal", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Tail", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Pick up", FLG_MENU_HIDDEN );
 		set_flg_menu( *pp, "Open", FLG_MENU_HIDDEN );
@@ -1214,7 +1355,12 @@ char	*cb_menu_auto_mark_act_init( menu_t **pp, menu_t **dflt )
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼ιư򤷤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_act( menu_t **pp, menu_t **dflt )
 {
@@ -1224,7 +1370,12 @@ char	*cb_menu_auto_mark_act( menu_t **pp, menu_t **dflt )
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼ξ
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_cond_init(
 	menu_t **pp, menu_t **dflt
@@ -1296,14 +1447,19 @@ char	*cb_menu_auto_mark_cond_init(
 	p = srch_menu( *pp, "Title" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_TTL_MENU_AUTO_MARK_ACT,
+	sprintf( p->msg, MSG_TTL_MENU_AUTO_MARK_COND,
 			get_msg_ttl_auto_mark( auto_mark_cur_n ),
 			g_sel_mbr->name );
 
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼ξ򤷤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_cond( menu_t **pp, menu_t **dflt )
 {
@@ -1329,7 +1485,12 @@ char	*cb_menu_auto_mark_cond( menu_t **pp, menu_t **dflt )
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼Υƥ̤򤷤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_appoint_item(
 	menu_t **pp, menu_t **dflt
@@ -1380,14 +1541,14 @@ char	*cb_menu_auto_mark_appoint_item(
 	if( apo == NULL )
 		return STR_MENU_ERR;
 
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		char	*ret;
 
 		if( apo->mjr == FACE_MJR_NULL )
 			break;
 
 		ret = set_menu_auto_mark_appoint_item(
-				ls, &k, 'A' + i, &apo );
+				ls, &k, (char)('A' + i), &apo );
 		if( ret != NULL )
 			return ret;
 	}
@@ -1399,7 +1560,14 @@ char	*cb_menu_auto_mark_appoint_item(
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼Υƥ̤
+* menu_t *ls : ˥塼
+* long *k : Υ˥塼ֹ
+* char mjr_key : 򥭡
+* auto_mark_appoint_item_t **apo : ƥ̤Υǡ
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*set_menu_auto_mark_appoint_item(
 	menu_t *ls, long *k, char mjr_key,
@@ -1449,7 +1617,12 @@ char	*set_menu_auto_mark_appoint_item(
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼Υƥ̤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_appoint_item_init(
 	menu_t **pp, menu_t **dflt
@@ -1476,7 +1649,12 @@ char	*cb_menu_auto_mark_appoint_item_init(
 	return NULL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼Υƥ̤ξܺ٤򤷤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_appoint_item_mnr(
 	menu_t **pp, menu_t **dflt
@@ -1493,7 +1671,12 @@ char	*cb_menu_auto_mark_appoint_item_mnr(
 	return STR_MENU_CANCEL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼Ϣ³ޡ
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_cont( menu_t **pp, menu_t **dflt )
 {
@@ -1508,7 +1691,12 @@ char	*cb_menu_auto_mark_cont( menu_t **pp, menu_t **dflt )
 	return STR_MENU_CANCEL;
 }
 
-/**/
+/*****************************************************************
+* Խ˥塼ͥ
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_priority( menu_t **pp, menu_t **dflt )
 {
@@ -1523,7 +1711,12 @@ char	*cb_menu_auto_mark_priority( menu_t **pp, menu_t **dflt )
 	return STR_MENU_CANCEL;
 }
 
-/**/
+/*****************************************************************
+* Ƭ¤ٴ˥塼򤷤
+* menu_t **pp : ˥塼
+* menu_t **dflt : ˥塼Υǥե
+* return : ˥塼ΰư
+*****************************************************************/
 
 char	*cb_menu_auto_mark_arrange( menu_t **pp, menu_t **dflt )
 {
@@ -1566,7 +1759,11 @@ char	*cb_menu_auto_mark_arrange( menu_t **pp, menu_t **dflt )
 	return "/Auto mark/0";
 }
 
-/**/
+/*****************************************************************
+* оݤоݤʸѴ
+* mbr_t *mbr : С
+* return : оݤΥ˥塼ʸ
+*****************************************************************/
 
 char	*get_msg_menu_auto_mark_edit_trgt( mbr_t *mbr )
 {
@@ -1590,7 +1787,11 @@ char	*get_msg_menu_auto_mark_edit_trgt( mbr_t *mbr )
 	return MSG_NOT_NULL;
 }
 
-/**/
+/*****************************************************************
+* ֹ礤ֹ礤ʸѴ
+* mbr_t *mbr : С
+* return : ֹ礤Υ˥塼ʸ
+*****************************************************************/
 
 char	*get_msg_menu_auto_mark_edit_dist( mbr_t *mbr )
 {
@@ -1614,7 +1815,11 @@ char	*get_msg_menu_auto_mark_edit_dist( mbr_t *mbr )
 	return MSG_NOT_NULL;
 }
 
-/**/
+/*****************************************************************
+* ưưʸѴ
+* mbr_t *mbr : С
+* return : ưΥ˥塼ʸ
+*****************************************************************/
 
 char	*get_msg_menu_auto_mark_edit_act( mbr_t *mbr )
 {
@@ -1631,6 +1836,8 @@ char	*get_msg_menu_auto_mark_edit_act( mbr_t *mbr )
 		return MSG_MENU_AUTO_MARK_ACT_FIGHT;
 	case AUTO_MARK_ACT_THROW:
 		return MSG_MENU_AUTO_MARK_ACT_THROW;
+	case AUTO_MARK_ACT_STEAL:
+		return MSG_MENU_AUTO_MARK_ACT_STEAL;
 	case AUTO_MARK_ACT_TAIL:
 		return MSG_MENU_AUTO_MARK_ACT_TAIL;
 	case AUTO_MARK_ACT_PICK_UP:
@@ -1647,14 +1854,18 @@ char	*get_msg_menu_auto_mark_edit_act( mbr_t *mbr )
 		return MSG_MENU_AUTO_MARK_ACT_BREAK;
 	case AUTO_MARK_ACT_PEEP:
 		return MSG_MENU_AUTO_MARK_ACT_PEEP;
-	case AUTO_MARK_ACT_SRCH:
-		return MSG_MENU_AUTO_MARK_ACT_SRCH;
+	case AUTO_MARK_ACT_CHK:
+		return MSG_MENU_AUTO_MARK_ACT_CHK;
 	}
 
 	return MSG_NOT_NULL;
 }
 
-/**/
+/*****************************************************************
+* 狼ʸѴ
+* mbr_t *mbr : С
+* return : Υ˥塼ʸ
+*****************************************************************/
 
 char	*get_msg_menu_auto_mark_edit_cond( mbr_t *mbr )
 {
@@ -1680,14 +1891,20 @@ char	*get_msg_menu_auto_mark_edit_cond( mbr_t *mbr )
 	return MSG_NOT_NULL;
 }
 
-/**/
+/*****************************************************************
+* ߤμưޡֹ֤
+* return : ߤμưޡֹ
+*****************************************************************/
 
 long	get_auto_mark_cur_n( void )
 {
 	return auto_mark_cur_n;
 }
 
-/**/
+/*****************************************************************
+* ߤμưޡΥƥ̤֤
+* return : ߤμưޡΥƥ
+*****************************************************************/
 
 auto_mark_appoint_item_t	*get_auto_mark_appoint_item( mbr_t *mbr )
 {
@@ -1701,11 +1918,29 @@ auto_mark_appoint_item_t	*get_auto_mark_appoint_item( mbr_t *mbr )
 	return auto_mark_appoint_item[mbr->ls_mbr_n][n];
 }
 
-/**/
+/*****************************************************************
+* ưޡΥȥʸ֤
+* n : ưޡֹ
+* return : ưޡΥȥʸ֤
+*****************************************************************/
 
 char	*get_msg_ttl_auto_mark( long n )
 {
 	return msg_ttl_auto_mark[n];
 }
 
-/**/
+/*****************************************************************
+* ưޡΥǡ֤
+* long ls_mbr_n : СꥹȤֹ
+* return : ưޡΥǡ֤
+*****************************************************************/
+
+auto_mark_t	*get_ls_auto_mark( long ls_mbr_n )
+{
+	if( ls_mbr_n < 0 )
+		return NULL;
+	if( ls_mbr_n >= LS_MBR_MAX_N )
+		return NULL;
+
+	return( ls_auto_mark[ls_mbr_n] );
+}
