@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,23 +25,27 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: curs.h,v 1.20 2003/11/23 05:42:51 bowkenken Exp $
+* $Id: curs.h,v 1.25 2004/07/26 10:44:24 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	CURS_H
 #define	CURS_H
 
+/* curses Υåѡ */
+
 #include	"msg.h"
 
 #if	HAVE_LIBCURSES
-#  include	<curses.h>
-#  ifndef	COLOR_WHITE
-#    include	"curs_def.h"
-#  endif	/* COLOR_WHITE */
+# include	<curses.h>
+# ifndef	COLOR_WHITE
+#  include	"curs-def.h"
+# endif	/* COLOR_WHITE */
 #else	/* HAVE_LIBCURSES */
-#  include	"curs_def.h"
+# include	"curs-def.h"
 #endif	/* HAVE_LIBCURSES */
 
+/**/
+
 #ifndef	KEY_RESIZE
 # define KEY_RESIZE	0632		/* Terminal resize event */
 #endif	/* KEY_RESIZE */
