@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,18 +25,25 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: draw.h,v 1.13 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: draw.h,v 1.31 2004/06/30 13:20:45 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	DRAW_H
 #define	DRAW_H
 
+/*  */
+
 #include	<stdio.h>
 
 #include	"pet.h"
 #include	"menu.h"
+#include	"request.h"
 
-#define	OUT_FP	stdout
+/**/
+
+#define	OUT_FP	stdout	/* ϥե롦ݥ */
+
+/* ꡼󡦥 */
 
 #define	SCREEN_WIDTH	80
 #define	SCREEN_HEIGHT	24
@@ -46,15 +52,41 @@
 #define	SCREEN_MAX_X	(SCREEN_WIDTH - 1)
 #define	SCREEN_MAX_Y	(SCREEN_HEIGHT - 1)
 
-/* vfx begin */
+#define	WIN_MAP_WIDTH	20
+#define	WIN_MAP_HEIGHT	20
+
+/* VFX */
 
 #define	VFX_ARW_MAX_N	32
 #define	ANIM_FRAME	60
 
 #define	ANIM_LINE_LEN	32
 
+/* ʸVFX */
+
+#define	TYPHOON_DUST_MAX_N	24
+#define	TYPHOON_SPEED	10
+#define	ANIM_TYPHOON_SEC	(4.0)
+#define	ANIM_TYPHOON_FRAME_N	(ANIM_FRAME * 5 / 100)
+
+#define	ANIM_CONFLAGRATION_SEC	(4.0)
+#define	ANIM_CONFLAGRATION_FRAME_N	(ANIM_FRAME * 5 / 100)
+#define	ANIM_CONFLAGRATION_HEIGHT	(WIN_MAP_DRAW_HEIGHT / 2)
+
+#define	ANIM_EARTHQUAKE_SEC	(4.0)
+#define	ANIM_EARTHQUAKE_FRAME_N	(ANIM_FRAME * 1 / 10)
+
+#define	ANIM_THUNDER_FRAME_N	(ANIM_FRAME * 5 / 100)
+#define	ANIM_THUNDER_CLR_FRAME_N	(ANIM_FRAME * 20 / 10)
+#define	ANIM_THUNDER_HEIGHT	(WIN_MAP_DRAW_HEIGHT / 2)
+
+#define	ANIM_INUNDATION_FRAME_N	(ANIM_FRAME * 5 / 100)
+#define	ANIM_INUNDATION_CLR_FRAME_N	(ANIM_FRAME * 20 / 10)
+
+/* ¾VFX */
+
 #define	ANIM_ARW_FRAME_N	\
-		((ANIM_FRAME * 10 / 10) - ANIM_ARW_CLR_FRAME_N)
+		((ANIM_FRAME * 15 / 10) - ANIM_ARW_CLR_FRAME_N)
 #define	ANIM_ARW_CLR_FRAME_N	(ANIM_FRAME * 5 / 10)
 
 #define	ANIM_CRTCL_FRAME_N	\
@@ -69,10 +101,10 @@
 
 #define	ANIM_SPELL_LEN	8
 #define	ANIM_SPELL_FRAME_N	\
-		((ANIM_FRAME * 10 / 10) - ANIM_ARW_CLR_FRAME_N)
+		((ANIM_FRAME * 15 / 10) - ANIM_SPELL_CLR_FRAME_N)
 #define	ANIM_SPELL_CLR_FRAME_N	(ANIM_FRAME * 5 / 10)
 
-/* vfx end */
+/* ɥΥ */
 
 #define	WIN_MAP_DRAW_MIN_X	0
 #define	WIN_MAP_DRAW_MIN_Y	0
@@ -90,7 +122,7 @@
 
 #define	WIN_MSG_DRAW_MIN_X	0
 #define	WIN_MSG_DRAW_MIN_Y	(WIN_MAP_DRAW_MAX_Y + 2)
-#define	WIN_MSG_DRAW_WIDTH	SCREEN_WIDTH
+#define	WIN_MSG_DRAW_WIDTH	(SCREEN_WIDTH - 1)
 #define	WIN_MSG_DRAW_HEIGHT	1
 #define	WIN_MSG_DRAW_MAX_X	(WIN_MSG_DRAW_MIN_X + WIN_MSG_DRAW_WIDTH - 1)
 #define	WIN_MSG_DRAW_MAX_Y	(WIN_MSG_DRAW_MIN_Y + WIN_MSG_DRAW_HEIGHT - 1)
@@ -99,6 +131,9 @@
 #define	WIN_MORE_DRAW_MIN_Y	WIN_MSG_DRAW_MIN_Y
 #define	WIN_MORE_DRAW_WIDTH	(str_len_draw( MSG_MORE ))
 
+#define	WIN_REQUEST_DRAW_WIDTH	40
+#define	WIN_REQUEST_DRAW_MIN_Y	4
+
 /**/
 
 #define	WIN_STAT_DRAW_MIN_X	(WIN_MAP_DRAW_MAX_X + 1 + 1)
@@ -110,35 +145,40 @@
 #define	WIN_STAT_DRAW_MAX_Y	(WIN_STAT_DRAW_MIN_Y \
 		+ ((MBR_MAX_N) * WIN_STAT_DRAW_HEIGHT - 1) - 1)
 
-#define	FLG_MSG_ERR	(1 << 0)
-#define	FLG_MSG_MORE	(1 << 1)
-#define	FLG_MSG_NO_MORE_PREV	(1 << 2)
-#define	FLG_MSG_NO_MORE_NEXT	(1 << 3)
+/* åɽؿΥե饰 */
 
 typedef unsigned long	flg_msg_t;
+#define	FLG_MSG_ERR	(1 << 0)	/* 顼 */
+#define	FLG_MSG_MORE	(1 << 1)	/* ³Ԥ */
+#define	FLG_MSG_NO_MORE_PREV	(1 << 2)	/* 1³Ԥ̵ */
+#define	FLG_MSG_NO_MORE_NEXT	(1 << 3)	/* 1ļ³Ԥ̵ */
+#define	FLG_MSG_DEBUG	(1 << 4)	/* ǥХåΤɽ */
 
-#define	WIN_MAP_WIDTH	20
-#define	WIN_MAP_HEIGHT	20
+#define	LS_MSG_DRAW_N	20	/* åꥹȤɽԿ */
+#define	LS_MSG_MAX_N	(LS_MSG_DRAW_N * 8)	/* åκԿ */
 
-#define	LS_MSG_DRAW_N	20
-#define	LS_MSG_MAX_N	(LS_MSG_DRAW_N * 8)
+/* VFX */
 
-#define	ANIM_FRAME_N	((long)(1.0 * 60.0))
+#define	ANIM_FRAME_N	((long)(1.0 * 60.0))	/* ե졼ࡦ졼 */
 
-typedef unsigned long	flg_print_t;
-
-#define	VFX_WAIT_MIN_N	1
-#define	VFX_WAIT_MAX_N	200
-#define	FLG_VFX_MBR_HP_ADD	(1 << 0)
-#define	FLG_VFX_MBR_HP_SUB	(1 << 1)
-#define	FLG_VFX_MBR_HP_SUB_CRTCL	(1 << 2)
-#define	FLG_VFX_MBR_ARW	(1 << 3)
-#define	FLG_VFX_MNSTR_HP_ADD	(1 << 0)
-#define	FLG_VFX_MNSTR_HP_SUB	(1 << 1)
-#define	FLG_VFX_MNSTR_HP_SUB_CRTCL	(1 << 2)
-#define	FLG_VFX_MNSTR_ARW	(1 << 3)
 typedef unsigned long	flg_vfx_mbr_t;
 typedef unsigned long	flg_vfx_mnstr_t;
+#define	VFX_WAIT_MIN_N	1
+#define	VFX_WAIT_MAX_N	200
+#define	FLG_VFX_MBR_CURE	(1 << 0)
+#define	FLG_VFX_MBR_DAM	(1 << 1)
+#define	FLG_VFX_MBR_CRTCL	(1 << 2)
+#define	FLG_VFX_MBR_ARW	(1 << 3)
+#define	FLG_VFX_MNSTR_CURE	(1 << 0)
+#define	FLG_VFX_MNSTR_DAM	(1 << 1)
+#define	FLG_VFX_MNSTR_CRTCL	(1 << 2)
+#define	FLG_VFX_MNSTR_ARW	(1 << 3)
+
+/* ơ֥ */
+
+#define	KEY_TAB_MAX_N	512
+
+/* ɽСǡμ */
 
 typedef enum {
 	DATA_N_NULL,
@@ -179,6 +219,9 @@ typedef enum {
 	DATA_N_STAT_HUNGRY,
 	DATA_N_STAT_FAINT,
 	DATA_N_STAT_DRUNK,
+	DATA_N_STAT_VANISH,
+	DATA_N_STAT_IRON_BODY,
+	DATA_N_STAT_PASSWALL,
 
 	DATA_N_LEV_LEFT,
 	DATA_N_EXP_LEFT,
@@ -192,8 +235,8 @@ typedef enum {
 	DATA_N_DEX,
 	DATA_N_STR,
 	DATA_N_AGI,
-	DATA_N_WIS,
 	DATA_N_INT,
+	DATA_N_WIS,
 	DATA_N_CHA,
 
 	DATA_N_RESI_TTL,
@@ -233,6 +276,8 @@ typedef enum {
 	DATA_N_MAX_N,
 } data_n_t;
 
+/* Խ륯饹ǡμ */
+
 typedef enum {
 	EDIT_CLASS_NULL,
 
@@ -241,8 +286,8 @@ typedef enum {
 	EDIT_CLASS_DEX,
 	EDIT_CLASS_STR,
 	EDIT_CLASS_AGI,
-	EDIT_CLASS_WIS,
 	EDIT_CLASS_INT,
+	EDIT_CLASS_WIS,
 	EDIT_CLASS_CHA,
 
 	EDIT_CLASS_TOTAL,
