@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: dun.c,v 1.34 2003/11/23 05:13:28 bowkenken Exp $
+* $Id: dun.c,v 1.65 2004/07/26 10:29:52 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -39,36 +38,40 @@
 /*#include	"chr.h"*/
 /*#include	"party.h"*/
 /*#include	"mnstr.h"*/
-/*#include	"pet.h"*/
+#include	"pet.h"
 /*#include	"fight.h"*/
 /*#include	"fx.h"*/
 /*#include	"trap.h"*/
-/*#include	"draw.h"*/
+#include	"draw.h"
 /*#include	"curs.h"*/
 #include	"menu.h"
+/*#include	"amenu.h"*/
+#include	"request.h"
+#include	"gfile.h"
 /*#include	"msg.h"*/
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-/*#include	"turn_prot.h"*/
-#include	"dun_prot.h"
-#include	"town_prot.h"
-#include	"item_prot.h"
-/*#include	"spell_prot.h"*/
-#include	"chr_prot.h"
-#include	"party_prot.h"
-#include	"mnstr_prot.h"
-/*#include	"pet_prot.h"*/
-/*#include	"fight_prot.h"*/
-#include	"fx_prot.h"
-#include	"trap_prot.h"
-#include	"draw_prot.h"
-#include	"curs_prot.h"
-#include	"menu_prot.h"
-/*#include	"tmenu_prot.h"*/
-/*#include	"amenu_prot.h"*/
-#include	"gfile_prot.h"
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+/*#include	"turn-prot.h"*/
+#include	"dun-prot.h"
+#include	"town-prot.h"
+#include	"item-prot.h"
+/*#include	"spell-prot.h"*/
+#include	"chr-prot.h"
+#include	"party-prot.h"
+#include	"mnstr-prot.h"
+/*#include	"pet-prot.h"*/
+/*#include	"fight-prot.h"*/
+#include	"fx-prot.h"
+#include	"trap-prot.h"
+#include	"draw-prot.h"
+#include	"curs-prot.h"
+#include	"menu-prot.h"
+/*#include	"tmenu-prot.h"*/
+/*#include	"amenu-prot.h"*/
+#include	"request-prot.h"
+#include	"gfile-prot.h"
+/*#include	"msg-prot.h"*/
 
 /**/
 
@@ -80,18 +83,21 @@
 
 #define	MIMIC_RATE	50
 
+#define	DUN_NPC_AVE_N	6
+
 /**/
 
 static dun_t	dun;
-static long	room_ptn_max_n;
+static area_t	room_ptn_max_n, boss_room_ptn_max_n;
 
 static crsr_ptn_t	g_crsr_ptn, g_crsr_ptn_usr[CRSR_PTN_MAX_N];
+static crsr_ptn_t	g_sub_crsr_ptn;
 static long	g_crsr_ptn_n, g_crsr_ptn_max_n;
 
 /**/
 
-#include	"dun_tab.h"
-#include	"crsr_ptn.h"
+#include	"dun-tab.h"
+#include	"crsr-ptn.h"
 
 /**/
 
@@ -107,18 +113,20 @@ void	init_dun( void )
 
 	if( load_crsr_ptn( g_crsr_ptn_usr, &g_crsr_ptn_max_n ) ){
 		for( i = 0; i < g_crsr_ptn_max_n; i++ )
-			set_crsr_ptn( &(g_crsr_ptn_usr[i]) );
+			set_crsr_ptn( &(g_crsr_ptn_usr[i]),
+					&(g_crsr_ptn_usr[i]) );
 	} else {
 		g_crsr_ptn_max_n = sizeof( crsr_ptn_dflt )
 				/ sizeof( crsr_ptn_dflt[0] );
 
 		for( i = 0; i < g_crsr_ptn_max_n; i++ ){
-			set_crsr_ptn( &(crsr_ptn_dflt[i]) );
+			set_crsr_ptn( &(crsr_ptn_dflt[i]),
+					&(crsr_ptn_dflt[i]) );
 			g_crsr_ptn_usr[i] = crsr_ptn_dflt[i];
 		}
 	}
 
-	set_crsr_ptn( &(g_crsr_ptn_usr[g_crsr_ptn_n]) );
+	reset_crsr_ptn();
 }
 
 /**/
@@ -163,7 +171,11 @@ long	set_crsr_ptn_n( long n )
 
 void	reset_crsr_ptn( void )
 {
-	set_crsr_ptn( &(g_crsr_ptn_usr[g_crsr_ptn_n]) );
+	long	sub_n;
+
+	sub_n = (g_crsr_ptn_n + 1) % g_crsr_ptn_max_n;
+	set_crsr_ptn( &g_crsr_ptn, &(g_crsr_ptn_usr[g_crsr_ptn_n]) );
+	set_crsr_ptn( &g_sub_crsr_ptn, &(g_crsr_ptn_usr[sub_n]) );
 }
 
 /**/
@@ -211,30 +223,39 @@ crsr_ptn_t	*get_crsr_ptn( void )
 
 /**/
 
-bool_t	set_crsr_ptn( crsr_ptn_t *ptn )
+crsr_ptn_t	*get_sub_crsr_ptn( void )
+{
+	return &g_sub_crsr_ptn;
+}
+
+/**/
+
+bool_t	set_crsr_ptn( crsr_ptn_t *dst, crsr_ptn_t *src )
 {
 	long	max_len;
 	long	y;
 
-	if( ptn == NULL )
+	if( src == NULL )
+		return FALSE;
+	if( dst == NULL )
 		return FALSE;
 
-	g_crsr_ptn = *ptn;
+	*dst = *src;
 
 	max_len = 0;
 	for( y = 0; y < CRSR_PTN_MAX_Y_LEN; y++ ){
 		long	len;
 
-		if( g_crsr_ptn.face[y][0] == '\0' )
+		if( dst->face[y][0] == '\0' )
 			break;
 
-		len = str_len_std( g_crsr_ptn.face[y] ) / 2;
-		max_len = max( max_len, len );
+		len = str_len_std( dst->face[y] ) / 2;
+		max_len = max_l( max_len, len );
 	}
-	g_crsr_ptn.x_len = max_len;
-	g_crsr_ptn.y_len = y;
-	ptn->x_len = max_len;
-	ptn->y_len = y;
+	dst->x_len = max_len;
+	dst->y_len = y;
+	src->x_len = max_len;
+	src->y_len = y;
 
 	return TRUE;
 }
@@ -243,26 +264,39 @@ bool_t	set_crsr_ptn( crsr_ptn_t *ptn )
 
 void	up_dun( long d )
 {
+	party_t	*pty = get_party();
 	long	i;
-	party_t	*pty;
-
-	pty = get_party();
 
 	dun.lev += d;
 
+	if( dun.lev > +DUN_MAX_LEV_FLOOR )
+		dun.lev = +DUN_MAX_LEV_FLOOR;
+	if( dun.lev < -DUN_MAX_LEV_BASE )
+		dun.lev = -DUN_MAX_LEV_BASE;
+
+	if( sgn_l( dun.lev ) >= +1 ){
+		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_FLOOR,
+				labs( dun.lev ) );
+	} else if( sgn_l( dun.lev ) <= -1 ){
+		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_BASE,
+				labs( dun.lev ) );
+	} else {
+		print_msg( FLG_NULL, MSG_UP_DOWN_DUN_GROUND );
+	}
+
 	for( i = 0; i < MBR_MAX_N; i++ ){
-		if( sgn( dun.lev ) >= +1 ){
+		if( sgn_l( dun.lev ) >= +1 ){
 			if( pty->mbr[i]->dun_max_lev_floor
-					< abs( dun.lev ) ){
+					< labs( dun.lev ) ){
 				pty->mbr[i]->dun_max_lev_floor
-						= abs( dun.lev );
+						= labs( dun.lev );
 			}
 		}
-		if( sgn( dun.lev ) <= -1 ){
+		if( sgn_l( dun.lev ) <= -1 ){
 			if( pty->mbr[i]->dun_max_lev_base
-					< abs( dun.lev ) ){
+					< labs( dun.lev ) ){
 				pty->mbr[i]->dun_max_lev_base
-						= abs( dun.lev );
+						= labs( dun.lev );
 			}
 		}
 	}
@@ -281,6 +315,7 @@ void	reset_all( bool_t flg_appear )
 	clr_dun_item();
 	reset_mnstr();
 	release_all_mbr();
+
 	if( dun.lev == 0 ){
 		make_town();
 		if( flg_appear ){
@@ -293,8 +328,11 @@ void	reset_all( bool_t flg_appear )
 			appear_party( FALSE );
 		}
 	}
+	put_request_all_obj();
+
 	next_pos_square( 0 );
 	set_flg_find_all_mnstr();
+
 	redraw_all();
 }
 
@@ -302,58 +340,83 @@ void	reset_all( bool_t flg_appear )
 
 void	make_dun( void )
 {
-	long	i, j;
+	long	x, y;
+	bool_t	flg_boss;
 	long	item_n, item_max_n;
 	long	mnstr_n, mnstr_max_n;
+	long	npc_n, npc_max_n;
 	long	trap_n, trap_max_n;
+	long	i, j;
 
 	reset_dun();
 
-	room_ptn_max_n = sizeof( room_ptn ) / sizeof( room_ptn_t );
+	room_ptn_max_n = sizeof( room_ptn )
+			/ sizeof( room_ptn_t );
+	boss_room_ptn_max_n = sizeof( boss_room_ptn )
+			/ sizeof( room_ptn_t );
+
+	/*  */
+
+	for( y = 0; y < AREA_MAP_MAX_Y; y++ ){
+		for( x = 0; x < AREA_MAP_MAX_X; x++ ){
+			dun.area[y][x] = AREA_PATH;
+		}
+	}
+
+	x = randm( AREA_MAP_MAX_X );
+	y = randm( AREA_MAP_MAX_Y );
+	flg_boss = make_area_boss( boss_room_ptn,
+			boss_room_ptn_max_n, x, y );
 
 	/* ¤ */
-	for( i = 0; i < AREA_MAP_MAX_Y; i++ ){
-		for( j = 0; j < AREA_MAP_MAX_X; j++ ){
-			make_area( room_ptn, j, i );
+
+	for( y = 0; y < AREA_MAP_MAX_Y; y++ ){
+		for( x = 0; x < AREA_MAP_MAX_X; x++ ){
+			make_area( room_ptn, room_ptn_max_n, x, y );
 		}
 	}
 
 	/* ϩ¤ */
-	for( i = 0; i < AREA_MAP_MAX_Y; i++ ){
-		for( j = 0; j < AREA_MAP_MAX_X; j++ ){
-			if( dun.area[i][j] == AREA_PATH ){
-				make_path( j, i );
+
+	for( y = 0; y < AREA_MAP_MAX_Y; y++ ){
+		for( x = 0; x < AREA_MAP_MAX_X; x++ ){
+			if( dun.area[y][x] == AREA_PATH ){
+				make_path( x, y );
 			}
 		}
 	}
 
 	/* ʿ */
-	for( i = 0; i < AREA_MAP_MAX_Y; i++ ){
-		for( j = 0; j < AREA_MAP_MAX_X - 1; j++ ){
-			if( !chk_room( j, i ) )
+
+	for( y = 0; y < AREA_MAP_MAX_Y; y++ ){
+		for( x = 0; x < AREA_MAP_MAX_X - 1; x++ ){
+			if( !chk_room( x, y ) )
 				continue;
-			if( chk_room( j + 1, i ) )
-				connect_room( j, i, +1, 0 );
+			if( chk_room( x + 1, y ) )
+				connect_room( x, y, +1, 0 );
 		}
 	}
 
 	/* ľ */
-	for( i = 0; i < AREA_MAP_MAX_Y - 1; i++ ){
-		for( j = 0; j < AREA_MAP_MAX_X; j++ ){
-			if( !chk_room( j, i ) )
+
+	for( y = 0; y < AREA_MAP_MAX_Y - 1; y++ ){
+		for( x = 0; x < AREA_MAP_MAX_X; x++ ){
+			if( !chk_room( x, y ) )
 				continue;
-			if( chk_room( j, i + 1 ) )
-				connect_room( j, i, 0, +1 );
+			if( chk_room( x, y + 1 ) )
+				connect_room( x, y, 0, +1 );
 		}
 	}
 
 	/* ޥåפüɤˤ */
+
 	make_fence();
 
 	/* ʤ¤ */
+
 	/*  */
-	for( i = 0; i < STAIRS_UP_MAX_N; i++ ){
-		for( j = 0; j < 4096; j++ ){
+	for( y = 0; y < STAIRS_UP_MAX_N; y++ ){
+		for( x = 0; x < LOOP_MAX_4000; x++ ){
 			long	x, y;
 
 			x = randm( MAP_MAX_X );
@@ -364,9 +427,10 @@ void	make_dun( void )
 			}
 		}
 	}
+
 	/*  */
-	for( i = 0; i < STAIRS_DOWN_MAX_N; i++ ){
-		for( j = 0; j < 4096; j++ ){
+	for( y = 0; y < STAIRS_DOWN_MAX_N; y++ ){
+		for( x = 0; x < LOOP_MAX_4000; x++ ){
 			long	x, y;
 
 			x = randm( MAP_MAX_X );
@@ -382,42 +446,43 @@ void	make_dun( void )
 
 	item_n = 0;
 
-	for( i = 0; i < MAP_MAX_Y; i++ ){
-		for( j = 0; j < MAP_MAX_X; j++ ){
-			if( dun.map.obj.mjr[i][j] != FACE_ROOM_PTN_ITEM )
+	for( y = 0; y < MAP_MAX_Y; y++ ){
+		for( x = 0; x < MAP_MAX_X; x++ ){
+			if( dun.map.obj.mjr[y][x] != FACE_ROOM_PTN_ITEM )
 				continue;
 
-			dun.map.obj.mjr[i][j] = FACE_MJR_FLOOR;
-			dun.map.obj.mnr[i][j] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[i][j] = FLG_MAP_OBJ_PASS;
-			if( make_item( j, i, dun.lev )
+			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
+			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
+			dun.map.obj.flg[y][x] = FLG_MAP_OBJ_PASS;
+			if( make_item( x, y, dun.lev )
 					!= ITEM_KIND_NULL ){
 				item_n++;
 			}
 		}
 	}
 
-	/* 󥹥 */
+	/* ߥߥå */
 
-	for( i = 0; i < MAP_MAX_Y; i++ ){
-		for( j = 0; j < MAP_MAX_X; j++ ){
-			if( dun.map.obj.mjr[i][j] != FACE_ROOM_PTN_MIMIC )
+	for( y = 0; y < MAP_MAX_Y; y++ ){
+		for( x = 0; x < MAP_MAX_X; x++ ){
+			if( dun.map.obj.mjr[y][x] != FACE_ROOM_PTN_MIMIC )
 				continue;
 
-			dun.map.obj.mjr[i][j] = FACE_MJR_FLOOR;
-			dun.map.obj.mnr[i][j] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[i][j] |= FLG_MAP_OBJ_PASS;
+			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
+			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
+			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
 			if( rate_randm( MIMIC_RATE ) ){
-				make_mnstr( j, i, dun.lev,
+				make_mnstr( x, y, dun.lev,
 						MNSTR_KIND_MIMIC );
 			} else {
 				item_t	*item;
 
 				item = make_item_chest( dun.lev,
-						get_chest_n( dun.lev ) );
+						get_chest_n( dun.lev ),
+						TRUE );
 				if( item == NULL )
 					continue;
-				if( !put_item( j, i, 1, item, FALSE ) ){
+				if( !put_item( item, x, y, 1, FALSE ) ){
 					del_item( item );
 					continue;
 				}
@@ -425,28 +490,52 @@ void	make_dun( void )
 		}
 	}
 
-	/*  */
+	/* 󥹥 */
 
-	for( i = 0; i < MAP_MAX_Y; i++ ){
-		for( j = 0; j < MAP_MAX_X; j++ ){
-			if( dun.map.obj.mjr[i][j] != FACE_ROOM_PTN_TRAP )
+	for( y = 0; y < MAP_MAX_Y; y++ ){
+		for( x = 0; x < MAP_MAX_X; x++ ){
+			mnstr_kind_t	mnstr_kind;
+			char	mjr, mnr;
+
+			mjr = dun.map.obj.mjr[y][x];
+			mnr = dun.map.obj.mnr[y][x];
+
+			/* GOD ֤ʤ */
+			if( !isalpha( mjr ) )
 				continue;
 
-			dun.map.obj.mjr[i][j] = FACE_MJR_FLOOR;
-			dun.map.obj.mnr[i][j] = FACE_MNR_FLOOR;
-			dun.map.obj.flg[i][j] |= FLG_MAP_OBJ_PASS;
+			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
+			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
+			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
 
-			make_trap( j, i, dun.lev );
+			mnstr_kind = get_mnstr_kind_from_face( mjr, mnr );
+			if( mnstr_kind == MNSTR_KIND_NULL )
+				continue;
+
+			make_mnstr( x, y, dun.lev, mnstr_kind );
+		}
+	}
+
+	/*  */
+
+	for( y = 0; y < MAP_MAX_Y; y++ ){
+		for( x = 0; x < MAP_MAX_X; x++ ){
+			if( dun.map.obj.mjr[y][x] != FACE_ROOM_PTN_TRAP )
+				continue;
+
+			dun.map.obj.mjr[y][x] = FACE_MJR_FLOOR;
+			dun.map.obj.mnr[y][x] = FACE_MNR_FLOOR;
+			dun.map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
+
+			make_trap( x, y, dun.lev );
 		}
 	}
 
 	/* ॢƥ */
 
-	item_max_n = roll_dice( DUN_ITEM_AVE_N );
-
-	for( i = 0; i < 10240; i++ ){
-		long	x, y;
+	item_max_n = roll_dice( DUN_ITEM_AVE_N, TRUE );
 
+	for( i = 0; i < LOOP_MAX_10000; i++ ){
 		if( item_n >= item_max_n )
 			break;
 
@@ -459,14 +548,12 @@ void	make_dun( void )
 
 	/* 󥹥 */
 
-	mnstr_max_n = roll_dice( MNSTR_AVE_N );
+	mnstr_max_n = roll_dice( MNSTR_AVE_N, TRUE );
 	if( mnstr_max_n > MNSTR_MAX_N )
 		mnstr_max_n = MNSTR_MAX_N;
 
 	mnstr_n = 0;
-	for( i = 0; i < 4096; i++ ){
-		long	x, y;
-
+	for( i = 0; i < LOOP_MAX_4000; i++ ){
 		if( mnstr_n >= mnstr_max_n )
 			break;
 
@@ -476,20 +563,37 @@ void	make_dun( void )
 			mnstr_n++;
 	}
 
+	/*  NPC */
+
+	npc_max_n = roll_dice( DUN_NPC_AVE_N, FALSE );
+	if( npc_max_n > (MNSTR_MAX_N - mnstr_n) )
+		npc_max_n = (MNSTR_MAX_N - mnstr_n);
+
+	npc_n = 0;
+	for( i = 0; i < LOOP_MAX_4000; i++ ){
+		if( npc_n >= npc_max_n )
+			break;
+
+		x = 1 + randm( MAP_MAX_X - 2 );
+		y = 1 + randm( MAP_MAX_Y - 2 );
+
+		if( make_npc_rand( x, y, 0 ) != NULL )
+			npc_n++;
+	}
+
 	/*  */
 
-	trap_max_n = roll_dice( TRAP_AVE_N );
+	trap_max_n = roll_dice( TRAP_AVE_N, TRUE );
 	if( trap_max_n > TRAP_MAX_N )
 		trap_max_n = TRAP_MAX_N;
 
 	trap_n = 0;
-	for( i = 0; i < 1024; i++ ){
-		long	x, y;
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		trap_t	*trap;
 
 		x = 0;
 		y = 0;
-		for( j = 0; j < 128; j++ ){
+		for( j = 0; j < LOOP_MAX_100; j++ ){
 			x = 1 + randm( MAP_MAX_X - 2 );
 			y = 1 + randm( MAP_MAX_Y - 2 );
 			if( dun.map.obj.mjr[y][x] == FACE_MJR_FLOOR )
@@ -505,9 +609,16 @@ void	make_dun( void )
 
 	/* ȥå */
 
-	for( i = 0; i < AREA_MAP_MAX_Y; i++ )
-		for( j = 0; j < AREA_MAP_MAX_X; j++ )
-			make_trap_room( j, i, dun.lev );
+	for( y = 0; y < AREA_MAP_MAX_Y; y++ )
+		for( x = 0; x < AREA_MAP_MAX_X; x++ )
+			make_trap_room( x, y, dun.lev );
+
+	if( flg_boss ){
+		curs_attrset_chr( get_mbr_randm( MAP_DEL_X, MAP_DEL_Y ),
+				-1, FALSE );
+		print_msg( FLG_NULL, MSG_FEEL_BOSS );
+		curs_attrset_dflt();
+	}
 }
 
 /**/
@@ -553,41 +664,68 @@ void	make_fence( void )
 
 /**/
 
-void	make_area( room_ptn_t *ptn, long x, long y )
+void	make_area( room_ptn_t *ptn, area_t max_n, long x, long y )
 {
 	long	i;
 	long	flg;
 
-	dun.area[y][x] = AREA_PATH;
-
-	if( chk_rate_room() ){
+	if( dun.area[y][x] != AREA_PATH )
 		return;
-	}
 
-	for( i = 0; i < (room_ptn_max_n * 64); i++ ){
-		long	n;
+	if( !chk_rate_room() )
+		return;
 
-		n = randm( room_ptn_max_n );
-		if( abs( dun.lev ) < ptn[n].min_lev )
+	for( i = 0; i < (max_n * LOOP_MAX_100); i++ ){
+		area_t	n;
+
+		n = (area_t)randm( max_n );
+		if( labs( dun.lev ) < ptn[n].min_lev )
 			continue;
-		if( ptn[n].max_lev < abs( dun.lev ) )
+		if( labs( dun.lev ) > ptn[n].max_lev )
 			continue;
-		if( sgn( dun.lev ) != ptn[n].sgn_lev )
+		if( sgn_l( dun.lev ) != ptn[n].sgn_lev )
 			continue;
 
-		flg = (randm( 8 ) >> 1);	/* ľʿե饰 */
-		set_room( ptn, x, y, n, flg );
+		flg = randm( 8 ) / 2;	/* ľʿե饰 */
+		set_room( ptn, n, x, y, flg );
 		break;
 	}
 }
 
 /**/
 
-void	set_room( room_ptn_t *room, long x, long y, long n, long flg )
+bool_t	make_area_boss( room_ptn_t *ptn, area_t max_n, long x, long y )
+{
+	area_t	i;
+	long	flg;
+
+	if( dun.area[y][x] != AREA_PATH )
+		return FALSE;
+
+	for( i = 0; i < max_n; i++ ){
+		if( labs( dun.lev ) < ptn[i].min_lev )
+			continue;
+		if( labs( dun.lev ) > ptn[i].max_lev )
+			continue;
+		if( sgn_l( dun.lev ) != ptn[i].sgn_lev )
+			continue;
+
+		flg = 0;	/* ľʿե饰 */
+		set_room( ptn, i, x, y, flg );
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+/**/
+
+void	set_room( room_ptn_t *room, area_t n, long x, long y, long flg_rev )
 {
 	long	i, j, ii, jj;
 	long	mx, my;
-	char	face;
+	char	mjr, mnr;
+	flg_map_t	flg;
 	sect_t	sect;
 	flg_map_t	flg_light;
 
@@ -596,12 +734,12 @@ void	set_room( room_ptn_t *room, long x, long y, long n, long flg )
 
 	for( i = 0; i < AREA_MAX_Y; i++ ){
 		for( j = 0; j < AREA_MAX_X; j++ ){
-			if( chk_flg( flg, 0x02 ) )
+			if( chk_flg( flg_rev, 0x02 ) )
 				ii = AREA_MAX_Y - 1 - i;
 			else
 				ii = i;
 
-			if( chk_flg( flg, 0x01 ) )
+			if( chk_flg( flg_rev, 0x01 ) )
 				jj = AREA_MAX_X - 1 - j;
 			else
 				jj = j;
@@ -609,24 +747,29 @@ void	set_room( room_ptn_t *room, long x, long y, long n, long flg )
 			mx = x * AREA_MAX_X + jj;
 			my = y * AREA_MAX_Y + ii;
 
-			face = room[n].ptn[i][j];
+			mjr = room[n].ptn[i][j * 2 + 0];
+			mnr = room[n].ptn[i][j * 2 + 1];
 			sect = room[n].sect[i][j];
-			if( face == FACE_MJR_FLOOR ){
-				dun.map.obj.mjr[my][mx] = FACE_MJR_FLOOR;
-				dun.map.obj.mnr[my][mx] = FACE_MNR_FLOOR;
-				dun.map.obj.flg[my][mx] = FLG_MAP_OBJ_PASS;
-				dun.map.sect[my][mx] = sect;
-			} else if( face == FACE_MJR_WALL ){
-				dun.map.obj.mjr[my][mx] = FACE_MJR_WALL;
-				dun.map.obj.mnr[my][mx] = FACE_MNR_WALL;
-				dun.map.obj.flg[my][mx] = FLG_NULL;
-				dun.map.sect[my][mx] = sect;
-			} else {
-				dun.map.obj.mjr[my][mx] = face;
-				dun.map.obj.mnr[my][mx] = FACE_MNR_NULL;
-				dun.map.obj.flg[my][mx] = FLG_MAP_OBJ_PASS;
-				dun.map.sect[my][mx] = sect;
+
+			dun.map.obj.mjr[my][mx] = mjr;
+			dun.map.obj.mnr[my][mx] = mnr;
+			dun.map.obj.flg[my][mx] = FLG_NULL;
+			dun.map.sect[my][mx] = sect;
+
+			switch( mjr ){
+			case FACE_MJR_FLOOR:
+			case FACE_MJR_DOOR_OPEN:
+				flg = FLG_MAP_OBJ_PASS;
+				break;
+			case FACE_MJR_WALL:
+			case FACE_MJR_DOOR_CLOSE:
+				flg = FLG_NULL;
+				break;
+			default:
+				flg = FLG_MAP_OBJ_PASS;
 			}
+			dun.map.obj.flg[my][mx] = flg;
+
 			if( flg_light )
 				dun.map.light_depth_obj[my][mx] = 1;
 			else
@@ -641,6 +784,9 @@ void	set_room( room_ptn_t *room, long x, long y, long n, long flg )
 			mx = x * AREA_MAX_X + j;
 			my = y * AREA_MAX_Y + i;
 			if( dun.map.obj.mjr[my][mx]
+					== FACE_MJR_DOOR_OPEN ){
+				make_door_room( mx, my );
+			} else if( dun.map.obj.mjr[my][mx]
 					== FACE_MJR_DOOR_CLOSE ){
 				make_door_room( mx, my );
 			}
@@ -654,7 +800,7 @@ bool_t	chk_lighted_room( void )
 {
 	rate_t	rate;
 
-	rate = 80 - (randm( abs( dun.lev ) ) * 2);
+	rate = 80 - (randm( labs( dun.lev ) ) * 2);
 
 	if( rate <= 0 )
 		return FALSE;
@@ -667,7 +813,7 @@ bool_t	chk_lighted_path( void )
 {
 	rate_t	rate;
 
-	rate = 40 - (randm( abs( dun.lev ) ) * 2);
+	rate = 40 - (randm( labs( dun.lev ) ) * 2);
 
 	if( rate <= 0 )
 		return FALSE;
@@ -1126,7 +1272,42 @@ void	make_door_room( long mx, long my )
 
 /**/
 
-bool_t	make_door( long bx, long by, long ex, long ey, char mnr )
+door_t	*make_door( long bx, long by, long ex, long ey, char mnr )
+{
+	flg_door_t	flg;
+
+	flg = FLG_NULL;
+
+	if( mnr == FACE_MNR_NULL ){
+		if( chk_rate_door_open() )
+			flg |= FLG_DOOR_OPEN;
+
+		if( chk_rate_door_jammed() )
+			flg |= FLG_DOOR_JAMMED;
+
+		if( chk_rate_door_broken() )
+			flg |= FLG_DOOR_BROKEN;
+
+		if( chk_rate_door_secret() )
+			flg |= FLG_DOOR_SECRET;
+
+		flg &= ~(FLG_DOOR_CHKED);
+	} else if( mnr == FACE_MNR_GATE ){
+		flg |= FLG_DOOR_GATE;
+	} else {
+		flg |= FLG_DOOR_SHOP;
+	}
+
+	return make_door_flg( bx, by, ex, ey, mnr, flg );
+}
+
+/**/
+
+door_t	*make_door_flg(
+	long bx, long by,
+	long ex, long ey,
+	char mnr, flg_door_t flg
+)
 {
 	long	i;
 	long	tmp;
@@ -1134,7 +1315,8 @@ bool_t	make_door( long bx, long by, long ex, long ey, char mnr )
 	long	n;
 
 	if( dun.door_n >= DOOR_MAX_N )
-		return FALSE;
+		return NULL;
+
 	n = dun.door_n;
 
 	if( bx > ex ){
@@ -1147,21 +1329,33 @@ bool_t	make_door( long bx, long by, long ex, long ey, char mnr )
 		by = ey;
 		ey = tmp;
 	}
-	if( bx < 0 )
-		bx = 0;
-	if( ex > MAP_MAX_X - 1 )
-		ex = MAP_MAX_X - 1;
-	if( by < 0 )
-		by = 0;
-	if( ey > MAP_MAX_Y - 1 )
-		ey = MAP_MAX_Y - 1;
+
+	if( mnr == FACE_MNR_GATE ){
+		if( bx < 0 )
+			bx = 0;
+		if( ex > MAP_MAX_X - 1 )
+			ex = MAP_MAX_X - 1;
+		if( by < 0 )
+			by = 0;
+		if( ey > MAP_MAX_Y - 1 )
+			ey = MAP_MAX_Y - 1;
+	} else {
+		if( bx < 1 )
+			bx = 1;
+		if( ex > MAP_MAX_X - 2 )
+			ex = MAP_MAX_X - 2;
+		if( by < 1 )
+			by = 1;
+		if( ey > MAP_MAX_Y - 2 )
+			ey = MAP_MAX_Y - 2;
+	}
 
 	dx = ex - bx + 1;
 	dy = ey - by + 1;
 	if( dx <= 0 )
-		return FALSE;
+		return NULL;
 	if( dy <= 0 )
-		return FALSE;
+		return NULL;
 
 	/* ΰ֤ˤǤ˥ɥʤ return */
 	for( i = 0; i < n; i++ ){
@@ -1174,7 +1368,7 @@ bool_t	make_door( long bx, long by, long ex, long ey, char mnr )
 		if( dun.door[i].dy != dy )
 			continue;
 
-		return FALSE;
+		return NULL;
 	}
 
 	dun.door[n].n = n;
@@ -1182,36 +1376,14 @@ bool_t	make_door( long bx, long by, long ex, long ey, char mnr )
 	dun.door[n].y = by;
 	dun.door[n].dx = dx;
 	dun.door[n].dy = dy;
-
-	dun.door[n].flg = FLG_NULL;
-
-	if( mnr == FACE_MNR_NULL ){
-		if( chk_rate_door_open() )
-			dun.door[n].flg |= FLG_DOOR_OPEN;
-
-		if( chk_rate_door_jammed() )
-			dun.door[n].flg |= FLG_DOOR_JAMMED;
-
-		if( chk_rate_door_broken() )
-			dun.door[n].flg |= FLG_DOOR_BROKEN;
-
-		if( chk_rate_door_secret() )
-			dun.door[n].flg |= FLG_DOOR_SECRET;
-
-		dun.door[n].flg &= ~(FLG_DOOR_SRCHED);
-	} else if( mnr == FACE_MNR_GATE ){
-		dun.door[n].flg |= FLG_DOOR_GATE;
-		dun.door[n].mnr = FACE_MNR_GATE;
-	} else {
-		dun.door[n].flg |= FLG_DOOR_SHOP;
-		dun.door[n].mnr = mnr;
-	}
+	dun.door[n].flg = flg;
+	dun.door[n].mnr = mnr;
 
 	dun.door_n++;
 
 	set_face_door( n );
 
-	return TRUE;
+	return( &(dun.door[n]) );
 }
 
 /**/
@@ -1219,7 +1391,7 @@ bool_t	make_door( long bx, long by, long ex, long ey, char mnr )
 void	set_face_door( long dr_n )
 {
 	long	i, j;
-	long	mjr, mnr;
+	char	mjr, mnr;
 	flg_map_t	flg;
 
 	mjr = FACE_MJR_DOOR_CLOSE;
@@ -1236,7 +1408,7 @@ void	set_face_door( long dr_n )
 	if( chk_flg( dun.door[dr_n].flg, FLG_DOOR_JAMMED ) )
 		mnr = FACE_MNR_DOOR_JAMMED;
 
-	if( chk_flg( dun.door[dr_n].flg, FLG_DOOR_SRCHED ) )
+	if( chk_flg( dun.door[dr_n].flg, FLG_DOOR_CHKED ) )
 		flg |= FLG_MAP_OBJ_FIND;
 	else
 		mnr = FACE_MNR_DOOR_NORM;
@@ -1376,13 +1548,13 @@ void	make_stairs( long x, long y, char face )
 
 	if( (face == FACE_MJR_STAIRS_DOWN)
 			|| (face == FACE_MNR_STAIRS_DOWN) ){
-		if( dun.lev <= DUN_MAX_LEV_BASE )
+		if( labs( dun.lev ) >= labs( DUN_MAX_LEV_BASE ) )
 			return;
 
 		mjr = FACE_MJR_STAIRS_DOWN;
 		mnr = FACE_MNR_STAIRS_DOWN;
 	} else {
-		if( dun.lev >= DUN_MAX_LEV_FLOOR )
+		if( labs( dun.lev ) >= labs( DUN_MAX_LEV_FLOOR ) )
 			return;
 
 		mjr = FACE_MJR_STAIRS_UP;
@@ -1402,10 +1574,10 @@ bool_t	set_statue( long x, long y, long dun_lev, bool_t flg_self )
 	if( dun.map.obj.mjr[y][x] != FACE_MJR_FLOOR )
 		return FALSE;
 	if( !flg_self && (dun.map.chr.mjr[y][x] != FACE_MJR_NULL) )
-			return FALSE;
+		return FALSE;
 
 	dun.map.obj.mjr[y][x] = FACE_MJR_WALL;
-	dun.map.obj.mnr[y][x] = FACE_MNR_WALL;
+	dun.map.obj.mnr[y][x] = FACE_MNR_STATUE;
 	dun.map.obj.flg[y][x] = FLG_NULL;
 
 	ix = x + randm( 3 ) - 1;
@@ -1419,23 +1591,30 @@ bool_t	set_statue( long x, long y, long dun_lev, bool_t flg_self )
 
 void	set_map_total( long x, long y, long dx, long dy )
 {
+	pos_t	*crsr = get_crsr();
+	pos_t	*sub_crsr = get_sub_crsr();
+	crsr_ptn_t	*ptn = get_crsr_ptn();
+	crsr_ptn_t	*sub_ptn = get_sub_crsr_ptn();
+
 	/* ط */
 	set_map_total_bg( x, y, dx, dy );
 
 	/* ° */
-	set_map_total_crsr_attr( x, y, dx, dy );
+	set_map_total_crsr_attr( sub_crsr, sub_ptn, x, y, dx, dy );
+	set_map_total_crsr_attr( crsr, ptn, x, y, dx, dy );
 
 	/* ֥ */
 	set_map_total_obj( x, y, dx, dy );
 
-	/* ۡࡦ */
+	/* 󥫡 */
 	set_map_total_square( x, y, dx, dy );
 
 	/* 󥹥ȥС */
 	set_map_total_chr( x, y, dx, dy );
 
 	/* Υѥ */
-	set_map_total_crsr_ptn( x, y, dx, dy );
+	set_map_total_crsr_ptn( sub_crsr, sub_ptn, x, y, dx, dy );
+	set_map_total_crsr_ptn( crsr, ptn, x, y, dx, dy );
 }
 
 /**/
@@ -1444,7 +1623,7 @@ void	set_map_total_bg( long x, long y, long dx, long dy )
 {
 	long	i, j;
 	long	xx, yy;
-	long	mjr, mnr;
+	char	mjr, mnr;
 	flg_map_t	flg;
 	long	dep;
 	curs_attr_n_t	attr_n;
@@ -1495,36 +1674,42 @@ void	set_map_total_bg( long x, long y, long dx, long dy )
 
 /**/
 
-void	set_map_total_crsr_attr( long x, long y, long dx, long dy )
+void	set_map_total_crsr_attr(
+	pos_t *crsr, crsr_ptn_t *ptn, long x, long y, long dx, long dy
+)
 {
 	long	i, j;
 	long	xx, yy;
 	curs_attr_n_t	attr_n;
 	curs_attr_t	*attr_dflt;
-	pos_t	*csr;
+
+	if( crsr == NULL )
+		return;
+	if( ptn == NULL )
+		return;
+	if( !clip_pos( crsr->x, crsr->y ) )
+		return;
 
 	attr_dflt = get_curs_attr();
-	csr = get_crsr();
-
 	attr_n = CURS_ATTR_N_MAP_CRSR;
-	for( i = 0; i < g_crsr_ptn.y_len; i++ ){
-		yy = csr->y + i + g_crsr_ptn.y;
+	for( i = 0; i < ptn->y_len; i++ ){
+		yy = crsr->y + i + ptn->y;
 		if( !clip_y( yy ) )
 			continue;
 
-		for( j = 0; j < g_crsr_ptn.x_len; j++ ){
+		for( j = 0; j < ptn->x_len; j++ ){
 			long	c;
 
-			xx = csr->x + j + g_crsr_ptn.x;
+			xx = crsr->x + j + ptn->x;
 			if( !clip_x( xx ) )
 				continue;
 
-			c = g_crsr_ptn.face[i][j * 2 + 0];
-			if( c != g_crsr_ptn.mask_chr )
+			c = ptn->face[i][j * 2 + 0];
+			if( c != ptn->mask_chr )
 				dun.map.attr[yy][xx] = attr_dflt[attr_n];
 
-			c = g_crsr_ptn.face[i][j * 2 + 1];
-			if( c != g_crsr_ptn.mask_chr )
+			c = ptn->face[i][j * 2 + 1];
+			if( c != ptn->mask_chr )
 				dun.map.attr[yy][xx] = attr_dflt[attr_n];
 		}
 	}
@@ -1536,7 +1721,7 @@ void	set_map_total_obj( long x, long y, long dx, long dy )
 {
 	long	i, j;
 	long	xx, yy;
-	long	mjr, mnr;
+	char	mjr, mnr;
 	flg_map_t	flg;
 	long	dep;
 	curs_attr_n_t	attr_n;
@@ -1593,6 +1778,9 @@ void	set_map_total_square( long x, long y, long dx, long dy )
 	attr_dflt = get_curs_attr();
 	pty = get_party();
 
+	if( !clip_pos( pty->square.x, pty->square.y ) )
+		return;
+
 	for( i = -1; i <= +1; i++ ){
 		yy = pty->square.y + i;
 		if( !clip_y( yy ) )
@@ -1628,7 +1816,7 @@ void	set_map_total_chr( long x, long y, long dx, long dy )
 {
 	long	i, j;
 	long	xx, yy;
-	long	mjr, mnr;
+	char	mjr, mnr;
 	flg_map_t	flg, flg_total;
 	curs_attr_n_t	attr_n;
 	curs_attr_t	*attr_dflt;
@@ -1698,25 +1886,29 @@ void	set_map_total_chr( long x, long y, long dx, long dy )
 
 /**/
 
-void	set_map_total_crsr_ptn( long x, long y, long dx, long dy )
+void	set_map_total_crsr_ptn(
+	pos_t *crsr, crsr_ptn_t *ptn, long x, long y, long dx, long dy
+)
 {
 	long	i, j;
 	long	xx, yy;
-	pos_t	*csr;
-	crsr_ptn_t	*ptn;
 
-	csr = get_crsr();
-	ptn = get_crsr_ptn();
+	if( crsr == NULL )
+		return;
+	if( ptn == NULL )
+		return;
+	if( !clip_pos( crsr->x, crsr->y ) )
+		return;
 
 	for( i = 0; i < ptn->y_len; i++ ){
-		yy = csr->y + i + ptn->y;
+		yy = crsr->y + i + ptn->y;
 		if( !clip_y( yy ) )
 			continue;
 
 		for( j = 0; j < ptn->x_len; j++ ){
 			long	c;
 
-			xx = csr->x + j + ptn->x;
+			xx = crsr->x + j + ptn->x;
 			if( !clip_x( xx ) )
 				continue;
 
@@ -1726,13 +1918,13 @@ void	set_map_total_crsr_ptn( long x, long y, long dx, long dy )
 			if( c == '\0' )
 				break;
 			if( c != ptn->mask_chr )
-				dun.map.total.mjr[yy][xx] = c;
+				dun.map.total.mjr[yy][xx] = (char)c;
 
 			c = ptn->face[i][j * 2 + 1];
 			if( c == '\0' )
 				break;
 			if( c != ptn->mask_chr )
-				dun.map.total.mnr[yy][xx] = c;
+				dun.map.total.mnr[yy][xx] = (char)c;
 		}
 	}
 }
@@ -1836,11 +2028,6 @@ bool_t	open_door( long n )
 	if( chk_flg( flg, FLG_DOOR_GATE ) )
 		return FALSE;
 
-	if( chk_flg( flg, FLG_DOOR_SHOP ) ){
-		open_door_shop( n );
-		return TRUE;
-	}
-
 	dun.door[n].flg |= FLG_DOOR_OPEN;
 	set_face_door( n );
 
@@ -1853,7 +2040,7 @@ bool_t	open_door( long n )
 
 bool_t	close_door( long n )
 {
-	bool_t	flg;
+	flg_door_t	flg;
 	long	i, j;
 
 	if( n < 0 )
@@ -1961,7 +2148,7 @@ bool_t	srch_door( long n )
 	if( n >= dun.door_n )
 		return FALSE;
 
-	dun.door[n].flg |= FLG_DOOR_SRCHED;
+	dun.door[n].flg |= FLG_DOOR_CHKED;
 	dun.door[n].flg &= ~(FLG_DOOR_SECRET);
 	set_face_door( n );
 
@@ -2234,7 +2421,7 @@ bool_t	chk_find( pos_t *pos1, pos_t *pos2 )
 	}
 
 	dx2 = 2 * (x2 - x1);
-	dy2 = 2 * abs( y2 - y1 );
+	dy2 = 2 * labs( y2 - y1 );
 	if( y1 <= y2 )
 		sgn_y = +1;
 	else
@@ -2321,17 +2508,17 @@ door_t	*get_door_nearest( long x, long y, act_kind_t act_kind )
 			continue;
 
 		if( p->x >= x ){
-			dx = abs( p->x - x );
+			dx = labs( p->x - x );
 			pos1.x = p->x;
 		} else {
-			dx = abs( x - (p->x + p->dx - 1) );
+			dx = labs( x - (p->x + p->dx - 1) );
 			pos1.x = p->x + p->dx - 1;
 		}
 		if( p->y >= y ){
-			dy = abs( p->y - y );
+			dy = labs( p->y - y );
 			pos1.y = p->y;
 		} else {
-			dy = abs( y - (p->y + p->dy - 1) );
+			dy = labs( y - (p->y + p->dy - 1) );
 			pos1.y = p->y + p->dy - 1;
 		}
 		tmp_r = dx + dy;
@@ -2383,17 +2570,17 @@ door_t	*get_door_near( long x, long y, act_kind_t act_kind )
 			continue;
 
 		if( p->x >= x ){
-			dx = abs( p->x - x );
+			dx = labs( p->x - x );
 			pos1.x = p->x;
 		} else {
-			dx = abs( x - (p->x + p->dx - 1) );
+			dx = labs( x - (p->x + p->dx - 1) );
 			pos1.x = p->x + p->dx - 1;
 		}
 		if( p->y >= y ){
-			dy = abs( p->y - y );
+			dy = labs( p->y - y );
 			pos1.y = p->y;
 		} else {
-			dy = abs( y - (p->y + p->dy - 1) );
+			dy = labs( y - (p->y + p->dy - 1) );
 			pos1.y = p->y + p->dy - 1;
 		}
 		tmp_r = dx + dy;
@@ -2449,17 +2636,17 @@ door_t	*get_door_disperse( mbr_t *mbr, act_kind_t act_kind )
 			continue;
 
 		if( p->x >= x ){
-			dx = abs( p->x - x );
+			dx = labs( p->x - x );
 			pos1.x = p->x;
 		} else {
-			dx = abs( x - (p->x + p->dx - 1) );
+			dx = labs( x - (p->x + p->dx - 1) );
 			pos1.x = p->x + p->dx - 1;
 		}
 		if( p->y >= y ){
-			dy = abs( p->y - y );
+			dy = labs( p->y - y );
 			pos1.y = p->y;
 		} else {
-			dy = abs( y - (p->y + p->dy - 1) );
+			dy = labs( y - (p->y + p->dy - 1) );
 			pos1.y = p->y + p->dy - 1;
 		}
 		tmp_r = dx + dy;
@@ -2511,17 +2698,17 @@ door_t	*get_door_randm( long x, long y, act_kind_t act_kind )
 			continue;
 
 		if( p->x >= x ){
-			dx = abs( p->x - x );
+			dx = labs( p->x - x );
 			pos1.x = p->x;
 		} else {
-			dx = abs( x - (p->x + p->dx - 1) );
+			dx = labs( x - (p->x + p->dx - 1) );
 			pos1.x = p->x + p->dx - 1;
 		}
 		if( p->y >= y ){
-			dy = abs( p->y - y );
+			dy = labs( p->y - y );
 			pos1.y = p->y;
 		} else {
-			dy = abs( y - (p->y + p->dy - 1) );
+			dy = labs( y - (p->y + p->dy - 1) );
 			pos1.y = p->y + p->dy - 1;
 		}
 
@@ -2592,13 +2779,11 @@ bool_t	chk_mark_door( door_t *p, act_kind_t act_kind )
 			return FALSE;
 		break;
 	case ACT_KIND_DOOR_JAM:
-		if( !chk_flg( p->flg, FLG_DOOR_SRCHED ) )
-			break;
 		if( chk_flg( p->flg, FLG_DOOR_JAMMED ) )
 			return FALSE;
 		break;
 	case ACT_KIND_DOOR_DISARM:
-		if( !chk_flg( p->flg, FLG_DOOR_SRCHED ) )
+		if( !chk_flg( p->flg, FLG_DOOR_CHKED ) )
 			break;
 		if( !chk_flg( p->flg, FLG_DOOR_JAMMED ) )
 			return FALSE;
@@ -2611,8 +2796,8 @@ bool_t	chk_mark_door( door_t *p, act_kind_t act_kind )
 		if( chk_flg( p->flg, FLG_DOOR_OPEN ) )
 			return FALSE;
 		break;
-	case ACT_KIND_DOOR_SRCH:
-		if( chk_flg( p->flg, FLG_DOOR_SRCHED ) )
+	case ACT_KIND_DOOR_CHK:
+		if( chk_flg( p->flg, FLG_DOOR_CHKED ) )
 			return FALSE;
 		break;
 	default:
@@ -2711,6 +2896,18 @@ bool_t	is_map_wall( long x, long y )
 
 /**/
 
+bool_t	clip_pos( long x, long y )
+{
+	if( !clip_x( x ) )
+		return FALSE;
+	if( !clip_y( y ) )
+		return FALSE;
+
+	return TRUE;
+}
+
+/**/
+
 bool_t	clip_x( long n )
 {
 	if( n == MAP_DEL_X )
@@ -2739,11 +2936,47 @@ bool_t	clip_y( long n )
 
 /**/
 
-bool_t	clip_pos( long x, long y )
+bool_t	clip_draw_pos( long x, long y )
 {
-	if( !clip_x( x ) )
+	if( !clip_draw_x( x ) )
 		return FALSE;
-	if( !clip_y( y ) )
+	if( !clip_draw_y( y ) )
+		return FALSE;
+
+	return TRUE;
+}
+
+/**/
+
+bool_t	clip_draw_x( long n )
+{
+	pos_t	*draw;
+
+	if( n == MAP_DEL_X )
+		return FALSE;
+
+	draw = get_map_draw_pos();
+	if( n < draw->x )
+		return FALSE;
+	if( n > (draw->x + WIN_MAP_WIDTH - 1) )
+		return FALSE;
+
+	return TRUE;
+}
+
+/**/
+
+bool_t	clip_draw_y( long n )
+{
+	pos_t	*draw;
+
+	if( n == MAP_DEL_Y )
+		return FALSE;
+
+	draw = get_map_draw_pos();
+	if( n < draw->y )
+		return FALSE;
+	if( n > (draw->y + WIN_MAP_HEIGHT - 1) )
 		return FALSE;
 
 	return TRUE;
@@ -2781,7 +3014,7 @@ bool_t	chk_rate_door_broken( void )
 
 bool_t	chk_rate_door_secret( void )
 {
-	return rate_randm( abs( dun.lev ) * 4);
+	return rate_randm( labs( dun.lev ) * 4);
 }
 
 /**/
