@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,19 +25,25 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: dun.h,v 1.12 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: dun.h,v 1.25 2004/07/24 07:32:40 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	DUN_H
 #define	DUN_H
 
+/* µ */
+
 #include	"gmain.h"
 #include	"curs.h"
 
-#define	DUN_MAX_LEV_FLOOR	1000
-#define	DUN_MAX_LEV_BASE	1000
+/**/
 
-#define	STAIRS_MAX_R	2
+#define	DUN_MAX_LEV_FLOOR	1000	/* Ͼκ糬 */
+#define	DUN_MAX_LEV_BASE	1000	/* ϲκ糬 */
+
+#define	STAIRS_MAX_R	2	/* ꡢꤽ줾γʤο */
+
+/* ޥåפι */
 
 #define	AREA_MAX_X	20
 #define	AREA_MAX_Y	20
@@ -49,106 +54,189 @@
 #define	MAP_DEL_X	-1
 #define	MAP_DEL_Y	-1
 
+/* ǥեȤΥ */
+
 #define	SECT_NULL	'a'
 
+/* ե饰 */
+
 typedef unsigned long	flg_map_t;
 
-#define	FLG_MAP_OBJ_FIND	(1 << 0)
-#define	FLG_MAP_OBJ_PASS	(1 << 1)
-#define	FLG_MAP_OBJ_LOOK_FLOOR	(1 << 4)
-#define	FLG_MAP_OBJ_LOOK_WALL	(1 << 5)
+/* ֥ȤΥե饰 */
 
-#define	FLG_MAP_CHR_FIND	(1 << 0)
-#define	FLG_MAP_CHR_INVISIBLE	(1 << 1)
-#define	FLG_MAP_CHR_NPC	(1 << 2)
+#define	FLG_MAP_OBJ_FIND	(1 << 0)	/* ȯ */
+#define	FLG_MAP_OBJ_PASS	(1 << 1)	/* ̹Բ */
+#define	FLG_MAP_OBJ_LOOK_FLOOR	(1 << 4)	/* ˸ */
+#define	FLG_MAP_OBJ_LOOK_WALL	(1 << 5)	/* ɤ˸ */
 
-#define	FLG_TOTAL_REV	(1 << 0)
-#define	FLG_TOTAL_DRAW	(1 << 1)
+/* 饯Υե饰 */
 
-#define	FLG_DOOR_OPEN	(1 << 0)
-#define	FLG_DOOR_JAMMED	(1 << 1)
-#define	FLG_DOOR_BROKEN	(1 << 2)
-#define	FLG_DOOR_SECRET	(1 << 3)
-#define	FLG_DOOR_SRCHED	(1 << 4)
-#define	FLG_DOOR_GATE	(1 << 8)
-#define	FLG_DOOR_SHOP	(1 << 9)
+#define	FLG_MAP_CHR_FIND	(1 << 0)	/* ȯ */
+#define	FLG_MAP_CHR_INVISIBLE	(1 << 1)	/* Ʃ */
+#define	FLG_MAP_CHR_NPC	(1 << 2)	/* NPC */
+
+/* ɥΥե饰 */
+
+#define	FLG_DOOR_OPEN	(1 << 0)	/* Ƥ */
+#define	FLG_DOOR_JAMMED	(1 << 1)	/* ꤵƤ */
+#define	FLG_DOOR_BROKEN	(1 << 2)	/* Ƥ */
+#define	FLG_DOOR_SECRET	(1 << 3)	/* Ƥ */
+#define	FLG_DOOR_CHKED	(1 << 4)	/* Ĵٺ */
+#define	FLG_DOOR_GATE	(1 << 8)	/*  */
+#define	FLG_DOOR_SHOP	(1 << 9)	/* Źν */
+
+/*  */
 
 #define	FACE_MJR_WALL	'#'
 #define	FACE_MNR_WALL	'#'
+#define	FACE_MNR_STATUE	'@'
+#define	FACE_MNR_STREETLAMP	'*'
+
 #define	FACE_MJR_FLOOR	'.'
 #define	FACE_MNR_FLOOR	' '
+
 #define	FACE_MJR_ITEM	'$'
+
 #define	FACE_MJR_DOOR_CLOSE	'+'
 #define	FACE_MJR_DOOR_OPEN	'\''
 #define	FACE_MNR_DOOR_NORM	' '
 #define	FACE_MNR_DOOR_JAMMED	'^'
 #define	FACE_MNR_DOOR_BROKEN	'*'
 #define	FACE_MNR_DOOR_SECRET	'#'
+#define	FACE_MNR_GATE	'+'
+#define	FACE_MNR_SHOP_INN	'i'
+#define	FACE_MNR_SHOP_BAR	'b'
+#define	FACE_MNR_SHOP_WPN	'w'
+#define	FACE_MNR_SHOP_ARMOR	'a'
+#define	FACE_MNR_SHOP_MAGIC	'm'
+#define	FACE_MNR_SHOP_TEMPLE	't'
+#define	FACE_MNR_SHOP_ALCHEMY	'l'
+#define	FACE_MNR_SHOP_MUSIC	'u'
+#define	FACE_MNR_SHOP_GROCERY	'g'
+#define	FACE_MNR_SHOP_RESTAURANT	'r'
+#define	FACE_MNR_SHOP_TEAROOM	'c'
+#define	FACE_MNR_SHOP_TOBACCO	's'
+#define	FACE_MNR_SHOP_PET_SHOP	'p'
+
 #define	FACE_MJR_STAIRS_UP	'<'
-#define	FACE_MNR_STAIRS_UP	'<'
+#define	FACE_MNR_STAIRS_UP	'u'
 #define	FACE_MJR_STAIRS_DOWN	'>'
-#define	FACE_MNR_STAIRS_DOWN	'>'
+#define	FACE_MNR_STAIRS_DOWN	'd'
+
 #define	FACE_MJR_TRAP	'^'
+#define	FACE_MNR_TRAP_SLEEP	's'
+#define	FACE_MNR_TRAP_ARW	'a'
+#define	FACE_MNR_TRAP_POISON_DARTS	'd'
+#define	FACE_MNR_TRAP_BOMB	'b'
+#define	FACE_MNR_TRAP_TELEPORT_PARTY	'l'
+#define	FACE_MNR_TRAP_TELEPORT	't'
+#define	FACE_MNR_TRAP_PIT	'p'
+#define	FACE_MNR_TRAP_CHUTE	'c'
+
 #define	FACE_MJR_MBR	'@'
+
 #define	FACE_MJR_SQUARE	'%'
 
+/* ɥ */
+
 #define	DOOR_MAX_N	64
 
+/* 롦ѥ */
+
 #define	CRSR_PTN_MAX_N	8
 #define	CRSR_PTN_MAX_Y_LEN	16
 #define	CRSR_PTN_MAX_X_LEN	16
 
+/*  */
+
 typedef char	sect_t;
 
+/* Υѥ */
+
 typedef struct {
+	/*  */
 	long	min_lev, max_lev, sgn_lev;
+	/* иΨ */
 	rate_t	rate;
-	char	ptn[AREA_MAX_Y][AREA_MAX_X];
+	/* ѥ */
+	char	ptn[AREA_MAX_Y][AREA_MAX_X * 2];
+	/*  */
 	sect_t	sect[AREA_MAX_Y][AREA_MAX_X];
 } room_ptn_t;
 
+/* ɥ */
+
 typedef unsigned long	flg_door_t;
 
 typedef struct {
-	long	n;
-	long	x, y;
-	long	dx, dy;
-	char	mjr, mnr;
-	flg_door_t	flg;
+	long	n;	/* ɥֹ */
+	long	x, y;	/*  */
+	long	dx, dy;	/*  */
+	char	mjr, mnr;	/*  */
+	flg_door_t	flg;	/* ե饰 */
 } door_t;
 
+/* ޥåפΥ쥤䡼 */
+
 typedef struct {
 	char	mjr[MAP_MAX_Y][MAP_MAX_X];	/* major face */
 	char	mnr[MAP_MAX_Y][MAP_MAX_X];	/* minor face */
 	flg_map_t	flg[MAP_MAX_Y][MAP_MAX_X];
 } map_t;
 
+/* ޥåפΥ쥤䡼 */
+
 typedef struct {
+	/* ֥ȤΥ쥤䡼 */
 	map_t	obj;
+	/* 饯Υ쥤䡼 */
 	map_t	chr;
+	/* 쥤䡼 */
 	map_t	total;
+	/*  */
 	sect_t	sect[MAP_MAX_Y][MAP_MAX_X];
+
+	/* ٥ */
 	long	light_depth_obj[MAP_MAX_Y][MAP_MAX_X];
+	/* 饯ˤ٥ */
 	long	light_depth_chr[MAP_MAX_Y][MAP_MAX_X];
+
+	/* ɽ° */
 	curs_attr_t	attr[MAP_MAX_Y][MAP_MAX_X];
 } all_map_t;
 
+/* ꥢ */
+
 typedef short	area_t;
 
+/* µ */
+
 typedef struct {
+	/* ޥå */
 	all_map_t	map;
-	area_t	area[AREA_MAP_MAX_Y][AREA_MAP_MAX_X];;
+	/* ꥢ */
+	area_t	area[AREA_MAP_MAX_Y][AREA_MAP_MAX_X];
+
+	/* ɥ */
 	door_t	door[DOOR_MAX_N];
+	/* ɥο */
 	long	door_n;
+
+	/* ߤγ */
 	long	lev;
 } dun_t;
 
 typedef struct {
+	/*  */
 	long	x, y;
+	/* ȹ⤵ */
 	long	x_len, y_len;
+
+	/* ޥ˻Ȥʸ */
 	long	mask_chr;
+
+	/* ѥ */
 	char	face[CRSR_PTN_MAX_Y_LEN][CRSR_PTN_MAX_X_LEN * 2 + 1];
 } crsr_ptn_t;
 
 #endif	/* DUN_H */
-
