@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: fight.c,v 1.49 2003/11/20 07:37:24 bowkenken Exp $
+* $Id: fight.c,v 1.94 2004/07/27 23:40:22 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -37,7 +36,7 @@
 #include	"item.h"
 #include	"spell.h"
 /*#include	"chr.h"*/
-/*#include	"party.h"*/
+#include	"party.h"
 /*#include	"mnstr.h"*/
 /*#include	"pet.h"*/
 #include	"fight.h"
@@ -46,29 +45,33 @@
 #include	"draw.h"
 /*#include	"curs.h"*/
 /*#include	"menu.h"*/
+/*#include	"amenu.h"*/
+/*#include	"request.h"*/
+/*#include	"gfile.h"*/
 /*#include	"msg.h"*/
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-/*#include	"turn_prot.h"*/
-#include	"dun_prot.h"
-/*#include	"town_prot.h"*/
-#include	"item_prot.h"
-/*#include	"spell_prot.h"*/
-#include	"chr_prot.h"
-/*#include	"party_prot.h"*/
-#include	"mnstr_prot.h"
-/*#include	"pet_prot.h"*/
-#include	"fight_prot.h"
-#include	"fx_prot.h"
-#include	"trap_prot.h"
-#include	"draw_prot.h"
-#include	"curs_prot.h"
-/*#include	"menu_prot.h"*/
-/*#include	"tmenu_prot.h"*/
-/*#include	"amenu_prot.h"*/
-/*#include	"gfile_prot.h"*/
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+/*#include	"turn-prot.h"*/
+#include	"dun-prot.h"
+/*#include	"town-prot.h"*/
+#include	"item-prot.h"
+/*#include	"spell-prot.h"*/
+#include	"chr-prot.h"
+#include	"party-prot.h"
+#include	"mnstr-prot.h"
+/*#include	"pet-prot.h"*/
+#include	"fight-prot.h"
+#include	"fx-prot.h"
+#include	"trap-prot.h"
+#include	"draw-prot.h"
+#include	"curs-prot.h"
+/*#include	"menu-prot.h"*/
+/*#include	"tmenu-prot.h"*/
+/*#include	"amenu-prot.h"*/
+/*#include	"request-prot.h"*/
+/*#include	"gfile-prot.h"*/
+/*#include	"msg-prot.h"*/
 
 /**/
 
@@ -88,8 +91,8 @@
 
 #define	HIT_FIT_AVE_N	10
 #define	HIT_HNT_AVE_N	10
-#define	WPN_CRTCL_FIT_AVE_N	10
-#define	WPN_CRTCL_HNT_AVE_N	10
+#define	WPN_CRTCL_FIT_AVE_N	20
+#define	WPN_CRTCL_HNT_AVE_N	20
 #define	DAM_FIT_AVE_N	10
 #define	DAM_HNT_AVE_N	10
 
@@ -100,25 +103,30 @@
 #define	DEF_MON_MUL	10
 #define	DEF_MON_ADD	0
 
+#define	WPN_CRTCL_THF_LEV_RATE	30
+#define	WPN_CRTCL_THF_MAX_RATE	30
+
 #define	DEF_MIN_RATE	30
 #define	DEF_AVE_N	10
-#define	ARMOR_CRTCL_AVE_N	5
+#define	ARMOR_CRTCL_AVE_N	10
 #define	AC_AVE_N	20
 
 /* ɸ(⤷Ƥʤ)ɸϥåΨ */
 #define	DEF_DEF_RATE	150
 #define	DEF_AC_RATE	150
 
-#define	DAM_WAND_MUL	2
-#define	DAM_WAND_ADD	10
+#define	DAM_WAND_RATE	((rate_t)120)
+#define	DAM_WAND_ADD	2
 
-#define	DAM_SPELL_MUL	2
-#define	DAM_SPELL_ADD	10
+#define	DAM_SPELL_RATE	((rate_t)150)
+#define	DAM_SPELL_ADD	2
 
 #define	DOOR_DEF	12
 #define	TRAP_DEF	8
 #define	ITEM_DEF	6
 
+#define	RATE_PER_RANGE	((rate_t)200)
+
 #define	TRAINING_EXP	1
 
 #define	ANIM_ARW_LEN	(ANIM_FRAME_N / 4)
@@ -127,33 +135,54 @@
 /**/
 
 static attack_t	mbr_attack_bash[] = {
-	{ ATTACK_KIND_PUNCH, 1, FALSE, TRUE, 10, 10, 10, RESI_KIND_KNOC,
+	{ ATTACK_KIND_PUNCH, 1, FALSE, TRUE,
+		WPN_HIT_UNARMED, WPN_CRTCL_UNARMED, WPN_DAM_UNARMED,
+		RESI_KIND_KNOC, 1, 1,
 		FX_KIND_NULL, 0,
-		SPELL_KIND_NULL, EXTENT_KIND_NULL, N_MSG_NULL, },
-	{ ATTACK_KIND_KICK, 1, FALSE, TRUE, 10, 10, 10, RESI_KIND_KNOC,
+		SPELL_KIND_NULL,
+		EXTENT_KIND_NULL, N_MSG_NULL, },
+	{ ATTACK_KIND_KICK, 1, FALSE, TRUE,
+		WPN_HIT_UNARMED, WPN_CRTCL_UNARMED, WPN_DAM_UNARMED,
+		RESI_KIND_KNOC, 1, 1,
 		FX_KIND_NULL, 0,
-		SPELL_KIND_NULL, EXTENT_KIND_NULL, N_MSG_NULL, },
-	{ ATTACK_KIND_NULL, 0, FALSE, FALSE, 0, 0, 0, RESI_KIND_MAX_N,
+		SPELL_KIND_NULL,
+		EXTENT_KIND_NULL, N_MSG_NULL, },
+	{ ATTACK_KIND_NULL, 0, FALSE, FALSE,
+		0, 0, 0,
+		RESI_KIND_MAX_N, 1, 1,
 		FX_KIND_NULL, 0,
-		SPELL_KIND_NULL, EXTENT_KIND_NULL, N_MSG_NULL, },
+		SPELL_KIND_NULL,
+		EXTENT_KIND_NULL, N_MSG_NULL, },
 };
 
 static attack_t	mbr_attack_throw[] = {
-	{ ATTACK_KIND_THROW, 1, TRUE, FALSE, 10, 10, 10, RESI_KIND_KNOC,
+	{ ATTACK_KIND_THROW, 1, TRUE, FALSE,
+		WPN_HIT_UNARMED, WPN_CRTCL_UNARMED, WPN_DAM_UNARMED,
+		RESI_KIND_KNOC, 1, 10,
 		FX_KIND_NULL, 0,
-		SPELL_KIND_NULL, EXTENT_KIND_NULL, N_MSG_NULL, },
-	{ ATTACK_KIND_NULL, 0, FALSE, FALSE, 0, 0, 0, RESI_KIND_MAX_N,
+		SPELL_KIND_NULL,
+		EXTENT_KIND_NULL, N_MSG_NULL, },
+	{ ATTACK_KIND_NULL, 0, FALSE, FALSE,
+		0, 0, 0,
+		RESI_KIND_MAX_N, 1, 1,
 		FX_KIND_NULL, 0,
-		SPELL_KIND_NULL, EXTENT_KIND_NULL, N_MSG_NULL, },
+		SPELL_KIND_NULL,
+		EXTENT_KIND_NULL, N_MSG_NULL, },
 };
 
 static attack_t	mbr_attack_throw_light[] = {
-	{ ATTACK_KIND_THROW, 1, TRUE, FALSE, 10, 10, 10, RESI_KIND_HEAT,
+	{ ATTACK_KIND_THROW, 1, TRUE, FALSE,
+		WPN_HIT_UNARMED, WPN_CRTCL_UNARMED, WPN_DAM_UNARMED,
+		RESI_KIND_HEAT, 1, 10,
 		FX_KIND_NULL, 0,
-		SPELL_KIND_NULL, EXTENT_KIND_NULL, N_MSG_NULL, },
-	{ ATTACK_KIND_NULL, 0, FALSE, FALSE, 0, 0, 0, RESI_KIND_MAX_N,
+		SPELL_KIND_NULL,
+		EXTENT_KIND_NULL, N_MSG_NULL, },
+	{ ATTACK_KIND_NULL, 0, FALSE, FALSE,
+		0, 0, 0,
+		RESI_KIND_MAX_N, 1, 1,
 		FX_KIND_NULL, 0,
-		SPELL_KIND_NULL, EXTENT_KIND_NULL, N_MSG_NULL, },
+		SPELL_KIND_NULL,
+		EXTENT_KIND_NULL, N_MSG_NULL, },
 };
 
 /**/
@@ -161,30 +190,35 @@ static attack_t	mbr_attack_throw_light[] = {
 bool_t	bash( chr_t *chr, chr_t *chr2, attack_t *dflt_attack )
 {
 	attack_t	attack;
+	long	d;
 
 	if( chr == NULL )
 		return FALSE;
 	if( chr2 == NULL )
 		return FALSE;
 
+	d = get_trgt_range( chr );
+
 	if( dflt_attack == NULL ){
-		if( !set_attack_chr_bash( &attack, chr, FALSE ) )
+		if( !set_attack_chr_bash( chr, &attack, 1, FALSE ) )
 			return FALSE;
 	} else {
 		attack = *dflt_attack;
 	}
 
-	return fight( chr, chr2, &attack );
+	return fight( chr, chr2, &attack, d );
 }
 
 /**/
 
-bool_t	set_attack_chr_bash( attack_t *attack, chr_t *chr, bool_t flg_ave )
+bool_t	set_attack_chr_bash(
+	chr_t *chr, attack_t *attack, long range, bool_t flg_ave
+)
 {
 	if( chr->kind == CHR_KIND_MBR )
-		return set_attack_mbr_bash( attack, chr, flg_ave );
+		return set_attack_mbr_bash( chr, attack, range, flg_ave );
 	if( chr->kind == CHR_KIND_MNSTR )
-		return set_attack_mnstr_bash( attack, chr, flg_ave );
+		return set_attack_mnstr_bash( chr, attack, range, flg_ave );
 
 	return FALSE;
 }
@@ -192,7 +226,7 @@ bool_t	set_attack_chr_bash( attack_t *attack, chr_t *chr, bool_t flg_ave )
 /**/
 
 bool_t	set_attack_mbr_bash(
-	attack_t *attack, mbr_t *mbr, bool_t flg_ave
+	mbr_t *mbr, attack_t *attack, long range, bool_t flg_ave
 )
 {
 	item_t	*eq;
@@ -219,14 +253,14 @@ bool_t	set_attack_mbr_bash(
 
 	if( wpn_tab == NULL ){
 		/* Ǽξ */
-		p = get_attack( flg_ave, mbr_attack_bash,
+		p = get_attack( flg_ave, mbr_attack_bash, range,
 				FALSE, TRUE, FALSE, FALSE );
 		if( p == NULL )
 			return FALSE;
 		*attack = *p;
 	} else {
 		/* ξ */
-		p = get_attack( flg_ave, wpn_tab->attack,
+		p = get_attack( flg_ave, wpn_tab->attack, range,
 				TRUE, FALSE, FALSE, FALSE );
 		if( p == NULL )
 			return FALSE;
@@ -243,7 +277,7 @@ bool_t	set_attack_mbr_bash(
 /**/
 
 bool_t	set_attack_mnstr_bash(
-	attack_t *attack, mnstr_t *mnstr, bool_t flg_ave
+	mnstr_t *mnstr, attack_t *attack, long range, bool_t flg_ave
 )
 {
 	attack_t	*p;
@@ -258,7 +292,7 @@ bool_t	set_attack_mnstr_bash(
 	if( mnstr->mnstr_tab == NULL )
 		return FALSE;
 
-	p = get_attack( flg_ave, mnstr->mnstr_tab->attack,
+	p = get_attack( flg_ave, mnstr->mnstr_tab->attack, range,
 			TRUE, TRUE, FALSE, FALSE );
 	if( p == NULL )
 		return FALSE;
@@ -274,6 +308,7 @@ bool_t	throw( chr_t *chr1, chr_t *chr2, attack_t *dflt_attack )
 {
 	item_t	*eq;
 	attack_t	attack;
+	long	d;
 
 	if( chr1 == NULL )
 		return FALSE;
@@ -292,24 +327,28 @@ bool_t	throw( chr_t *chr1, chr_t *chr2, attack_t *dflt_attack )
 		return TRUE;
 	}
 
+	d = get_trgt_range( chr1 );
+
 	if( dflt_attack == NULL ){
-		if( !set_attack_chr_throw( &attack, chr1, FALSE ) )
+		if( !set_attack_chr_throw( chr1, &attack, d, FALSE ) )
 			return FALSE;
 	} else {
 		attack = *dflt_attack;
 	}
 
-	return fight( chr1, chr2, &attack );
+	return fight( chr1, chr2, &attack, d );
 }
 
 /**/
 
-bool_t	set_attack_chr_throw( attack_t *attack, chr_t *chr, bool_t flg_ave )
+bool_t	set_attack_chr_throw(
+	chr_t *chr, attack_t *attack, long range, bool_t flg_ave
+)
 {
 	if( chr->kind == CHR_KIND_MBR )
-		return set_attack_mbr_throw( attack, chr, flg_ave );
+		return set_attack_mbr_throw( chr, attack, range, flg_ave );
 	if( chr->kind == CHR_KIND_MNSTR )
-		return set_attack_mnstr_throw( attack, chr, flg_ave );
+		return set_attack_mnstr_throw( chr, attack, range, flg_ave );
 
 	return FALSE;
 }
@@ -317,19 +356,19 @@ bool_t	set_attack_chr_throw( attack_t *attack, chr_t *chr, bool_t flg_ave )
 /**/
 
 bool_t	set_attack_mbr_throw(
-	attack_t *attack, mbr_t *mbr, bool_t flg_ave
+	mbr_t *mbr, attack_t *attack, long range, bool_t flg_ave
 )
 {
 	item_t	*eq;
 	wpn_tab_t	*bow_tab, *arw_tab;
 	attack_t	*p;
 
-	if( attack == NULL )
-		return FALSE;
 	if( mbr == NULL )
 		return FALSE;
 	if( mbr->kind != CHR_KIND_MBR )
 		return FALSE;
+	if( attack == NULL )
+		return FALSE;
 
 	eq = get_mbr_item_asgn_equip( mbr );
 	if( eq == NULL )
@@ -348,7 +387,8 @@ bool_t	set_attack_mbr_throw(
 
 	if( chk_dagger( arw_tab ) ){
 		/* ξ */
-		p = get_attack( flg_ave, arw_tab->attack,
+
+		p = get_attack( flg_ave, arw_tab->attack, range,
 				FALSE, FALSE, TRUE, FALSE );
 		if( p == NULL )
 			return FALSE;
@@ -365,7 +405,8 @@ bool_t	set_attack_mbr_throw(
 		/* ݤʤɤξ */
 
 		/* bow */
-		p = get_attack( flg_ave, bow_tab->attack,
+
+		p = get_attack( flg_ave, bow_tab->attack, range,
 				FALSE, FALSE, TRUE, FALSE );
 		if( p == NULL )
 			return FALSE;
@@ -379,7 +420,8 @@ bool_t	set_attack_mbr_throw(
 				.dat.wpn.add_dam );
 
 		/* arrow */
-		p = get_attack( flg_ave, arw_tab->attack,
+
+		p = get_attack( flg_ave, arw_tab->attack, range,
 				FALSE, FALSE, TRUE, FALSE );
 		if( p == NULL )
 			return FALSE;
@@ -393,17 +435,21 @@ bool_t	set_attack_mbr_throw(
 				.dat.wpn.add_dam );
 
 		/* sum */
+
 		*attack = attack_bow;
 		attack->hit += attack_arw.hit;
 		attack->crtcl += attack_arw.crtcl;
 		attack->dam += attack_arw.dam;
 	} else {
 		/* ¾Υƥξ */
+
 		if( eq[EQUIP_KIND_ARW].kind == ITEM_KIND_LIGHT ){
-			p = get_attack( flg_ave, mbr_attack_throw_light,
+			p = get_attack( flg_ave,
+					mbr_attack_throw_light, range,
 					FALSE, FALSE, TRUE, FALSE );
 		} else {
-			p = get_attack( flg_ave, mbr_attack_throw,
+			p = get_attack( flg_ave,
+					mbr_attack_throw, range,
 					FALSE, FALSE, TRUE, FALSE );
 		}
 		if( p == NULL )
@@ -417,22 +463,21 @@ bool_t	set_attack_mbr_throw(
 /**/
 
 bool_t	set_attack_mnstr_throw(
-	attack_t *attack, mnstr_t *mnstr, bool_t flg_ave
+	mnstr_t *mnstr, attack_t *attack, long range, bool_t flg_ave
 )
 {
 	attack_t	*p;
 
-	if( attack == NULL )
-		return FALSE;
 	if( mnstr == NULL )
 		return FALSE;
 	if( mnstr->kind != CHR_KIND_MNSTR )
 		return FALSE;
-
 	if( mnstr->mnstr_tab == NULL )
 		return FALSE;
+	if( attack == NULL )
+		return FALSE;
 
-	p = get_attack( flg_ave, mnstr->mnstr_tab->attack,
+	p = get_attack( flg_ave, mnstr->mnstr_tab->attack, range,
 			FALSE, FALSE, TRUE, FALSE );
 	if( p == NULL )
 		return FALSE;
@@ -444,50 +489,18 @@ bool_t	set_attack_mnstr_throw(
 
 /**/
 
-bool_t	chk_dagger( const wpn_tab_t *arw_tab )
-{
-	if( arw_tab == NULL )
-		return FALSE;
-	if( arw_tab->bow_kind != BOW_KIND_DAGGER )
-		return FALSE;
-	if( !chk_flg( arw_tab->flg, FLG_WPN_TAB_ARW ) )
-		return FALSE;
-
-	return TRUE;
-}
-
-/**/
-
-bool_t	chk_bow( const wpn_tab_t *arw_tab, const wpn_tab_t *bow_tab )
-{
-	if( arw_tab == NULL )
-		return FALSE;
-	if( bow_tab == NULL )
-		return FALSE;
-	if( arw_tab->bow_kind != bow_tab->bow_kind )
-		return FALSE;
-	if( !chk_flg( bow_tab->flg, FLG_WPN_TAB_BOW ) )
-		return FALSE;
-	if( !chk_flg( arw_tab->flg, FLG_WPN_TAB_ARW ) )
-		return FALSE;
-
-	return TRUE;
-}
-
-/**/
-
 attack_t	*get_attack(
-	bool_t flg_ave, attack_t *ls,
+	bool_t flg_ave, attack_t *ls, long range,
 	bool_t flg_bash, bool_t flg_monk,
 	bool_t flg_throw, bool_t flg_spell
 )
 {
 	if( flg_ave ){
-		return( get_attack_ave( ls,
+		return( get_attack_ave( ls, range,
 				flg_bash, flg_monk,
 				flg_throw, flg_spell ) );
 	} else {
-		return( get_attack_randm( ls,
+		return( get_attack_randm( ls, range,
 				flg_bash, flg_monk,
 				flg_throw, flg_spell ) );
 	}
@@ -496,13 +509,13 @@ attack_t	*get_attack(
 /**/
 
 attack_t	*get_attack_ave(
-	attack_t *ls,
+	attack_t *ls, long range,
 	bool_t flg_bash, bool_t flg_monk,
 	bool_t flg_throw, bool_t flg_spell
 )
 {
 	long	i;
-	ratio_t	sum;
+	ratio_t	n, sum;
 	static attack_t	ave;
 
 	if( ls == NULL )
@@ -518,23 +531,32 @@ attack_t	*get_attack_ave(
 	ave.dam = 0;
 
 	sum = 0;
-	for( i = 0; i < 1024; i++ ){
-		ratio_t	n;
-
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		if( ls[i].kind == ATTACK_KIND_NULL )
 			break;
+
 		if( ls[i].kind == ATTACK_KIND_SPELL ){
 			if( !flg_spell )
 				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_SELF ){
+			if( !flg_spell )
+				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_FRIEND ){
+			if( !flg_spell )
+				continue;
 		} else if( ls[i].flg_monk ){
 			if( !flg_monk )
 				continue;
 		} else if( ls[i].flg_throw ){
 			if( !flg_throw )
 				continue;
+			if( ls[i].range_throw < range )
+				continue;
 		} else {
 			if( !flg_bash )
 				continue;
+			if( ls[i].range_bash < range )
+				continue;
 		}
 
 		n = ls[i].ratio;
@@ -557,7 +579,7 @@ attack_t	*get_attack_ave(
 /**/
 
 attack_t	*get_attack_randm(
-	attack_t *ls,
+	attack_t *ls, long range,
 	bool_t flg_bash, bool_t flg_monk,
 	bool_t flg_throw, bool_t flg_spell
 )
@@ -569,21 +591,32 @@ attack_t	*get_attack_randm(
 		return NULL;
 
 	sum = 0;
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		if( ls[i].kind == ATTACK_KIND_NULL )
 			break;
+
 		if( ls[i].kind == ATTACK_KIND_SPELL ){
 			if( !flg_spell )
 				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_SELF ){
+			if( !flg_spell )
+				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_FRIEND ){
+			if( !flg_spell )
+				continue;
 		} else if( ls[i].flg_monk ){
 			if( !flg_monk )
 				continue;
 		} else if( ls[i].flg_throw ){
 			if( !flg_throw )
 				continue;
+			if( ls[i].range_throw < range )
+				continue;
 		} else {
 			if( !flg_bash )
 				continue;
+			if( ls[i].range_bash < range )
+				continue;
 		}
 
 		sum += ls[i].ratio;
@@ -595,12 +628,18 @@ attack_t	*get_attack_randm(
 	n = randm( sum );
 
 	sum = 0;
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		if( ls[i].kind == ATTACK_KIND_NULL )
 			break;
 		if( ls[i].kind == ATTACK_KIND_SPELL ){
 			if( !flg_spell )
 				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_SELF ){
+			if( !flg_spell )
+				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_FRIEND ){
+			if( !flg_spell )
+				continue;
 		} else if( ls[i].flg_monk ){
 			if( !flg_monk )
 				continue;
@@ -635,12 +674,18 @@ ratio_t	calc_attack_sum(
 		return 0;
 
 	sum = 0;
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		if( ls[i].kind == ATTACK_KIND_NULL )
 			break;
 		if( ls[i].kind == ATTACK_KIND_SPELL ){
 			if( !flg_spell )
 				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_SELF ){
+			if( !flg_spell )
+				continue;
+		} else if( ls[i].kind == ATTACK_KIND_SPELL_FRIEND ){
+			if( !flg_spell )
+				continue;
 		} else if( ls[i].flg_monk ){
 			if( !flg_monk )
 				continue;
@@ -660,7 +705,7 @@ ratio_t	calc_attack_sum(
 
 /**/
 
-bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
+bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack, long range )
 {
 	chr_kind_t	o_k, d_k;
 	long	o_hit, o_dam;
@@ -674,13 +719,16 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 	flg_stat_t	stat;
 	item_t	*eq;
 	pos_t	bgn, end;
-	bool_t	flg_crtcl;
+	bool_t	flg_crtcl, flg_ninja_crtcl;
+	bool_t	flg_dead;
 	bool_t	flg_d_null;
 	bool_t	flg_auto_hit;
 	bool_t	flg_serious;
+	bool_t	flg_err_range;
 	char	mjr;
-	curs_attr_n_t	attr_err = CURS_ATTR_N_ERR;
-	curs_attr_n_t	attr_dam, attr_crtcl;
+
+	if( attack == NULL )
+		return FALSE;
 
 	if( o_p == NULL )
 		return FALSE;
@@ -704,14 +752,36 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 		if( ((mbr_t *)o_p)->trgt.kind == TRGT_KIND_MNSTR_NULL )
 			flg_d_null = TRUE;
 
-	if( d_k == CHR_KIND_MBR ){
-		attr_dam = CURS_ATTR_N_FX_MBR_MINUS;
-		attr_crtcl = CURS_ATTR_N_FX_MBR_CRTCL;
-	} else {
-		attr_dam = CURS_ATTR_N_FX_MNSTR_MINUS;
-		attr_crtcl = CURS_ATTR_N_FX_MNSTR_CRTCL;
+	eq = NULL;
+	if( attack->flg_throw && (o_k == CHR_KIND_MBR) ){
+		eq = get_mbr_item_asgn_equip( (mbr_t *)o_p );
+		if( eq[EQUIP_KIND_ARW].kind == ITEM_KIND_NULL ){
+			print_msg( FLG_MSG_ERR, MSG_ERR_THROW, o_name );
+
+			clr_chr_trgt_act( (mbr_t *)o_p, TRUE );
+
+			return FALSE;
+		}
 	}
 
+	/* ΥΥå */
+
+	flg_err_range = FALSE;
+	if( attack->flg_throw ){
+		if( attack->range_throw < range )
+			flg_err_range = TRUE;
+	} else {
+		if( attack->range_bash < range )
+			flg_err_range = TRUE;
+	}
+	if( flg_err_range ){
+		print_msg( FLG_MSG_ERR, MSG_ERR_ATTACK_RANGE, o_p->name );
+
+		return FALSE;
+	}
+
+	/* ø */
+
 	flg_serious = TRUE;
 	if( d_p != NULL ){
 		if( d_p->attitude == ATTITUDE_MATE ){
@@ -719,12 +789,12 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 				/* ̣ɤʤø */
 				flg_serious = FALSE;
 			}
-		} else if( o_p->kind == CHR_KIND_MBR ){
-			/* ɸ¦̣ʳʤŨˤޤ魯 */
-			set_attitude_mnstr_group(
-					d_p->face.mjr, ATTITUDE_HOSTILE );
+		} else {
+			set_enemy_mnstr_group( o_p, d_p );
 		}
 	}
+	if( chk_flg( get_flg_party(), FLG_PARTY_ACTUAL_FIGHTING ) )
+		flg_serious = TRUE;
 
 	/* ¦̾μ */
 
@@ -746,18 +816,6 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 		d_y = d_p->y;
 	}
 
-	eq = NULL;
-	if( attack->flg_throw && (o_k == CHR_KIND_MBR) ){
-		eq = get_mbr_item_asgn_equip( (mbr_t *)o_p );
-		if( eq[EQUIP_KIND_ARW].kind == ITEM_KIND_NULL ){
-			print_msg( attr_err, FLG_NULL,
-					MSG_ERR_THROW, o_name );
-			clr_chr_trgt_act( (mbr_t *)o_p, TRUE );
-
-			return FALSE;
-		}
-	}
-
 	/* ¦廊뤫 */
 
 	if( o_k == CHR_KIND_MBR )
@@ -802,11 +860,9 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 	if( !flg_d_null ){
 		if( !chk_find_chr( o_p, d_p ) ){
 			if( o_k == CHR_KIND_MBR ){
-				curs_attrset_n( attr_err );
-				print_msg( FLG_NULL,
+				print_msg( FLG_MSG_ERR,
 						MSG_ERR_NOT_FIND,
 						o_name, d_name );
-				curs_attrset_dflt();
 			}
 			return FALSE;
 		}
@@ -835,7 +891,7 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 		d_resi = _100_PERCENT;
 	} else {
 		d_def = get_def( d_p );
-		d_def = roll_dice( d_def );
+		d_def = roll_dice( d_def, FALSE );
 		d_def += d_def * d_p->fx_data.bless;
 		d_ac = get_ac( d_p );
 		if( d_k == CHR_KIND_MBR ){
@@ -855,22 +911,38 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 	/* ¦Υǡ */
 
 	o_hit = get_hit( o_p, attack );
-	o_hit = roll_dice( o_hit );
+	o_hit = roll_dice( o_hit, FALSE );
 	o_hit += o_hit * o_p->fx_data.bless;
+	o_hit = o_hit * get_chr_throw_range_rate( o_p, d_p ) / _100_PERCENT;
+
 	o_dam = get_dam( o_p, attack );
 	flg_crtcl = rate_randm( get_crtcl( o_p, d_p, attack ) );
 
 	if( !flg_d_null ){
-		o_dam = roll_dice( o_dam );
+		o_dam = roll_dice( o_dam, TRUE );
 		o_dam = calc_resi( d_resi, o_dam );
 		o_dam -= d_ac;
 	}
 	if( o_dam <= 0 )
 		o_dam = 1;
-	if( flg_crtcl ){
+
+	/* ƥ */
+
+	if( flg_crtcl && flg_serious )
+		flg_ninja_crtcl = chk_ninja_crtcl( o_p, d_p );
+	else
+		flg_ninja_crtcl = FALSE;
+
+	if( flg_ninja_crtcl ){
+		curs_attrset_chr( d_p, -1, TRUE );
+		print_msg( FLG_NULL, MSG_FIGHT_CRTCL_NINJA, o_name );
+		curs_attrset_dflt();
+
+		vfx_ninja_crtcl( d_k, end.x, end.y );
+	} else if( flg_crtcl ){
 		o_dam *= 2;
 
-		curs_attrset_n( attr_crtcl );
+		curs_attrset_chr( d_p, -1, TRUE );
 		print_msg( FLG_NULL, MSG_FIGHT_CRTCL_STD, o_name );
 		curs_attrset_dflt();
 
@@ -882,15 +954,26 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 		o_dam *= 2;
 	}
 
+	print_msg( FLG_MSG_DEBUG, "o_hit: %ld, d_def: %ld",
+			o_hit, d_def );
 
 	/* ʤ */
 
 	if( flg_d_null || (!flg_crtcl && (d_def > o_hit)) ){
-		if( flg_d_null )
-			print_msg( FLG_NULL, MSG_ERR_MNSTR_NULL );
-		else
+		if( flg_d_null ){
+			print_msg( FLG_MSG_ERR, MSG_ERR_MNSTR_NULL );
+		} else {
+			fx_t	*fx;
+
+			fx = srch_fx( &(d_p->fx), FX_KIND_SUBSTITUTE );
+			if( fx != NULL ){
+				print_msg( FLG_NULL, MSG_FIGHT_SUBSTITUTE,
+						d_name );
+			}
+
 			print_msg( FLG_NULL, MSG_FIGHT_MISS_STD,
 					o_name, d_name );
+		}
 
 		if( o_k != CHR_KIND_MBR )
 			return TRUE;
@@ -914,6 +997,29 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 
 	/* ⤬ä */
 
+	/* 񤹤 */
+
+	if( attack->flg_throw && (eq != NULL) && (o_k == CHR_KIND_MBR) ){
+		if( (eq[EQUIP_KIND_ARW].kind == ITEM_KIND_WPN)
+				&& chk_flg( eq[EQUIP_KIND_ARW].dat.wpn.flg,
+				FLG_WPN_TAB_NOT_SPEND ) ){
+			/* 񤻤 */
+			dec_throw( (mbr_t *)o_p, TRUE, d_x, d_y );
+		} else {
+			dec_throw( (mbr_t *)o_p, FALSE, d_x, d_y );
+		}
+	}
+
+	/* Ƥ */
+
+	if( chk_flg( d_p->stat, FLG_STAT_IRON_BODY ) ){
+		print_msg( FLG_NULL, MSG_REPEL_ATTACK,
+				d_p->name, o_p->name );
+		return FALSE;
+	}
+
+	/* Υե */
+
 	fight_fx( o_p, d_p, attack );
 
 	/* ݡä */
@@ -942,9 +1048,11 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 
 	/* ᡼ɽ */
 
-	curs_attrset_n( attr_dam );
-	print_msg( FLG_NULL, MSG_FIGHT_HIT_STD, d_name, o_dam );
-	curs_attrset_dflt();
+	if( !flg_ninja_crtcl ){
+		curs_attrset_chr( d_p, -1, FALSE );
+		print_msg( FLG_NULL, MSG_FIGHT_HIT_STD, d_name, o_dam );
+		curs_attrset_dflt();
+	}
 
 	/*  */
 
@@ -961,9 +1069,9 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 			o_abl = ABL_KIND_MON;
 		else
 			o_abl = ABL_KIND_FIG;
-		o_lev = get_lev( o_p, o_abl );
-
 		d_abl = get_def_abl( d_p );
+
+		o_lev = get_lev( o_p, o_abl );
 		d_lev = get_lev( d_p, d_abl );
 
 		if( o_lev >= d_lev )
@@ -972,30 +1080,30 @@ bool_t	fight( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 		exp_chr( (mbr_t *)o_p, TRAINING_EXP, TRUE, TRUE );
 	} while( 0 );
 
-	/* HP򸺤餹 */
+	/* ɲø */
 
 	added_fx( d_p, attack->resi_kind );
-	if( (d_hp != NULL) && (d_hp->n <= o_dam) ){
+
+	/* HP򸺤餹 */
+
+	flg_dead = FALSE;
+	if( flg_ninja_crtcl )
+		flg_dead = TRUE;
+	if( (d_hp != NULL) && (d_hp->n <= o_dam) )
+		flg_dead = TRUE;
+
+	if( flg_dead ){
 		/*  */
-		curs_attrset_n( attr_crtcl );
+		curs_attrset_chr( d_p, -1, TRUE );
 		print_msg( FLG_NULL, MSG_FIGHT_KILL_STD,
 				o_name, d_name );
 		curs_attrset_dflt();
 	}
-	add_hp( d_p, -o_dam );
 
-	/* 񤹤 */
-
-	if( attack->flg_throw && (eq != NULL) && (o_k == CHR_KIND_MBR) ){
-		if( (eq[EQUIP_KIND_ARW].kind == ITEM_KIND_WPN)
-				&& chk_flg( eq[EQUIP_KIND_ARW].dat.wpn.flg,
-				FLG_WPN_TAB_NOT_SPEND ) ){
-			/* 񤻤 */
-			dec_throw( (mbr_t *)o_p, TRUE, d_x, d_y );
-		} else {
-			dec_throw( (mbr_t *)o_p, FALSE, d_x, d_y );
-		}
-	}
+	if( flg_ninja_crtcl )
+		die_chr( d_p, TRUE, TRUE, TRUE );
+	else
+		add_hp( d_p, -o_dam, TRUE, FALSE );
 
 	return TRUE;
 }
@@ -1035,17 +1143,20 @@ bool_t	spell_attack(
 	long	dam;
 	long	ac;
 	bool_t	flg_serious;
-	curs_attr_n_t	attr_dam, attr_crtcl;
 
 	if( o_p == NULL )
 		return FALSE;
 	if( d_p == NULL ){
-		print_msg( FLG_NULL, MSG_ERR_MNSTR_NULL );
+		print_msg( FLG_MSG_ERR, MSG_ERR_MNSTR_NULL );
+
 		if( o_p->kind == CHR_KIND_MBR )
 			clr_chr_trgt_act( (mbr_t *)o_p, FALSE );
+
 		return FALSE;
 	}
 
+	/* ø */
+
 	flg_serious = TRUE;
 	if( d_p != NULL ){
 		if( d_p->attitude == ATTITUDE_MATE ){
@@ -1053,12 +1164,12 @@ bool_t	spell_attack(
 				/* ̣ɤʤø */
 				flg_serious = FALSE;
 			}
-		} else if( o_p->kind == CHR_KIND_MBR ){
-			/* ɸ¦̣ʳʤŨˤޤ魯 */
-			set_attitude_mnstr_group(
-					d_p->face.mjr, ATTITUDE_HOSTILE );
+		} else {
+			set_enemy_mnstr_group( o_p, d_p );
 		}
 	}
+	if( chk_flg( get_flg_party(), FLG_PARTY_ACTUAL_FIGHTING ) )
+		flg_serious = TRUE;
 
 	/* ¦ǡ */
 
@@ -1073,17 +1184,6 @@ bool_t	spell_attack(
 	d_pos.y = d_p->y;
 	strncpy( d_name, d_p->name, CHR_NAME_MAX_LEN );
 
-	if( d_p->kind == CHR_KIND_MBR ){
-		attr_dam = CURS_ATTR_N_FX_MBR_MINUS;
-		attr_crtcl = CURS_ATTR_N_FX_MBR_CRTCL;
-	} else if( d_p->kind == CHR_KIND_MNSTR ){
-		attr_dam = CURS_ATTR_N_FX_MNSTR_MINUS;
-		attr_crtcl = CURS_ATTR_N_FX_MNSTR_CRTCL;
-	} else {
-		attr_dam = CURS_ATTR_N_NORMAL;
-		attr_crtcl = CURS_ATTR_N_NORMAL;
-	}
-
 	/* Ǥ뤫 */
 
 	if( chk_flg_or( o_stat, FLG_STAT_NOT_EXIST
@@ -1099,7 +1199,8 @@ bool_t	spell_attack(
 	/* Ƥ뤫 */
 
 	if( !chk_find( &o_pos, &d_pos ) ){
-		print_msg( FLG_NULL, MSG_ERR_NOT_FIND, o_name, d_name );
+		print_msg( FLG_MSG_ERR, MSG_ERR_NOT_FIND, o_name, d_name );
+
 		return FALSE;
 	}
 
@@ -1108,9 +1209,9 @@ bool_t	spell_attack(
 	if( flg_stick )
 		dam = get_dam_wand( o_p );
 	else
-		dam = get_dam_spell( o_p, abl );
+		dam = get_dam_spell( o_p, abl, get_sub_abl( abl ) );
 	dam = dam * rate / _100_PERCENT;
-	dam = roll_dice( dam );
+	dam = roll_dice( dam, TRUE );
 
 	/* ޡ饹 */
 
@@ -1135,7 +1236,7 @@ bool_t	spell_attack(
 
 	/* ᡼ɽ */
 
-	curs_attrset_n( attr_dam );
+	curs_attrset_chr( d_p, -1, FALSE );
 	print_msg( FLG_NULL, MSG_FIGHT_HIT_STD, d_name, dam );
 	curs_attrset_dflt();
 
@@ -1144,11 +1245,11 @@ bool_t	spell_attack(
 	added_fx( d_p, resi );
 	if( d_p->abl.hp.n <= dam ){
 		/*  */
-		curs_attrset_n( attr_crtcl );
+		curs_attrset_chr( d_p, -1, TRUE );
 		print_msg( FLG_NULL, MSG_FIGHT_KILL_STD, o_name, d_name );
 		curs_attrset_dflt();
 	}
-	add_hp( d_p, -dam );
+	add_hp( d_p, -dam, TRUE, FALSE );
 
 	return TRUE;
 }
@@ -1175,11 +1276,11 @@ void	added_fx( chr_t *chr, resi_kind_t resi )
 	case RESI_KIND_STIN:
 	case RESI_KIND_HEAT:
 	case RESI_KIND_COLD:
+	case RESI_KIND_MIND:
 		break;
 	case RESI_KIND_ACID:
 		fx_acid( chr );
 		break;
-	case RESI_KIND_MIND:
 	case RESI_KIND_ELEC:
 		break;
 	case RESI_KIND_POIS:
@@ -1226,16 +1327,16 @@ bool_t	chr_break_door( chr_t *chr, long dr_n )
 
 	clr_chr_trgt_act( chr, FALSE );
 
-	if( set_attack_chr_bash( &attack, chr, FALSE ) )
+	if( set_attack_chr_bash( chr, &attack, 1, FALSE ) )
 		hit = get_hit( chr, &attack );
 	else
 		hit = get_hit( chr, NULL );
-	hit = roll_dice( hit );
+	hit = roll_dice( hit, FALSE );
 	hit += hit * chr->fx_data.bless;
 
 	if( DOOR_DEF > hit ){
 		if( chr->kind == CHR_KIND_MBR ){
-			print_msg( FLG_NULL, MSG_ERR_DOOR_BREAK_LEV,
+			print_msg( FLG_MSG_ERR, MSG_ERR_DOOR_BREAK_LEV,
 					chr->name );
 		}
 		return FALSE;
@@ -1260,15 +1361,16 @@ bool_t	chr_break_trap( chr_t *chr, trap_t *trap )
 	if( chr == NULL )
 		return FALSE;
 
-	if( set_attack_chr_bash( &attack, chr, FALSE ) )
+	if( set_attack_chr_bash( chr, &attack, 1, FALSE ) )
 		hit = get_hit( chr, &attack );
 	else
 		hit = get_hit( chr, NULL );
-	hit = roll_dice( hit );
+	hit = roll_dice( hit, FALSE );
 	hit += hit * chr->fx_data.bless;
 
 	if( TRAP_DEF > hit ){
-		print_msg( FLG_NULL, MSG_ERR_TRAP_BREAK_LEV );
+		print_msg( FLG_MSG_ERR, MSG_ERR_TRAP_BREAK_LEV );
+
 		caught_trap( trap, chr );
 	} else {
 		print_msg( FLG_NULL, MSG_TRAP_BREAK, chr->name );
@@ -1292,20 +1394,23 @@ bool_t	chr_break_item( chr_t *chr, item_t *item )
 		return FALSE;
 
 	if( item->kind == ITEM_KIND_JUNK ){
-		print_msg( FLG_NULL, MSG_ERR_ITEM_BREAK_BROKEN, item->name );
+		print_msg( FLG_MSG_ERR, MSG_ERR_ITEM_BREAK_BROKEN,
+				item->name );
+
 		return TRUE;
 	}
 
 
-	if( set_attack_chr_bash( &attack, chr, FALSE ) )
+	if( set_attack_chr_bash( chr, &attack, 1, FALSE ) )
 		hit = get_hit( chr, &attack );
 	else
 		hit = get_hit( chr, NULL );
-	hit = roll_dice( hit );
+	hit = roll_dice( hit, FALSE );
 	hit += hit * chr->fx_data.bless;
 
 	if( ITEM_DEF > hit ){
-		print_msg( FLG_NULL, MSG_ERR_ITEM_BREAK_LEV, item->name );
+		print_msg( FLG_MSG_ERR, MSG_ERR_ITEM_BREAK_LEV, item->name );
+
 		return FALSE;
 	}
 
@@ -1416,7 +1521,6 @@ long	get_hit( chr_t *p, const attack_t *attack )
 
 	/* ǽ */
 
-	hit = 0;
 	wpn_hit = attack->hit;
 
 	if( attack->flg_throw )
@@ -1426,8 +1530,10 @@ long	get_hit( chr_t *p, const attack_t *attack )
 	else
 		abl = ABL_KIND_FIG;
 
+	hit = 0;
 	hit += calc_lev_to_val( get_lev( p, ABL_KIND_DEX ), ABL_KIND_DEX );
 	hit += calc_lev_to_val( get_lev( p, abl ), abl );
+	hit /= 2;
 
 	if( abl == ABL_KIND_MON ){
 		long	n;
@@ -1444,6 +1550,10 @@ long	get_hit( chr_t *p, const attack_t *attack )
 		}
 	}
 
+	/* FXͤä */
+
+	wpn_hit += p->fx_data.wpn_hit;
+
 #ifdef	FLG_ITEM_ADD_RATE
 
 	hit += HIT_FIT_AVE_N * wpn_hit / _100_PERCENT;
@@ -1465,7 +1575,7 @@ long	get_hit( chr_t *p, const attack_t *attack )
 long	get_dam( chr_t *p, const attack_t *attack )
 {
 	long	dam, wpn_dam;
-	abl_kind_t	abl;
+	abl_kind_t	abl, sub_abl;
 
 	if( p == NULL )
 		return 1;
@@ -1474,18 +1584,23 @@ long	get_dam( chr_t *p, const attack_t *attack )
 
 	/* ǽ */
 
-	dam = 0;
 	wpn_dam = attack->dam;
 
-	if( attack->flg_throw )
+	if( attack->flg_throw ){
 		abl = ABL_KIND_HUN;
-	else if( attack->flg_monk )
+		sub_abl = ABL_KIND_DEX;
+	} else if( attack->flg_monk ){
 		abl = ABL_KIND_MON;
-	else
+		sub_abl = ABL_KIND_STR;
+	} else {
 		abl = ABL_KIND_FIG;
+		sub_abl = ABL_KIND_STR;
+	}
 
-	dam += calc_lev_to_val( get_lev( p, ABL_KIND_STR ), ABL_KIND_STR );
+	dam = 0;
 	dam += calc_lev_to_val( get_lev( p, abl ), abl );
+	dam += calc_lev_to_val( get_lev( p, sub_abl ), sub_abl );
+	dam /= 2;
 
 	if( abl == ABL_KIND_MON ){
 		long	n;
@@ -1502,6 +1617,10 @@ long	get_dam( chr_t *p, const attack_t *attack )
 		}
 	}
 
+	/* FXͤä */
+
+	wpn_dam += p->fx_data.wpn_dam;
+
 #ifdef	FLG_ITEM_ADD_RATE
 
 	dam += DAM_FIT_AVE_N * wpn_dam / _100_PERCENT;
@@ -1532,11 +1651,12 @@ long	get_def( chr_t *p )
 
 	/* ǽ */
 
-	def = 0;
 	abl = get_def_abl( p );
 
+	def = 0;
 	def += calc_lev_to_val( get_lev( p, ABL_KIND_AGI ), ABL_KIND_AGI );
 	def += calc_lev_to_val( get_lev( p, abl ), abl );
+	def /= 2;
 
 	if( p->kind == CHR_KIND_MBR ){
 		rate = calc_rate_abl_equip( p, abl );
@@ -1549,9 +1669,13 @@ long	get_def( chr_t *p )
 		armor_def = DEF_MIN_RATE;
 
 #ifdef	FLG_ITEM_ADD_RATE
+
 	def += DEF_AVE_N * armor_def / _100_PERCENT;
+
 #else	/* FLG_ITEM_ADD_RATE */
+
 	def = def * armor_def / _100_PERCENT;
+
 #endif	/* FLG_ITEM_ADD_RATE */
 
 	if( def < 1 )
@@ -1622,8 +1746,10 @@ long	get_armor_def( chr_t *p, abl_kind_t abl )
 		for( n = 0; n < EQUIP_KIND_MAX_N; n++ ){
 			if( eq[n].kind != ITEM_KIND_ARMOR )
 				continue;
+			if( eq[n].tab.armor == NULL )
+				continue;
 
-			armor_def += eq[n].dat.armor.def;
+			armor_def += eq[n].tab.armor->def;
 			armor_def += add_def( eq[n].dat.armor.add_def );
 		}
 	} else if( p->kind == CHR_KIND_MNSTR ){
@@ -1633,6 +1759,7 @@ long	get_armor_def( chr_t *p, abl_kind_t abl )
 	}
 
 	/* FXͤä */
+
 	armor_def += p->fx_data.armor_def;
 
 	return armor_def;
@@ -1664,8 +1791,10 @@ long	get_ac( chr_t *p )
 		for( n = 0; n < EQUIP_KIND_MAX_N; n++ ){
 			if( eq[n].kind != ITEM_KIND_ARMOR )
 				continue;
+			if( eq[n].tab.armor == NULL )
+				continue;
 
-			armor_ac += eq[n].dat.armor.ac;
+			armor_ac += eq[n].tab.armor->ac;
 			armor_ac += add_ac( eq[n].dat.armor.add_ac );
 		}
 	} else if( p->kind == CHR_KIND_MNSTR ){
@@ -1675,6 +1804,7 @@ long	get_ac( chr_t *p )
 	}
 
 	/* FXͤä */
+
 	armor_ac += p->fx_data.armor_ac;
 
 	ac = AC_AVE_N * armor_ac / _100_PERCENT;
@@ -1704,8 +1834,7 @@ rate_t	get_crtcl( chr_t *o_p, chr_t *d_p, const attack_t *attack )
 
 rate_t	get_wpn_crtcl( chr_t *p, const attack_t *attack )
 {
-	long	thf;
-	rate_t	crtcl;
+	rate_t	crtcl, wpn_crtcl, thf;
 
 	if( p == NULL )
 		return 1;
@@ -1714,17 +1843,32 @@ rate_t	get_wpn_crtcl( chr_t *p, const attack_t *attack )
 
 	/* ǽ */
 
-	thf = calc_lev_to_val( get_lev( p, ABL_KIND_THI ), ABL_KIND_THI );
-	crtcl = thf / 2;
+	thf = 0;
+	thf += get_lev( p, ABL_KIND_THI );
+	thf += get_lev( p, ABL_KIND_DEX );
+	thf /= 2;
+	thf = thf * WPN_CRTCL_THF_LEV_RATE / _100_PERCENT;
+	if( thf > WPN_CRTCL_THF_MAX_RATE )
+		thf = WPN_CRTCL_THF_MAX_RATE;
+
+	crtcl = thf;
+
+	/* ͤä */
+
+	wpn_crtcl = attack->crtcl;
+
+	/* FXͤä */
+
+	wpn_crtcl += p->fx_data.wpn_crtcl;
 
 #ifdef	FLG_ITEM_ADD_RATE
 
-	crtcl += WPN_CRTCL_FIT_AVE_N * attack->crtcl / _100_PERCENT;
+	crtcl += WPN_CRTCL_FIT_AVE_N * wpn_crtcl / _100_PERCENT;
 
 #else	/* FLG_ITEM_ADD_RATE */
 
 	crtcl += WPN_CRTCL_FIT_AVE_N;
-	crtcl = crtcl * attack->crtcl / _100_PERCENT;
+	crtcl = crtcl * wpn_crtcl / _100_PERCENT;
 
 #endif	/* FLG_ITEM_ADD_RATE */
 
@@ -1738,8 +1882,7 @@ rate_t	get_wpn_crtcl( chr_t *p, const attack_t *attack )
 
 rate_t	get_armor_crtcl( chr_t *p )
 {
-	rate_t	crtcl;
-	rate_t	armor_crtcl;
+	rate_t	crtcl, armor_crtcl;
 
 	if( p == NULL )
 		return 1;
@@ -1752,26 +1895,32 @@ rate_t	get_armor_crtcl( chr_t *p )
 		equip_kind_t	n;
 
 		/* ɶΥơ֥ */
+
 		eq = get_mbr_item_asgn_equip( (mbr_t *)p );
 		if( eq == NULL )
 			return 0;
 
 		/* ɶͤä */
+
 		for( n = 0; n < EQUIP_KIND_MAX_N; n++ ){
 			if( eq[n].kind != ITEM_KIND_ARMOR )
 				continue;
+			if( eq[n].tab.armor == NULL )
+				continue;
 
-			armor_crtcl += eq[n].dat.armor.crtcl;
+			armor_crtcl += eq[n].tab.armor->crtcl;
 			armor_crtcl += add_armor_crtcl(
 					eq[n].dat.armor.add_crtcl );
 		}
 	} else if( p->kind == CHR_KIND_MNSTR ){
 		/* ɶͤä */
+
 		armor_crtcl += p->mnstr_tab->armor_crtcl;
 		armor_crtcl += add_armor_crtcl( p->add_crtcl );
 	}
 
 	/* FXͤä */
+
 	armor_crtcl += p->fx_data.armor_crtcl;
 
 	crtcl = ARMOR_CRTCL_AVE_N * armor_crtcl / _100_PERCENT;
@@ -1784,17 +1933,69 @@ rate_t	get_armor_crtcl( chr_t *p )
 
 /**/
 
+bool_t	chk_ninja_crtcl( chr_t *o_p, chr_t *d_p )
+{
+	abl_kind_t	o_abl_m, o_abl_s;
+	abl_kind_t	d_abl_m, d_abl_s;
+	long	o_val, d_val;
+	long	o_dice, d_dice;
+
+	if( o_p == NULL )
+		return FALSE;
+	if( d_p == NULL )
+		return FALSE;
+
+	if( chk_resi_stat( d_p, FLG_STAT_DEAD ) )
+		return FALSE;
+
+	o_abl_m = ABL_KIND_NIN;
+	o_abl_s = ABL_KIND_DEX;
+	o_val = 0;
+	o_val += calc_lev_to_val( get_lev( o_p, o_abl_m ), o_abl_m );
+	o_val += calc_lev_to_val( get_lev( o_p, o_abl_s ), o_abl_s );
+	o_val /= 2;
+
+	d_abl_m = get_specialist_abl( d_p );
+	d_abl_s = ABL_KIND_DEX;
+	d_val = 0;
+	d_val += calc_lev_to_val( get_lev( d_p, d_abl_m ), d_abl_m );
+	d_val += calc_lev_to_val( get_lev( d_p, d_abl_s ), d_abl_s );
+	d_val /= 2;
+
+	if( get_lev( o_p, ABL_KIND_NIN ) <= 0 )
+		return FALSE;
+
+	o_val = o_val * 150 / _100_PERCENT;
+
+	o_dice = roll_dice( o_val, FALSE );
+	d_dice = roll_dice( d_val, FALSE );
+
+	if( o_dice > d_dice )
+		return TRUE;
+	else
+		return FALSE;
+}
+
+/**/
+
 long	get_dam_wand( chr_t *p )
 {
+	abl_kind_t	abl_main, abl_sub;
+
 	long	dam;
 
 	if( p == NULL )
 		return 1;
 
-	dam = get_specialist_lev( p );
-	dam *= 2;
+	abl_main = get_specialist_lev( p );
+	abl_sub = get_specialist_lev( p );
 
-	dam *= DAM_WAND_MUL;
+	dam = 0;
+	dam += calc_lev_to_val( get_lev( p, abl_main ), abl_main );
+	dam += calc_lev_to_val( get_lev( p, abl_sub ), abl_sub );
+	dam /= 2;
+
+	dam = dam * DAM_WAND_RATE / _100_PERCENT;
 	dam += DAM_WAND_ADD;
 
 	if( dam < 1 )
@@ -1805,46 +2006,19 @@ long	get_dam_wand( chr_t *p )
 
 /**/
 
-long	get_dam_spell( chr_t *p, abl_kind_t abl )
+long	get_dam_spell( chr_t *p, abl_kind_t abl_main, abl_kind_t abl_sub )
 {
-	abl_kind_t	sub_abl;
 	long	dam;
 
 	if( p == NULL )
 		return 1;
 
-	switch( abl ){
-	case ABL_KIND_MAG:
-		sub_abl = ABL_KIND_INT;
-		break;
-	case ABL_KIND_SOR:
-		sub_abl = ABL_KIND_INT;
-		break;
-	case ABL_KIND_ENC:
-		sub_abl = ABL_KIND_INT;
-		break;
-	case ABL_KIND_SUM:
-		sub_abl = ABL_KIND_INT;
-		break;
-	case ABL_KIND_PRI:
-		sub_abl = ABL_KIND_WIS;
-		break;
-	case ABL_KIND_SHA:
-		sub_abl = ABL_KIND_CHA;
-		break;
-	case ABL_KIND_BAR:
-		sub_abl = ABL_KIND_CHA;
-		break;
-	case ABL_KIND_NIN:
-		sub_abl = ABL_KIND_NIN;
-		break;
-	default:
-		return 1;
-	}
+	dam = 0;
+	dam += calc_lev_to_val( get_lev( p, abl_main ), abl_main );
+	dam += calc_lev_to_val( get_lev( p, abl_sub ), abl_sub );
+	dam /= 2;
 
-	dam = get_lev( p, abl );
-	dam += get_lev( p, sub_abl );
-	dam *= DAM_SPELL_MUL;
+	dam = dam * DAM_SPELL_RATE / _100_PERCENT;
 	dam += DAM_SPELL_ADD;
 
 	if( dam < 1 )
@@ -1863,8 +2037,8 @@ rate_t	get_resi_def( chr_t *p, resi_kind_t resi )
 		return _100_PERCENT;
 
 	n = get_resi( p, resi );
-	n = n * get_resi_armor( p, resi ) / _100_PERCENT;
-
+	if( n == RESI_PERFECT_N )
+		return RESI_PERFECT_N;
 	if( n < 1)
 		n = 1;
 
@@ -1873,12 +2047,57 @@ rate_t	get_resi_def( chr_t *p, resi_kind_t resi )
 
 /**/
 
+rate_t	get_chr_throw_range_rate( chr_t *o_p, chr_t *d_p )
+{
+	rate_t	rate;
+
+	if( o_p == NULL )
+		return 0;
+	if( d_p == NULL )
+		return 0;
+
+	rate = get_throw_range_rate(
+			o_p->trgt.true_x, o_p->trgt.true_y,
+			d_p->trgt.true_x, d_p->trgt.true_y );
+
+	return rate;
+}
+
+/**/
+
+rate_t	get_throw_range_rate( long x1, long y1, long x2, long y2 )
+{
+	long	dx, dy, r;
+	rate_t	rate;
+
+	if( !clip_pos( x1, y1 ) )
+		return 0;
+	if( !clip_pos( x2, y2 ) )
+		return 0;
+
+	dx = labs( x1 - x2 );
+	dy = labs( y1 - y2 );
+
+	r = max_l( dx, dy );
+
+	rate = _100_PERCENT;
+	rate -= r * RATE_PER_RANGE / _100_PERCENT;
+
+	print_msg( FLG_MSG_DEBUG, "fight range rate: %ld", rate );
+
+	return rate;
+}
+
+/**/
+
 char	*get_msg_attack( const attack_t *attack )
 {
 	switch( attack->kind ){
 	case ATTACK_KIND_NULL:
 		break;
 	case ATTACK_KIND_SPELL:
+	case ATTACK_KIND_SPELL_SELF:
+	case ATTACK_KIND_SPELL_FRIEND:
 		break;
 	case ATTACK_KIND_PUNCH:
 		return MSG_ATTACK_PUNCH;
@@ -1898,6 +2117,8 @@ char	*get_msg_attack( const attack_t *attack )
 		return MSG_ATTACK_THROW;
 	case ATTACK_KIND_BITE:
 		return MSG_ATTACK_BITE;
+	case ATTACK_KIND_PECK:
+		return MSG_ATTACK_PECK;
 	case ATTACK_KIND_CLAW:
 		return MSG_ATTACK_CLAW;
 	case ATTACK_KIND_NIP:
