@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,9 +25,11 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: gmain.c,v 1.32 2003/11/21 17:51:58 bowkenken Exp $
+* $Id: gmain.c,v 1.57 2004/07/30 14:29:27 bowkenken Exp $
 *****************************************************************/
 
+#include	<signal.h>
+
 #include	"gmain.h"
 #include	"misc.h"
 #include	"turn.h"
@@ -45,31 +46,39 @@
 #include	"draw.h"
 #include	"curs.h"
 #include	"menu.h"
+#include	"amenu.h"
+#include	"request.h"
+#include	"gfile.h"
 #include	"msg.h"
 #include	"ver.h"
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-#include	"turn_prot.h"
-#include	"dun_prot.h"
-#include	"town_prot.h"
-#include	"item_prot.h"
-#include	"spell_prot.h"
-#include	"chr_prot.h"
-#include	"party_prot.h"
-#include	"mnstr_prot.h"
-#include	"pet_prot.h"
-#include	"fight_prot.h"
-#include	"fx_prot.h"
-#include	"trap_prot.h"
-#include	"draw_prot.h"
-#include	"curs_prot.h"
-#include	"menu_prot.h"
-#include	"tmenu_prot.h"
-#include	"amenu_prot.h"
-#include	"gfile_prot.h"
-#include	"msg_prot.h"
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+#include	"turn-prot.h"
+#include	"dun-prot.h"
+#include	"town-prot.h"
+#include	"item-prot.h"
+#include	"spell-prot.h"
+#include	"chr-prot.h"
+#include	"party-prot.h"
+#include	"mnstr-prot.h"
+#include	"pet-prot.h"
+#include	"fight-prot.h"
+#include	"fx-prot.h"
+#include	"trap-prot.h"
+#include	"draw-prot.h"
+#include	"curs-prot.h"
+#include	"menu-prot.h"
+#include	"tmenu-prot.h"
+#include	"amenu-prot.h"
+#include	"request-prot.h"
+#include	"gfile-prot.h"
+#include	"msg-prot.h"
 
-#include	"main_unix_prot.h"
+#ifdef	D_DOS
+# include	"main-dos-prot.h"
+#else	/* D_DOS */
+# include	"main-unix-prot.h"
+#endif	/* D_DOS */
 
 /**/
 
@@ -83,8 +92,12 @@ void	init_arg( void )
 	g_flg_debug = FALSE;
 #endif	/* NDEBUG */
 
+	g_flg_gui = FALSE;
+	g_flg_cui = TRUE;
 	g_flg_clr_key_buf = TRUE;
-	g_flg_use_keypad = TRUE;
+	g_flg_echo_key = FALSE;
+	g_flg_use_curs = TRUE;
+	g_flg_use_keypad = FALSE;
 	g_flg_measure_clk_tck = FALSE;
 }
 
@@ -94,11 +107,11 @@ void	init_game( void )
 {
 	dun_t	*dun;
 
+	init_signal();
 	init_scrn();
 
 	srandm( time( NULL ) );
-	set_game_day( 999, 4, 1 );
-	set_turn( 10 * TURN_PER_HOUR );
+	init_turn();
 
 	init_msg();
 	init_draw();
@@ -119,12 +132,36 @@ void	init_game( void )
 	init_menu();
 	init_tmenu();
 	init_auto_menu();
+	init_request();
+	init_mbr_formation();
 
-	init_clock( g_flg_measure_clk_tck );
+	if( g_flg_debug )
+		set_flg_no_save( TRUE );
+
+	if( !load_game_conf() )
+		g_flg_measure_clk_tck = TRUE;
+
+	if( g_flg_measure_clk_tck )
+		measure_clk_tck();
 
 	dun = get_dun();
 	dun->lev = 0;
 	up_dun( 0 );
+
+	if( !g_flg_debug ){
+		door_t	*dr;
+
+		print_msg( FLG_NULL, MSG_HINT_CANCEL_MENU_1 );
+		print_msg( FLG_NULL, MSG_HINT_CANCEL_MENU_2 );
+
+		menu_bar_load();
+
+		dr = get_shop_door( SHOP_N_BAR );
+		if( dr != NULL ){
+			open_door_shop( dr->n );
+			exec_all_phase();
+		}
+	}
 }
 
 /**/
@@ -151,8 +188,7 @@ void	exit_game( long code )
 	else
 		fp = stderr;
 
-	fprintf( fp, STR_FMT_COPYRIGHT, LS_STR_COPYRIGHT );
-	fflush( fp );
+	version( fp );
 
 	exit( code );
 }
@@ -163,6 +199,8 @@ void	about_game( void )
 {
 	static char	tmp[1023 + 1];
 
+	curs_move( WIN_MSG_DRAW_MIN_Y, SCREEN_MIN_X );
+
 	dis_scrn();
 
 	fprintf( OUT_FP, MSG_FMT_COPYRIGHT, LS_STR_COPYRIGHT );
@@ -190,7 +228,44 @@ void	version( FILE *fp )
 
 void	child_ps( void )
 {
+#ifdef	D_DOS
+	child_ps_dos();
+#else	/* D_DOS */
 	child_ps_unix();
+#endif	/* D_DOS */
+}
+
+/**/
+
+void	init_signal( void )
+{
+#ifdef	D_X68K
+	signal( SIGINT, signal_handler );
+	signal( SIGABRT, signal_handler );
+	signal( SIGTERM, signal_handler );
+#elif	defined( D_DOS )
+	signal( SIGINT, signal_handler );
+	signal( SIGSEGV, signal_handler );
+	signal( SIGABRT, signal_handler );
+	signal( SIGTERM, signal_handler );
+#else	/* D_X68K */
+	signal( SIGHUP, signal_handler );
+	signal( SIGINT, signal_handler );
+	signal( SIGKILL, signal_handler );
+	signal( SIGBUS, signal_handler );
+	signal( SIGSEGV, signal_handler );
+	signal( SIGABRT, signal_handler );
+	signal( SIGTERM, signal_handler );
+#endif	/* D_X68K */
+}
+
+/**/
+
+void	signal_handler( int sig )
+{
+	reset_scrn();
+
+	exit( EXIT_FAILURE );
 }
 
 /**/
@@ -243,28 +318,66 @@ void	dis_scrn( void )
 
 void	init_term( void )
 {
+#ifdef	D_DOS
+	init_term_dos();
+#else	/* D_DOS */
 	init_term_unix();
+#endif	/* D_DOS */
 }
 
 /**/
 
 bool_t	chk_term( void )
 {
+#ifdef	D_DOS
+	return chk_term_dos();
+#else	/* D_DOS */
 	return chk_term_unix();
+#endif	/* D_DOS */
 }
 
 /**/
 
 void	ena_term( void )
 {
+#ifdef	D_DOS
+	ena_term_dos();
+#else	/* D_DOS */
 	ena_term_unix();
+#endif	/* D_DOS */
 }
 
 /**/
 
 void	dis_term( void )
 {
+#ifdef	D_DOS
+	dis_term_dos();
+#else	/* D_DOS */
 	dis_term_unix();
+#endif	/* D_DOS */
+}
+
+/**/
+
+void	set_key_wait( void )
+{
+#ifdef	D_DOS
+	set_key_wait_dos();
+#else	/* D_DOS */
+	set_key_wait_unix();
+#endif	/* D_DOS */
+}
+
+/**/
+
+void	clr_key_wait( void )
+{
+#ifdef	D_DOS
+	clr_key_wait_dos();
+#else	/* D_DOS */
+	clr_key_wait_unix();
+#endif	/* D_DOS */
 }
 
 /**/
