@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,12 +25,16 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: gmain.h,v 1.16 2003/11/20 17:33:49 bowkenken Exp $
+* $Id: gmain.h,v 1.32 2004/07/29 07:05:00 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	GMAIN_H
 #define	GMAIN_H
 
+#if	D_DOS
+# include	"lnl-dos/config-dos.h"
+#endif	/* USE_DOS */
+
 #if	HAVE_CONFIG_H
 # include	"config.h"
 #endif	/* HAVE_CONFIG_H */
@@ -52,10 +55,18 @@
 # include	<stdarg.h>
 #endif	/* HAVE_STDARG_H */
 
+#if	HAVE_LIMITS_H
+# include	<limits.h>
+#endif	/* HAVE_LIMITS_H */
+
 #if	HAVE_STRING_H
 # include	<string.h>
 #endif	/* HAVE_STRING_H */
 
+#if	HAVE_STRINGS_H
+# include	<strings.h>
+#endif	/* HAVE_STRING_H */
+
 #if	HAVE_CTYPE_H
 # include	<ctype.h>
 #endif	/* HAVE_CTYPE_H */
@@ -64,6 +75,14 @@
 # include	<time.h>
 #endif	/* HAVE_TIME_H */
 
+#if	HAVE_MATH_H
+# include	<math.h>
+#endif	/* HAVE_MATH_H */
+
+#if	HAVE_DIRECT_H
+# include <direct.h>
+#endif	/* HAVE_DIRECT_H */
+
 #if	HAVE_SYS_STAT_H
 # include	<sys/stat.h>
 #endif	/* HAVE_SYS_STAT_H */
@@ -72,23 +91,45 @@
 # include	<unistd.h>
 #endif	/* HAVE_UNISTD_H */
 
+#ifndef	EXIT_SUCCESS
+#define	EXIT_SUCCESS	0
+#endif	/* EXIT_SUCCESS */
+
+#ifndef	EXIT_FAILURE
+#define	EXIT_FAILURE	1
+#endif	/* EXIT_FAILURE */
+
 /**/
 
 #include	"misc.h"
 #include	"turn.h"
 
-/**/
+#include	"msg-t.h"
 
-#ifdef	D_CLK_TCK
-# undef	CLK_TCK
-# define	CLK_TCK	D_CLK_TCK
-#endif	/* D_CLK_TCK */
+/* ե饰 */
 
-/**/
+/* GUI ⡼ */
+bool_t	g_flg_gui;
 
-bool_t	g_flg_debug;
+/* CUI ⡼ */
+bool_t	g_flg_cui;
+
+/* Хåե򥯥ꥢ */
 bool_t	g_flg_clr_key_buf;
+
+/* Ϥ줿ɽ */
+bool_t	g_flg_echo_key;
+
+/* ̤ curses 饤֥Ѥ */
+bool_t	g_flg_use_curs;
+
+/* 롦Ѥ */
 bool_t	g_flg_use_keypad;
+
+/* ե졼ࡦ졼Ȥ¬ */
 bool_t	g_flg_measure_clk_tck;
 
+/* ǥХå⡼ */
+bool_t	g_flg_debug;
+
 #endif	/* GMAIN_H */
