@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: misc.h,v 1.4 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: misc.h,v 1.10 2004/06/30 13:20:45 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	MISC_H
@@ -34,13 +33,25 @@
 
 #define	FLG_NULL	0
 
-#define	FIGURE_MAX_N	10
+#define	FIGURE_MAX_N	10	/* κ */
+
+/* ޥå׾δ */
 
 #define	FACE_MJR_NULL	' '
 #define	FACE_MNR_NULL	' '
 #define	FACE_MJR_ERR	'?'
 #define	FACE_MNR_ERR	'?'
 
+/* ̵¥롼ɻѤκ롼׿ */
+
+#define	LOOP_MAX_100	128
+#define	LOOP_MAX_200	256
+#define	LOOP_MAX_1000	1024
+#define	LOOP_MAX_4000	4096
+#define	LOOP_MAX_10000	10240
+
+/* ֡ꥢ */
+
 typedef char	bool_t;
 #ifndef	FALSE
 # define	FALSE	0
@@ -49,20 +60,30 @@ typedef char	bool_t;
 # define	TRUE	1
 #endif	/* TRUE */
 
+/* ɴʬΨ(%) */
+
 typedef long	rate_t;
-#define	_100_PERCENT	100
-#define	MODIFIED_MIN_RATE	1
+#define	_100_PERCENT	((rate_t)100)	/* 100% */
+#define	MODIFIED_MIN_RATE	1	/* ͤκǾ */
+
+/* Ψ(N:M) */
 
 typedef long	ratio_t;
 
+/* ɸ */
+
 typedef struct {
 	long	x, y;
 } pos_t;
 
+/*  */
+
 typedef long	gold_t;
 
-#define	sgn( n )	((n) > 0 ? 1 : ((n) < 0 ? -1 : 0))
-#define	min( n1, n2 )	((n1) < (n2) ? (n1) : (n2))
-#define	max( n1, n2 )	((n1) > (n2) ? (n1) : (n2))
+/* С */
+
+typedef struct {
+	long	mjr, mnr, pat;
+} ver_t;
 
 #endif	/* MISC_H */
