@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: mnstr.c,v 1.90 2003/11/21 18:35:21 bowkenken Exp $
+* $Id: mnstr.c,v 1.147 2004/07/30 16:02:14 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -42,33 +41,37 @@
 #include	"pet.h"
 /*#include	"fight.h"*/
 /*#include	"fx.h"*/
-/*#include	"trap.h"*/
+#include	"trap.h"
 #include	"draw.h"
-/*#include	"curs.h"*/
+#include	"curs.h"
 /*#include	"menu.h"*/
+/*#include	"amenu.h"*/
+/*#include	"request.h"*/
+/*#include	"gfile.h"*/
 /*#include	"msg.h"*/
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-/*#include	"turn_prot.h"*/
-#include	"dun_prot.h"
-/*#include	"town_prot.h"*/
-#include	"item_prot.h"
-#include	"spell_prot.h"
-#include	"chr_prot.h"
-#include	"party_prot.h"
-#include	"mnstr_prot.h"
-#include	"pet_prot.h"
-#include	"fight_prot.h"
-#include	"fx_prot.h"
-/*#include	"trap_prot.h"*/
-#include	"draw_prot.h"
-/*#include	"curs_prot.h"*/
-/*#include	"menu_prot.h"*/
-/*#include	"tmenu_prot.h"*/
-/*#include	"amenu_prot.h"*/
-/*#include	"gfile_prot.h"*/
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+/*#include	"turn-prot.h"*/
+#include	"dun-prot.h"
+/*#include	"town-prot.h"*/
+#include	"item-prot.h"
+#include	"spell-prot.h"
+#include	"chr-prot.h"
+#include	"party-prot.h"
+#include	"mnstr-prot.h"
+#include	"pet-prot.h"
+#include	"fight-prot.h"
+#include	"fx-prot.h"
+#include	"trap-prot.h"
+#include	"draw-prot.h"
+#include	"curs-prot.h"
+/*#include	"menu-prot.h"*/
+/*#include	"tmenu-prot.h"*/
+/*#include	"amenu-prot.h"*/
+/*#include	"request-prot.h"*/
+/*#include	"gfile-prot.h"*/
+/*#include	"msg-prot.h"*/
 
 /**/
 
@@ -77,14 +80,16 @@ static mnstr_t	mnstr_used, mnstr_free;
 
 /**/
 
-#include	"mnstr_tab.h"
+#include	"mnstr-tab.h"
 
 /**/
 
 #define	GARGOYLE_STATUE_N	3
 
-#define	MNSTR_GOLD_MUL_N	1
-#define	MNSTR_GOLD_ADD_N	10
+#define	MNSTR_GOLD_MUL_N	12
+#define	MNSTR_GOLD_ADD_N	0
+
+#define	TOWNER_AVE_DUN_LEV	48
 
 /**/
 
@@ -158,6 +163,8 @@ void	new_mnstr( mnstr_t *mnstr )
 #endif
 	mnstr->kind = CHR_KIND_MNSTR;
 	mnstr->name[0] = '0';
+	mnstr->id1 = 0;
+	mnstr->id2 = 0;
 	mnstr->mbr_n = MBR_N_NOT_JOIN;
 	mnstr->ls_mbr_n = 0;
 	mnstr->face.mjr = FACE_MJR_NULL;
@@ -173,6 +180,8 @@ void	new_mnstr( mnstr_t *mnstr )
 	mnstr->y = MAP_DEL_Y;
 	mnstr->pre_x = MAP_DEL_X;
 	mnstr->pre_y = MAP_DEL_Y;
+	mnstr->dx = 1;
+	mnstr->dy = 1;
 
 	mnstr->move.speed.rate = _100_PERCENT;
 	mnstr->move.speed.rate_org = _100_PERCENT;
@@ -194,10 +203,11 @@ void	new_mnstr( mnstr_t *mnstr )
 	strncpy( mnstr->act.dflt_msg_mnstr,
 			MSG_MENU_MARK_FIGHT, ACT_MSG_MAX_LEN );
 	clr_chr_trgt_act( mnstr, TRUE );
+	mnstr->pre_act = mnstr->act;
 
 	mnstr->flg_chr = FLG_NULL;
 	mnstr->flg_map = FLG_NULL;
-	mnstr->attitude = ATTITUDE_HOSTILE;
+	mnstr->attitude = ATTITUDE_ENEMY;
 	mnstr->owner = NULL;
 	mnstr->stat = FLG_NULL;
 
@@ -230,9 +240,21 @@ void	new_mnstr( mnstr_t *mnstr )
 	mnstr->fx.next = &(mnstr->fx);
 	mnstr->fx.prev = &(mnstr->fx);
 
-	mnstr->fx_data.armor_def = 0;
-	mnstr->fx_data.armor_crtcl = 0;
-	mnstr->fx_data.armor_ac = 0;
+	mnstr->fx_data.wpn_hit = (rate_t)0;
+	mnstr->fx_data.wpn_crtcl = (rate_t)0;
+	mnstr->fx_data.wpn_dam = (rate_t)0;
+
+	mnstr->fx_data.armor_def = (rate_t)0;
+	mnstr->fx_data.armor_crtcl = (rate_t)0;
+	mnstr->fx_data.armor_ac = (rate_t)0;
+
+	mnstr->fx_data.cure_hp.turn = 0;
+	mnstr->fx_data.cure_hp.rate = 0;
+	mnstr->fx_data.cure_hp.n = 0;
+
+	mnstr->fx_data.cure_mp.turn = 0;
+	mnstr->fx_data.cure_mp.rate = 0;
+	mnstr->fx_data.cure_mp.n = 0;
 
 	mnstr->fx_data.infra_vision = 0;
 
@@ -257,9 +279,12 @@ void	new_mnstr( mnstr_t *mnstr )
 	mnstr->work.flg = FLG_NULL;
 	mnstr->work.caught = 0;
 	mnstr->work.catch_chr = NULL;
+	mnstr->work.link_chr = NULL;
+	mnstr->work.request = NULL;
 	mnstr->work.towner_dx = 0;
 	mnstr->work.towner_dy = 0;
 	mnstr->work.enter_door = NULL;
+	mnstr->work.dismissal_turn = 0;
 }
 
 /**/
@@ -268,12 +293,9 @@ mnstr_t	*make_mnstr_rand( long x, long y, long dun_lev )
 {
 	mnstr_tab_t	*tab;
 	mnstr_t	*p;
-	short	group;
+	long	group;
 	long	i, j;
 
-	if( !can_move_chr( NULL, x, y ) )
-		return NULL;
-
 	if( dun_lev == 0 )
 		tab = get_towner_tab_rand();
 	else
@@ -281,22 +303,23 @@ mnstr_t	*make_mnstr_rand( long x, long y, long dun_lev )
 	if( tab == NULL )
 		return NULL;
 
+	if( !can_move_mnstr_tab( tab, x, y ) )
+		return NULL;
+
 	p = make_mnstr( x, y, dun_lev, tab->mnstr_kind );
 
-	group = roll_dice( tab->group );
+	group = roll_dice( tab->group, TRUE );
 	for( i = 0; i < tab->group - 1; i++ ){
 		long	nx, ny;
+		mnstr_t	*pp;
 
-		for( j = 128; j > 0; j-- ){
+		for( j = LOOP_MAX_100; j > 0; j-- ){
 			nx = x + (randm( 10 + 1 + 10 ) - 10);
 			ny = y + (randm( 10 + 1 + 10 ) - 10);
-			if( can_move_chr( NULL, nx, ny ) )
+			pp = make_mnstr( nx, ny, dun_lev, tab->mnstr_kind );
+			if( pp != NULL )
 				break;
 		}
-		if( j <= 0 )
-			continue;
-
-		make_mnstr( nx, ny, dun_lev, tab->mnstr_kind );
 	}
 
 	return p;
@@ -318,7 +341,7 @@ mnstr_t	*make_mnstr(
 	new_mnstr( p );
 
 	if( make_mnstr_alloc( p, x, y, dun_lev, mnstr_kind ) == NULL ){
-		die_chr( p, FALSE );
+		die_chr( p, FALSE, FALSE, TRUE );
 		return NULL;
 	}
 
@@ -334,7 +357,6 @@ mnstr_t	*make_mnstr_alloc(
 	long	i;
 	dun_t	*dun;
 	mnstr_tab_t	*tab;
-	mnstr_lev_tab_t	*lev_tab;
 
 	dun = get_dun();
 
@@ -342,20 +364,14 @@ mnstr_t	*make_mnstr_alloc(
 		return NULL;
 	if( mnstr_kind == MNSTR_KIND_NULL )
 		return NULL;
-	if( clip_pos( x, y ) ){
-		if( dun->map.chr.mjr[y][x] != FACE_MJR_NULL )
-			return NULL;
-		if( !chk_flg( dun->map.obj.flg[y][x], FLG_MAP_OBJ_PASS ) )
-			return NULL;
-	}
 
 	tab = get_mnstr_tab( mnstr_kind, dun_lev );
 	if( tab == NULL )
 		return NULL;
 
-	lev_tab = get_mnstr_lev_tab( mnstr_kind, dun_lev );
-	if( lev_tab == NULL )
-		return NULL;
+	if( clip_pos( x, y ) )
+		if( !can_move_mnstr_tab( tab, x, y ) )
+			return NULL;
 
 	new_mnstr( p );
 
@@ -364,15 +380,22 @@ mnstr_t	*make_mnstr_alloc(
 	p->y = y;
 	p->pre_x = p->x;
 	p->pre_y = p->y;
+	p->dx = tab->dx;
+	p->dy = tab->dy;
 	p->face.mjr = tab->face_mjr;
 	p->face.mnr = tab->face_mnr;
 	p->face.mjr_org = tab->face_mjr;
 	p->face.mnr_org = tab->face_mnr;
+	if( per_randm( 2 ) )
+		p->sex.org = MALE;
+	else
+		p->sex.org = FEMALE;
+	p->sex.cur = p->sex.org;
 	p->move = tab->move;
 	for( i = 0; i < ABL_KIND_MAX_N; i++ ){
 		long	lev;
 
-		lev = calc_mnstr_abl_lev( lev_tab->min_lev,
+		lev = calc_mnstr_abl_lev( dun_lev,
 				tab->abl_lev_rate[i] );
 
 		p->abl.lev[i].n = lev;
@@ -395,11 +418,11 @@ mnstr_t	*make_mnstr_alloc(
 	p->add_def = 0;
 	p->add_crtcl = 0;
 	p->add_ac = 0;
-	p->gold = calc_mnstr_gold( lev_tab->min_lev, tab->gold_rate );
+	p->gold = calc_mnstr_gold( dun_lev, tab->gold_rate );
 	p->flg_chr = tab->flg_chr;
-	p->flg_map = tab->flg_map;
+	p->flg_map = FLG_NULL;
 	p->attitude = tab->attitude;
-	p->stat = FLG_NULL;
+	p->stat = tab->stat;
 	p->trgt.kind = TRGT_KIND_NULL;
 	p->trgt.p = NULL;
 	p->trgt.n = 0;
@@ -422,10 +445,13 @@ mnstr_t	*make_mnstr_alloc(
 
 	/**/
 
-	if( chk_flg( p->flg_map, FLG_MAP_CHR_INVISIBLE ) )
+	if( chk_flg( p->flg_map, FLG_MAP_CHR_INVISIBLE )
+			|| chk_flg( p->stat, FLG_STAT_VANISH ) ){
+		p->flg_map |= FLG_MAP_CHR_INVISIBLE;
 		p->stat |= FLG_STAT_VANISH;
+	}
 
-	if( p->attitude == ATTITUDE_HOSTILE )
+	if( p->attitude == ATTITUDE_ENEMY )
 		p->flg_map &= ~FLG_MAP_CHR_NPC;
 	else
 		p->flg_map |= FLG_MAP_CHR_NPC;
@@ -442,23 +468,40 @@ mnstr_t	*make_mnstr_alloc(
 
 bool_t	make_mnstr_special( long dun_lev, mnstr_t *p )
 {
-	long	i;
+	bool_t	ret = TRUE;
+	static long	link_n = 0;
+	mnstr_t	*p2;
 	long	n;
 	item_t	*chest;
+	long	i;
 
 	if( p == NULL )
 		return FALSE;
 
+	link_n++;
+
 	switch( p->mnstr_kind ){
 	default:
 		break;
+	case MNSTR_KIND_GIGANTIC_CENTIPEDE:
+		if( link_n >= GIGANTIC_CENTIPEDE_LINK_N )
+			break;
+
+		p2 = make_mnstr( p->x, p->y + 1, dun_lev, p->mnstr_kind );
+		if( p2 == NULL )
+			break;
+		mark_mnstr( p2, p, DIST_NEAR );
+		p2->work.link_chr = p;
+		break;
 	case MNSTR_KIND_GARGOYLE:
 		n = 0;
-		if( !set_statue( p->x, p->y, dun_lev, TRUE ) )
-			return FALSE;
+		if( !set_statue( p->x, p->y, dun_lev, TRUE ) ){
+			ret = FALSE;
+			break;
+		}
 		n++;
 
-		for( i = 0; i < 1024; i++ ){
+		for( i = 0; i < LOOP_MAX_1000; i++ ){
 			long	x, y;
 
 			if( n >= GARGOYLE_STATUE_N )
@@ -473,27 +516,52 @@ bool_t	make_mnstr_special( long dun_lev, mnstr_t *p )
 		}
 		break;
 	case MNSTR_KIND_MIMIC:
-		if( get_item( p->x, p->y ) != NULL )
-			return FALSE;
+		if( get_item( p->x, p->y ) != NULL ){
+			ret = FALSE;
+			break;
+		}
 
-		chest = make_item_chest( dun_lev, get_chest_n( dun_lev ) );
+		chest = make_item_chest( dun_lev,
+				get_chest_n( dun_lev ), TRUE );
 
-		if( chest == NULL )
-			return FALSE;
-		if( !put_item( p->x, p->y, 1, chest, FALSE ) ){
+		if( chest == NULL ){
+			ret = FALSE;
+			break;
+		}
+		if( !put_item( chest, p->x, p->y, 1, FALSE ) ){
 			del_item( chest );
-			return FALSE;
+			ret = FALSE;
+			break;
 		}
 
 		break;
 	}
 
-	return TRUE;
+	link_n--;
+
+	return ret;
 }
 
 /**/
 
-void	act_all_mnstr( void )
+mnstr_t	*make_npc_rand( long x, long y, long dun_lev )
+{
+	mnstr_kind_t	kind;
+
+	kind = get_npc_kind_randm();
+
+	if( dun_lev == 0 )
+		dun_lev = TOWNER_AVE_DUN_LEV;
+	dun_lev = roll_dice( dun_lev, FALSE );
+	if( dun_lev < 1 )
+		dun_lev = 1;
+
+	return make_mnstr( x, y, dun_lev, kind );
+}
+
+/**/
+
+void	move_phase_all_mnstr( void )
 {
 	mnstr_t	*head, *p, *pp;
 
@@ -504,7 +572,23 @@ void	act_all_mnstr( void )
 			break;
 
 		set_act_mnstr( pp );
-		act_chr( pp );
+		move_phase_chr( pp );
+	}
+}
+
+/**/
+
+void	act_phase_all_mnstr( void )
+{
+	mnstr_t	*head, *p, *pp;
+
+	head = &mnstr_used;
+	for( p = head->next->next; p != NULL; p = p->next ){
+		pp = p->prev;
+		if( pp == head )
+			break;
+
+		act_phase_chr( pp );
 		set_modifier( pp );
 	}
 }
@@ -530,27 +614,42 @@ bool_t	chk_can_act_mnstr( mnstr_t *p )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( (abs( p->x - party->mbr[i]->x )
-				<= MNSTR_MOVE_MAX_X)
-				&& (abs( p->y - party->mbr[i]->y )
-				<= MNSTR_MOVE_MAX_Y) ){
-			break;
-		}
+		if( labs( p->x - party->mbr[i]->x ) > MNSTR_MOVE_MAX_X )
+			continue;
+		if( labs( p->y - party->mbr[i]->y ) > MNSTR_MOVE_MAX_Y )
+			continue;
+
+		break;
 	}
 	if( i >= MBR_MAX_N ){	/* СΥƤ */
+		long	bx, by;
+		long	ex, ey;
+		long	x, y;
+
 		p->flg_map &= ~FLG_MAP_CHR_FIND;
-		dun->map.chr.flg[p->y][p->x] = p->flg_map;
+
+		bx = get_chr_left( p->x, p->dx );
+		by = get_chr_top( p->y, p->dy );
+		ex = get_chr_right( p->x, p->dx );
+		ey = get_chr_bottom( p->y, p->dy );
+		for( y = by; y <= ey; y++ ){
+			for( x = bx; x <= ex; x++ ){
+				if( !clip_pos( x, y ) )
+					continue;
+
+				dun->map.chr.flg[y][x] = p->flg_map;
+			}
+		}
+
 		draw_mnstr( p );
 
 		return FALSE;
 	}
 
 	if( chk_flg_or( p->stat,
-			FLG_STAT_NOT_EXIST
-			| FLG_STAT_DEAD
-			| FLG_STAT_PARALYZE
-			| FLG_STAT_SLEEP
-			| FLG_STAT_FAINT ) ){
+			FLG_STAT_NOT_EXIST | FLG_STAT_DEAD
+			| FLG_STAT_STONE | FLG_STAT_PARALYZE
+			| FLG_STAT_SLEEP | FLG_STAT_FAINT ) ){
 		return FALSE;
 	}
 
@@ -602,9 +701,6 @@ mbr_t	*mnstr_mark_mbr_common( mnstr_t *p, sex_t sex )
 
 	clr_chr_trgt( p );
 
-	if( p->attitude != ATTITUDE_HOSTILE )
-		return NULL;
-
 	trgt_kind = TRGT_KIND_NULL;
 	mark_p = NULL;
 	r = MAP_MAX_X + MAP_MAX_Y;
@@ -612,13 +708,12 @@ mbr_t	*mnstr_mark_mbr_common( mnstr_t *p, sex_t sex )
 		long	tmp_r;
 
 		if( chk_flg_or( pty->mbr[i]->stat,
-				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
+				FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD
+				| FLG_STAT_VANISH ) ){
 			continue;
 		}
 
-		if( chk_flg( pty->mbr[i]->stat, FLG_STAT_VANISH ) )
-			continue;
-
 		if( !chk_flg( p->work.flg, FLG_WORK_FIND_MBR ) ){
 			if( !chk_find_chr( p, pty->mbr[i] ) )
 				continue;
@@ -628,8 +723,8 @@ mbr_t	*mnstr_mark_mbr_common( mnstr_t *p, sex_t sex )
 			if( sex != pty->mbr[i]->sex.cur )
 				continue;
 
-		tmp_r = abs( pty->mbr[i]->x - p->x )
-				+ abs( pty->mbr[i]->y - p->y );
+		tmp_r = labs( pty->mbr[i]->x - p->x )
+				+ labs( pty->mbr[i]->y - p->y );
 		if( tmp_r < r ){
 			trgt_kind = TRGT_KIND_MBR;
 			mark_p = pty->mbr[i];
@@ -658,8 +753,8 @@ mbr_t	*mnstr_mark_mbr_common( mnstr_t *p, sex_t sex )
 		if( sex != SEX_NULL )
 			continue;
 
-		tmp_r = abs( pty_pet[i]->x - p->x )
-				+ abs( pty_pet[i]->y - p->y );
+		tmp_r = labs( pty_pet[i]->x - p->x )
+				+ labs( pty_pet[i]->y - p->y );
 		if( tmp_r < r ){
 			trgt_kind = TRGT_KIND_MNSTR;
 			mark_p = pty_pet[i];
@@ -701,19 +796,24 @@ mbr_t	*mnstr_mark_mbr_adjoin( mnstr_t *p )
 	if( p == NULL )
 		return NULL;
 
-	if( p->attitude != ATTITUDE_HOSTILE )
+	if( p->attitude != ATTITUDE_ENEMY )
+		return NULL;
+	if( chk_flg( p->stat, FLG_STAT_CHARM ) )
 		return NULL;
 
 	trgt_kind = TRGT_KIND_NULL;
 	mark_p = NULL;
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		if( chk_flg_or( pty->mbr[i]->stat,
-				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
+				FLG_STAT_NOT_EXIST
+				| FLG_STAT_DEAD
+				| FLG_STAT_VANISH ) ){
 			continue;
 		}
-		if( abs( p->x - pty->mbr[i]->x ) > 1 )
+
+		if( labs( p->x - pty->mbr[i]->x ) > 1 )
 			continue;
-		if( abs( p->y - pty->mbr[i]->y ) > 1 )
+		if( labs( p->y - pty->mbr[i]->y ) > 1 )
 			continue;
 
 		trgt_kind = TRGT_KIND_MBR;
@@ -730,9 +830,9 @@ mbr_t	*mnstr_mark_mbr_adjoin( mnstr_t *p )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( abs( p->x - pty_pet[i]->x ) > 1 )
+		if( labs( p->x - pty_pet[i]->x ) > 1 )
 			continue;
-		if( abs( p->y - pty_pet[i]->y ) > 1 )
+		if( labs( p->y - pty_pet[i]->y ) > 1 )
 			continue;
 
 		trgt_kind = TRGT_KIND_MNSTR;
@@ -755,8 +855,11 @@ mbr_t	*mnstr_mark_mbr_adjoin( mnstr_t *p )
 		p->flg_map &= ~FLG_MAP_CHR_INVISIBLE;
 		p->stat &= ~FLG_STAT_VANISH;
 		draw_mnstr( p );
+
+		curs_attrset_chr( mark_p, -1, FALSE );
 		print_msg( FLG_NULL, MSG_MBR_FIND_INVISIBLE,
 				p->name );
+		curs_attrset_dflt();
 
 		item = get_item( p->x, p->y );
 		if( item != NULL )
@@ -778,6 +881,7 @@ mbr_t	*mnstr_mark_mbr_adjoin( mnstr_t *p )
 
 bool_t	mnstr_move_or_atack_std( mnstr_t *p )
 {
+	chr_t	*trgt;
 	attack_t	*ls;
 	ratio_t	sum_bash, sum_throw, sum_spell, n;
 
@@ -785,7 +889,14 @@ bool_t	mnstr_move_or_atack_std( mnstr_t *p )
 		return FALSE;
 	if( p->mnstr_tab == NULL )
 		return FALSE;
+	if( chk_flg( p->stat, FLG_STAT_CHARM ) )
+		return FALSE;
+	if( (p->trgt.kind != TRGT_KIND_MBR)
+			&& (p->trgt.kind != TRGT_KIND_MNSTR) ){
+		return FALSE;
+	}
 
+	trgt = (chr_t *)(p->trgt.p);
 	ls = p->mnstr_tab->attack;
 
 	sum_bash = calc_attack_sum( ls, TRUE, TRUE, FALSE, FALSE );
@@ -796,21 +907,52 @@ bool_t	mnstr_move_or_atack_std( mnstr_t *p )
 
 	if( n < sum_bash ){
 		set_chr_act( p, ACT_KIND_MBR_FIGHT,
-				(chr_t *)(p->trgt.p), NULL, 0, 0 );
+				trgt, NULL, 0, 0 );
 	} else if( n < sum_bash + sum_throw ){
 		set_chr_act( p, ACT_KIND_MBR_THROW,
-				(chr_t *)(p->trgt.p), NULL, 0, 0 );
+				trgt, NULL, 0, 0 );
 	} else {
 		attack_t	*atk;
+		long	d;
+		chr_t	*friend;
 
-		atk = get_attack( FALSE, ls,
+		d = max_l( labs( p->x - trgt->x ), labs( p->y - trgt->y ) );
+		atk = get_attack( FALSE, ls, d,
 				FALSE, FALSE, FALSE, TRUE );
 		if( atk == NULL )
 			return FALSE;
-		set_chr_act( p, ACT_KIND_SPELL,
-				get_spell_tab( atk->spell ),
-				MSG( atk->spell_fmt ),
-				atk->extent, 0 );
+
+		if( atk->kind == ATTACK_KIND_SPELL_SELF )
+			friend = p;
+		else if( atk->kind == ATTACK_KIND_SPELL_FRIEND )
+			friend = get_mnstr_randm( p->x, p->y );
+		else
+			friend = NULL;
+
+		if( friend != NULL ){
+			rate_t	rate;
+
+			trgt = friend;
+			rate = friend->abl.hp.n * _100_PERCENT
+					/ friend->abl.hp.max;
+			if( rate < 70 ){
+				mark_mnstr( p, friend, DIST_FAR );
+			} else {
+				set_chr_act( p, ACT_KIND_MBR_FIGHT,
+						trgt, NULL, 0, 0 );
+				return TRUE;
+			}
+		}
+
+		if( chk_find_chr( p, trgt ) ){
+			set_chr_act( p, ACT_KIND_SPELL,
+					get_spell_tab( atk->spell ),
+					MSG( atk->spell_fmt ),
+					atk->extent, 0 );
+		} else {
+			set_chr_act( p, ACT_KIND_MBR_FIGHT,
+					trgt, NULL, 0, 0 );
+		}
 	}
 
 	return TRUE;
@@ -818,18 +960,34 @@ bool_t	mnstr_move_or_atack_std( mnstr_t *p )
 
 /**/
 
-void	set_attitude_mnstr_group( char face_mjr, attitude_t att )
+void	set_enemy_mnstr_group( chr_t *o_p, chr_t *d_p )
 {
 	mnstr_t	*head, *p, *pp;
 
+	if( o_p == NULL )
+		return;
+	if( d_p == NULL )
+		return;
+
+	if( o_p->kind != CHR_KIND_MBR )
+		return;
+	if( d_p->kind == CHR_KIND_MBR )
+		return;
+	if( d_p->attitude == ATTITUDE_MATE )
+		return;
+	if( d_p->attitude == ATTITUDE_ENEMY )
+		return;
+
 	head = &mnstr_used;
 	for( p = head->next->next; p != NULL; p = p->next ){
 		pp = p->prev;
 		if( pp == head )
 			break;
 
-		if( pp->face.mjr == face_mjr )
-			pp->attitude = att;
+		if( pp->face.mjr == d_p->face.mjr ){
+			pp->attitude = ATTITUDE_ENEMY;
+			draw_mnstr( pp );
+		}
 	}
 }
 
@@ -856,28 +1014,24 @@ void	inc_turn_all_mnstr( void )
 bool_t	increase_mnstr( mnstr_t *p )
 {
 	long	nx, ny;
+	dun_t	*dun;
+	mnstr_t	*new_mnstr;
 
 	if( p == NULL )
 		return FALSE;
 
+	dun = get_dun();
+
 	nx = p->x + (randm( 3 ) - 1);
 	ny = p->y + (randm( 3 ) - 1);
-	if( can_move_chr( NULL, nx, ny ) ){
-		dun_t	*dun;
-		mnstr_t	*new_mnstr;
 
-		dun = get_dun();
+	new_mnstr = make_mnstr( nx, ny, dun->lev, p->mnstr_kind );
+	if( new_mnstr == NULL )
+		return FALSE;
 
-		new_mnstr = make_mnstr( nx, ny, dun->lev, p->mnstr_kind );
-		if( new_mnstr == NULL )
-			return FALSE;
+	new_mnstr->attitude = p->attitude;
 
-		new_mnstr->attitude = p->attitude;
-
-		return TRUE;
-	}
-
-	return FALSE;
+	return TRUE;
 }
 
 /**/
@@ -973,6 +1127,9 @@ bool_t	set_flg_find_mnstr( mnstr_t *p )
 	long	i;
 	dun_t	*dun;
 	party_t	*pty;
+	long	bx, by;
+	long	ex, ey;
+	long	x, y;
 
 	dun = get_dun();
 	pty = get_party();
@@ -983,44 +1140,43 @@ bool_t	set_flg_find_mnstr( mnstr_t *p )
 		return FALSE;
 
 	p->flg_map &= ~FLG_MAP_CHR_FIND;
-	dun->map.chr.flg[p->y][p->x] = p->flg_map;
-
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		if( chk_find_chr( pty->mbr[i], p ) ){
 			p->flg_map |= FLG_MAP_CHR_FIND;
-			dun->map.chr.flg[p->y][p->x] = p->flg_map;
-			return TRUE;
+			break;
 		}
 	}
 
-	return FALSE;
+	bx = get_chr_left( p->x, p->dx );
+	by = get_chr_top( p->y, p->dy );
+	ex = get_chr_right( p->x, p->dx );
+	ey = get_chr_bottom( p->y, p->dy );
+	for( y = by; y <= ey; y++ ){
+		for( x = bx; x <= ex; x++ ){
+			if( !clip_pos( x, y ) )
+				continue;
+
+				dun->map.chr.flg[y][x] = p->flg_map;
+		}
+	}
+
+	return( chk_flg( p->flg_map, FLG_MAP_CHR_FIND ) );
 }
 
 /**/
 
-long	add_hp_mnstr( mnstr_t *mnstr, long n )
+void	chk_trap_all_mnstr( void )
 {
-	long	pre_hp, add;
+	mnstr_t	*head, *p, *pp;
 
-	if( mnstr == NULL )
-		return 0;
+	head = &mnstr_used;
+	for( p = head->next->next; p != NULL; p = p->next ){
+		pp = p->prev;
+		if( pp == head )
+			break;
 
-	pre_hp = mnstr->abl.hp.n;
-
-	mnstr->abl.hp.n += n;
-	if( mnstr->abl.hp.n > mnstr->abl.hp.max )
-		mnstr->abl.hp.n = mnstr->abl.hp.max;
-	if( mnstr->abl.hp.n < 0 )
-		mnstr->abl.hp.n = 0;
-
-	add = mnstr->abl.hp.n - pre_hp;
-	if( add != 0 )
-		vfx_num( mnstr, add );
-
-	if( mnstr->abl.hp.n <= 0 )
-		die_chr( mnstr, TRUE );
-
-	return mnstr->abl.hp.n;
+		chk_trap( pp, pp );
+	}
 }
 
 /**/
@@ -1030,38 +1186,89 @@ void	die_mnstr( mnstr_t *p )
 	if( p == NULL )
 		return;
 
-	do {
-		long	n;
-		item_t	*coin;
+	if( is_pet( p ) ){
+		die_pet( p );
+		return;
+	}
 
-		if( p->gold <= 0 )
-			break;
-
-		n = get_coin_tab_n( get_coin_tab( COIN_KIND_GOLD ) );
-		if( n < 0 )
-			break;
-
-		coin = make_item_coin( 1, n );
-		if( coin == NULL )
-			break;
-		coin->n = p->gold;
-		set_item_name( coin );
-
-		if( !put_item( p->x, p->y, PUT_ITEM_R, coin, FALSE ) )
-			del_item( coin );
-	} while( 0 );
-
-	release_chr( p );
+	mnstr_drop_gold( p );
+	mnstr_drop_item( p );
 
 	ins_ls_mnstr( &mnstr_free, p );
 }
 
 /**/
 
+void	mnstr_drop_gold( mnstr_t *p )
+{
+	long	n;
+	item_t	*coin;
+
+	if( p == NULL )
+		return;
+
+	if( p->gold <= 0 )
+		return;
+
+	n = get_coin_tab_n( get_coin_tab( COIN_KIND_GOLD ) );
+	if( n < 0 )
+		return;
+
+	coin = make_item_coin( 1, n );
+	if( coin == NULL )
+		return;
+	coin->n = p->gold;
+	set_item_name( coin );
+
+	if( !put_item( coin, p->x, p->y, PUT_ITEM_R, FALSE ) )
+		del_item( coin );
+}
+
+/**/
+
+void	mnstr_drop_item( mnstr_t *p )
+{
+	dun_t	*dun = get_dun();
+	long	i, j;
+
+	if( p == NULL )
+		return;
+	if( p->mnstr_tab == NULL )
+		return;
+
+	for( i = 0; i < p->mnstr_tab->item_max_n; i++ )
+		make_item( p->x, p->y, dun->lev );
+
+	for( i = 0; i < MNSTR_ITEM_N_MAX_N; i++ ){
+		mnstr_item_n_t	*a;
+
+		a = &(p->mnstr_tab->item_n[i]);
+		if( a->kind == ITEM_KIND_NULL )
+			break;
+
+		for( j = 0; j < a->n; j++ ){
+			item_t	*item;
+
+			item = make_item_sel( a->kind,
+					a->k1, a->k2, a->k3, dun->lev );
+			if( item == NULL )
+				continue;
+
+			if( !put_item( item, p->x, p->y, PUT_ITEM_R, FALSE ) )
+				del_item( item );
+		}
+	}
+}
+
+/**/
+
 void	set_find_all_mnstr( void )
 {
 	mnstr_t	*head, *p, *pp;
 	dun_t	*dun;
+	long	bx, by;
+	long	ex, ey;
+	long	x, y;
 
 	dun = get_dun();
 
@@ -1073,7 +1280,19 @@ void	set_find_all_mnstr( void )
 
 		pp->flg_map |= FLG_MAP_CHR_FIND;
 		pp->flg_map &= ~FLG_MAP_CHR_INVISIBLE;
-		dun->map.chr.flg[pp->y][pp->x] = pp->flg_map;
+
+		bx = get_chr_left( pp->x, pp->dx );
+		by = get_chr_top( pp->y, pp->dy );
+		ex = get_chr_right( pp->x, pp->dx );
+		ey = get_chr_bottom( pp->y, pp->dy );
+		for( y = by; y <= ey; y++ ){
+			for( x = bx; x <= ex; x++ ){
+				if( !clip_pos( x, y ) )
+					continue;
+
+				dun->map.chr.flg[y][x] = pp->flg_map;
+			}
+		}
 	}
 }
 
@@ -1085,15 +1304,23 @@ void	print_ls_mnstr( void )
 	long	max_dun_lev_base, min_lev, max_lev;
 	long	i, max_i;
 
+	/* Сߤꤿֿ */
+
 	max_dun_lev_base = 0;
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		long	lev;
 
-		lev = abs( pty->mbr[i]->dun_max_lev_base );
+		lev = labs( pty->mbr[i]->dun_max_lev_base );
 		if( max_dun_lev_base < lev )
 			max_dun_lev_base = lev;
 	}
 
+	/* ߤγΤʤˤ */
+
+	max_dun_lev_base -= 1;
+
+	/* 󥹥ΥꥹȤʤ */
+
 	min_lev = 0;
 	max_i = sizeof( mnstr_lev_tab ) / sizeof( mnstr_lev_tab[0] );
 	for( i = 0; max_i; i++ ){
@@ -1102,16 +1329,24 @@ void	print_ls_mnstr( void )
 		if( mnstr_lev_tab[i].kind == MNSTR_KIND_NULL )
 			break;
 
+		/* ؤͤͤѴ */
+
 		min_lev += mnstr_lev_tab[i].add_lev;
 		max_lev = min_lev + mnstr_lev_tab[i].range_lev - 1;
 
+		/* ؤϰϥå */
+
+		if( min_lev > max_dun_lev_base )
+			continue;
+		if( max_lev > max_dun_lev_base )
+			max_lev = max_dun_lev_base;
+
+		/* 󥹥Υơ֥ */
+
 		tab = get_mnstr_tab( mnstr_lev_tab[i].kind, min_lev );
 		if( tab == NULL )
 			continue;
 
-		if( max_dun_lev_base < min_lev )
-			continue;
-
 		print_msg( FLG_NULL, MSG_LS_MNSTR,
 				min_lev, max_lev, tab->name );
 	}
@@ -1119,6 +1354,26 @@ void	print_ls_mnstr( void )
 
 /**/
 
+mnstr_kind_t	get_npc_kind_randm( void )
+{
+	long	n;
+	long	i, max_n;
+
+	max_n = sizeof( g_ls_npc_kind ) / sizeof( g_ls_npc_kind[0] );
+	for( i = 0; i < max_n; i++ ){
+		if( g_ls_npc_kind[i] == MNSTR_KIND_NULL )
+			break;
+	}
+	n = i;
+	if( n <= 0 )
+		n = 1;
+	n = randm( n );
+
+	return g_ls_npc_kind[n];
+}
+
+/**/
+
 char	*get_mnstr_name( mnstr_kind_t mnstr_kind )
 {
 	mnstr_tab_t	*tab;
@@ -1135,25 +1390,17 @@ char	*get_mnstr_name( mnstr_kind_t mnstr_kind )
 mnstr_tab_t	*get_mnstr_tab( mnstr_kind_t mnstr_kind, long dun_lev )
 {
 	mnstr_tab_t	*tab;
-	long	n;
+	long	i, n;
 
 	tab = NULL;
-	for( n = 0; mnstr_tab[n].mnstr_kind != MNSTR_KIND_NULL; n++ ){
-		mnstr_lev_tab_t	*lev_tab;
-
-		if( mnstr_tab[n].mnstr_kind != mnstr_kind )
+	n = 0;
+	for( i = 0; mnstr_tab[i].mnstr_kind != MNSTR_KIND_NULL; i++ ){
+		if( mnstr_tab[i].mnstr_kind != mnstr_kind )
 			continue;
 
-		tab = &(mnstr_tab[n]);
-
-		lev_tab = get_mnstr_lev_tab( mnstr_kind, dun_lev );
-		if( lev_tab != NULL ){
-			if( lev_tab->min_lev <= abs( dun_lev ) ){
-				if( abs( dun_lev ) <= lev_tab->max_lev ){
-					break;
-				}
-			}
-		}
+		n++;
+		if( per_randm( n ) )
+			tab = &(mnstr_tab[i]);
 	}
 
 	return tab;
@@ -1177,8 +1424,8 @@ mnstr_tab_t	*get_mnstr_tab_rand( long dun_lev )
 		if( lev_p->kind == MNSTR_KIND_NULL )
 			break;
 
-		if( lev_p->min_lev <= abs( dun_lev ) ){
-			if( abs( dun_lev ) <= lev_p->max_lev ){
+		if( lev_p->min_lev <= labs( dun_lev ) ){
+			if( labs( dun_lev ) <= lev_p->max_lev ){
 				n++;
 				if( per_randm( n ) ){
 					kind = lev_p->kind;
@@ -1192,6 +1439,27 @@ mnstr_tab_t	*get_mnstr_tab_rand( long dun_lev )
 
 /**/
 
+mnstr_tab_t	*get_mnstr_tab_friendly( void )
+{
+	mnstr_tab_t	*tab;
+	long	i, n;
+
+	tab = NULL;
+	n = 0;
+	for( i = 0; mnstr_tab[i].mnstr_kind != MNSTR_KIND_NULL; i++ ){
+		if( mnstr_tab[i].attitude != ATTITUDE_FRIENDLY )
+			continue;
+
+		n++;
+		if( per_randm( n ) )
+			tab = &(mnstr_tab[i]);
+	}
+
+	return tab;
+}
+
+/**/
+
 mnstr_tab_t	*get_towner_tab_rand( void )
 {
 	long	n, sum;
@@ -1230,24 +1498,34 @@ mnstr_tab_t	*get_towner_tab_rand( void )
 mnstr_lev_tab_t	*get_mnstr_lev_tab( mnstr_kind_t mnstr_kind, long dun_lev )
 {
 	mnstr_lev_tab_t	*tab;
-	long	i, max_i;
+	long	i, max_i, n;
 
 	tab = NULL;
+	n = 0;
 	max_i = sizeof( mnstr_lev_tab ) / sizeof( mnstr_lev_tab[0] );
 	for( i = 0; max_i; i++ ){
 		mnstr_lev_tab_t	*p;
 
 		p = &(mnstr_lev_tab[i]);
-
 		if( p->kind == MNSTR_KIND_NULL )
 			break;
 
-		if( p->min_lev <= abs( dun_lev ) ){
-			if( abs( dun_lev ) <= p->max_lev ){
-				tab = p;
-				break;
-			}
-		}
+		if( p->kind != mnstr_kind )
+			continue;
+
+		/* 󥸥٥˥ޥåʤäΰ */
+		n++;
+		if( per_randm( n ) )
+			tab = p;
+
+		if( p->min_lev > labs( dun_lev ) )
+			continue;
+		if( p->max_lev < labs( dun_lev ) )
+			continue;
+
+		/* 󥸥٥˥ޥå */
+		tab = p;
+		break;
 	}
 
 	return tab;
@@ -1323,7 +1601,7 @@ void	exp_chr_mnstr( chr_t *chr, long exp, bool_t flg_print_lev )
 
 	chr->abl.exp[left_k].n += left_exp;
 
-	chk_up_lev( chr, flg_print_lev );
+	chk_lev_up( chr, flg_print_lev );
 }
 
 /**/
@@ -1333,12 +1611,7 @@ long	calc_mnstr_exp( mnstr_t *p, long dun_lev )
 	long	lev;
 	long	exp;
 
-	if( p == NULL )
-		return 1;
-	if( p->mnstr_tab == NULL )
-		return 1;
-
-	lev = abs( dun_lev ) * RATE_LEV_PER_DUN_LEV / _100_PERCENT;
+	lev = labs( dun_lev ) * RATE_LEV_PER_DUN_LEV / _100_PERCENT;
 
 	exp = calc_need_exp( lev, ABL_KIND_HP );
 	exp -= calc_need_exp( lev - 1, ABL_KIND_HP );
@@ -1359,7 +1632,7 @@ long	calc_mnstr_abl_lev( long dun_lev, rate_t rate )
 {
 	long	lev;
 
-	lev = abs( dun_lev ) * RATE_LEV_PER_DUN_LEV / _100_PERCENT;
+	lev = labs( dun_lev ) * RATE_LEV_PER_DUN_LEV / _100_PERCENT;
 	lev = lev * rate / _100_PERCENT;
 
 	if( lev < 0 )
@@ -1377,7 +1650,7 @@ gold_t	calc_mnstr_gold( long dun_lev, rate_t rate )
 {
 	gold_t	gold;
 
-	gold = abs( dun_lev * dun_lev );
+	gold = labs( dun_lev );
 	gold *= MNSTR_GOLD_MUL_N;
 	gold += MNSTR_GOLD_ADD_N;
 	gold = gold * rate / _100_PERCENT;
@@ -1387,6 +1660,16 @@ gold_t	calc_mnstr_gold( long dun_lev, rate_t rate )
 
 /**/
 
+bool_t	can_move_mnstr_tab( mnstr_tab_t *tab, long x, long y )
+{
+	if( tab == NULL )
+		return FALSE;
+
+	return can_move_chr_pos( NULL, x, y, tab->dx, tab->dy );
+}
+
+/**/
+
 mnstr_t	*get_mnstr( long x, long y )
 {
 	mnstr_t	*head, *p, *pp;
@@ -1402,8 +1685,17 @@ mnstr_t	*get_mnstr( long x, long y )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( (pp->x == x) && (pp->y == y) )
-			return pp;
+
+		if( x < get_chr_left( pp->x, pp->dx ) )
+			continue;
+		if( x > get_chr_right( pp->x, pp->dx ) )
+			continue;
+		if( y < get_chr_top( pp->y, pp->dy ) )
+			continue;
+		if( y > get_chr_bottom( pp->y, pp->dy ) )
+			continue;
+
+		return pp;
 	}
 
 	pet = get_pet( x, y );
@@ -1437,10 +1729,10 @@ mnstr_t	*get_mnstr_nearest( long x, long y )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( pp->attitude != ATTITUDE_HOSTILE )
+		if( pp->attitude != ATTITUDE_ENEMY )
 			continue;
 
-		tmp_r = abs( pp->x - x ) + abs( pp->y - y );
+		tmp_r = labs( pp->x - x ) + labs( pp->y - y );
 		if( tmp_r < min_r ){
 			n = 0;
 			min_r = tmp_r;
@@ -1481,10 +1773,10 @@ mnstr_t	*get_mnstr_near( long x, long y )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( pp->attitude != ATTITUDE_HOSTILE )
+		if( pp->attitude != ATTITUDE_ENEMY )
 			continue;
 
-		tmp_r = abs( pp->x - x ) + abs( pp->y - y );
+		tmp_r = labs( pp->x - x ) + labs( pp->y - y );
 		if( tmp_r < min_r ){
 			n = 0;
 			min_r = tmp_r;
@@ -1529,10 +1821,10 @@ mnstr_t	*get_mnstr_disperse( mbr_t *mbr )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( pp->attitude != ATTITUDE_HOSTILE )
+		if( pp->attitude != ATTITUDE_ENEMY )
 			continue;
 
-		tmp_r = abs( pp->x - x ) + abs( pp->y - y );
+		tmp_r = labs( pp->x - x ) + labs( pp->y - y );
 		if( tmp_r < min_r ){
 			if( already_mark_mnstr( pp, mbr ) )
 				continue;
@@ -1575,7 +1867,7 @@ mnstr_t	*get_mnstr_randm( long x, long y )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( pp->attitude != ATTITUDE_HOSTILE )
+		if( pp->attitude != ATTITUDE_ENEMY )
 			continue;
 
 		n++;
@@ -1608,7 +1900,7 @@ bool_t	already_mark_mnstr( mnstr_t *mnstr, mbr_t *mbr )
 
 /**/
 
-mnstr_t	*get_mnstr_rand( long x, long y )
+mnstr_t	*get_mnstr_randm2( long x, long y )
 {
 	mnstr_t	*head, *p, *pp;
 	long	n, max_n, max_n2;
@@ -1630,7 +1922,7 @@ mnstr_t	*get_mnstr_rand( long x, long y )
 
 		max_n++;
 
-		if( (abs( pp->x - x ) < 1) && (abs( pp->y - y ) < 1) )
+		if( (labs( pp->x - x ) < 1) && (labs( pp->y - y ) < 1) )
 			max_n2++;
 	}
 	if( max_n <= 0 )
@@ -1672,8 +1964,8 @@ mnstr_t	*get_mnstr_rand( long x, long y )
 				continue;
 			}
 
-			if( (abs( pp->x - x ) < 1)
-					&& (abs( pp->y - y ) < 1) ){
+			if( (labs( pp->x - x ) < 1)
+					&& (labs( pp->y - y ) < 1) ){
 				n--;
 				if( n == 0 )
 					return pp;
@@ -1686,6 +1978,30 @@ mnstr_t	*get_mnstr_rand( long x, long y )
 
 /**/
 
+mnstr_kind_t	get_mnstr_kind_from_face( char mjr, char mnr )
+{
+	long	i;
+	long	n;
+	mnstr_kind_t	k;
+
+	n = 0;
+	k = MNSTR_KIND_NULL;
+	for( i = 0; mnstr_tab[i].mnstr_kind != MNSTR_KIND_NULL; i++ ){
+		if( mnstr_tab[i].face_mjr != mjr )
+			continue;
+		if( mnstr_tab[i].face_mnr != mnr )
+			continue;
+
+		n++;
+		if( per_randm( n ) )
+			k = mnstr_tab[i].mnstr_kind;
+	}
+
+	return k;
+}
+
+/**/
+
 mnstr_t	*get_mnstr_used( void )
 {
 	return &mnstr_used;
