@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: msg.c,v 1.10 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: msg.c,v 1.19 2004/07/29 07:05:00 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -46,29 +45,33 @@
 #include	"draw.h"
 /*#include	"curs.h"*/
 /*#include	"menu.h"*/
+/*#include	"amenu.h"*/
+/*#include	"request.h"*/
+/*#include	"gfile.h"*/
 #include	"msg.h"
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-/*#include	"misc_prot.h"*/
-/*#include	"turn_prot.h"*/
-/*#include	"dun_prot.h"*/
-/*#include	"town_prot.h"*/
-/*#include	"item_prot.h"*/
-/*#include	"spell_prot.h"*/
-/*#include	"chr_prot.h"*/
-/*#include	"party_prot.h"*/
-/*#include	"mnstr_prot.h"*/
-/*#include	"pet_prot.h"*/
-/*#include	"fight_prot.h"*/
-/*#include	"fx_prot.h"*/
-/*#include	"trap_prot.h"*/
-#include	"draw_prot.h"
-/*#include	"curs_prot.h"*/
-/*#include	"menu_prot.h"*/
-/*#include	"tmenu_prot.h"*/
-/*#include	"amenu_prot.h"*/
-/*#include	"gfile_prot.h"*/
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+/*#include	"misc-prot.h"*/
+/*#include	"turn-prot.h"*/
+/*#include	"dun-prot.h"*/
+/*#include	"town-prot.h"*/
+/*#include	"item-prot.h"*/
+/*#include	"spell-prot.h"*/
+/*#include	"chr-prot.h"*/
+/*#include	"party-prot.h"*/
+/*#include	"mnstr-prot.h"*/
+/*#include	"pet-prot.h"*/
+/*#include	"fight-prot.h"*/
+/*#include	"fx-prot.h"*/
+/*#include	"trap-prot.h"*/
+#include	"draw-prot.h"
+/*#include	"curs-prot.h"*/
+/*#include	"menu-prot.h"*/
+/*#include	"tmenu-prot.h"*/
+/*#include	"amenu-prot.h"*/
+/*#include	"request-prot.h"*/
+/*#include	"gfile-prot.h"*/
+/*#include	"msg-prot.h"*/
 
 /**/
 
@@ -85,11 +88,7 @@
 #define	LANG_TAB_MAX_N	64
 #define	STR_DFLT_LANG	"ja_JP.eucJP"
 
-typedef enum {
-	LANG_KIND_ENGLISH,
-	LANG_KIND_JAPANESE,
-	LANG_KIND_MAX_N,
-} lang_kind_t;
+lang_kind_t	g_lang_kind;
 
 typedef struct {
 	char	*name;
@@ -97,10 +96,26 @@ typedef struct {
 } lang_t;
 
 static lang_t	lang_tab[LANG_TAB_MAX_N] = {
-	{ "C", LANG_KIND_ENGLISH, },
-	{ "ja_JP.eucJP", LANG_KIND_JAPANESE, },
-	{ "ja_JP.ujis", LANG_KIND_JAPANESE, },
-	{ NULL, LANG_KIND_MAX_N, },
+	{
+		"C",
+		LANG_KIND_ENGLISH,
+	},
+	{
+		"ja_JP.eucJP",
+		LANG_KIND_JAPANESE_EUC,
+	},
+	{
+		"ja_JP.ujis",
+		LANG_KIND_JAPANESE_EUC,
+	},
+	{
+		"ja_JP.UTF-8",
+		LANG_KIND_JAPANESE_UTF_8,
+	},
+	{
+		NULL,
+		LANG_KIND_MAX_N,
+	},
 };
 
 /* msg_t */
@@ -112,10 +127,17 @@ typedef struct {
 
 static msg_t	all_lang_msg[LANG_KIND_MAX_N][N_MSG_MAX_N] = {
 	{
-#include	"msg_eng.h"
+#include	"msg-eng.h"
 	},
 	{
-#include	"msg_jpn.h"
+#include	"msg-jpn.h"
+	},
+	{
+#ifdef	D_DOS
+# include	"msg-jpn.h"
+#else	/* D_DOS */
+# include	"msg-jpn-utf8.h"
+#endif	/* D_DOS */
 	},
 };
 
@@ -126,7 +148,6 @@ static bool_t	flg_msg_def[N_MSG_MAX_N];
 void	init_msg( void )
 {
 	char	*lang;
-	lang_kind_t	lang_kind;
 	long	k;
 
 	n_msg_t	m;
@@ -139,16 +160,16 @@ void	init_msg( void )
 	if( lang == NULL )
 		lang = STR_DFLT_LANG;
 
-	lang_kind = LANG_KIND_MAX_N;
+	g_lang_kind = LANG_KIND_MAX_N;
 	for( k = 0; k < LANG_TAB_MAX_N; k++ ){
 		if( lang_tab[k].name == NULL )
 			break;
 		if( strcmp( lang, lang_tab[k].name ) == 0 ){
-			lang_kind = lang_tab[k].kind;
+			g_lang_kind = lang_tab[k].kind;
 			break;
 		}
 	}
-	if( lang_kind >= LANG_KIND_MAX_N ){
+	if( g_lang_kind >= LANG_KIND_MAX_N ){
 		print_err( STR_ERR_LANG, lang );
 		print_err( STR_S, STR_SUPPORT_LANG );
 		for( k = 0; k < LANG_TAB_MAX_N; k++ ){
@@ -165,20 +186,20 @@ void	init_msg( void )
 		flg_msg_def[m] = FALSE;
 
 	for( i = 0; i < N_MSG_MAX_N; i++ ){
-		if( all_lang_msg[lang_kind][i].s == NULL )
+		if( all_lang_msg[g_lang_kind][i].s == NULL )
 			break;
 
-		m = all_lang_msg[lang_kind][i].n;
+		m = all_lang_msg[g_lang_kind][i].n;
 		if( flg_msg_def[m] ){
 			char	*s;
 
 			/* åƤ */
-			s = all_lang_msg[lang_kind][m].s;
+			s = all_lang_msg[g_lang_kind][m].s;
 			print_err( STR_ERR_REDEFINED_MSG,
-					lang_tab[lang_kind].name, m, s );
+					lang_tab[g_lang_kind].name, m, s );
 			flg_exit = TRUE;
 		}
-		ary_msg[m] = all_lang_msg[lang_kind][i].s;
+		ary_msg[m] = all_lang_msg[g_lang_kind][i].s;
 		flg_msg_def[m] = TRUE;
 	}
 
@@ -186,7 +207,7 @@ void	init_msg( void )
 		if( !flg_msg_def[m] ){
 			/* åƤʤ */
 			print_err( STR_ERR_NOT_DEFINED_MSG,
-					lang_tab[lang_kind].name, m );
+					lang_tab[g_lang_kind].name, m );
 			flg_exit = TRUE;
 		}
 	}
@@ -197,3 +218,10 @@ void	init_msg( void )
 
 /**/
 
+lang_kind_t	get_lang_kind( void )
+{
+	return g_lang_kind;
+}
+
+/**/
+
