@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: party.c,v 1.61 2003/11/21 18:35:21 bowkenken Exp $
+* $Id: party.c,v 1.142 2004/07/31 07:27:27 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -43,42 +42,48 @@
 /*#include	"fight.h"*/
 /*#include	"fx.h"*/
 /*#include	"trap.h"*/
-/*#include	"draw.h"*/
+#include	"draw.h"
 /*#include	"curs.h"*/
 #include	"menu.h"
+#include	"amenu.h"
+/*#include	"request.h"*/
+/*#include	"gfile.h"*/
 /*#include	"msg.h"*/
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-/*#include	"turn_prot.h"*/
-#include	"dun_prot.h"
-#include	"town_prot.h"
-#include	"item_prot.h"
-#include	"spell_prot.h"
-#include	"chr_prot.h"
-#include	"party_prot.h"
-#include	"mnstr_prot.h"
-#include	"pet_prot.h"
-#include	"fight_prot.h"
-#include	"fx_prot.h"
-#include	"trap_prot.h"
-#include	"draw_prot.h"
-#include	"curs_prot.h"
-#include	"menu_prot.h"
-/*#include	"tmenu_prot.h"*/
-#include	"amenu_prot.h"
-/*#include	"gfile_prot.h"*/
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+/*#include	"turn-prot.h"*/
+#include	"dun-prot.h"
+#include	"town-prot.h"
+#include	"item-prot.h"
+#include	"spell-prot.h"
+#include	"chr-prot.h"
+#include	"party-prot.h"
+#include	"mnstr-prot.h"
+#include	"pet-prot.h"
+#include	"fight-prot.h"
+#include	"fx-prot.h"
+#include	"trap-prot.h"
+#include	"draw-prot.h"
+#include	"curs-prot.h"
+#include	"menu-prot.h"
+/*#include	"tmenu-prot.h"*/
+#include	"amenu-prot.h"
+/*#include	"request-prot.h"*/
+/*#include	"gfile-prot.h"*/
+/*#include	"msg-prot.h"*/
 
 /**/
 
-#define	DFLT_EXP	200
-#define	DFLT_GOLD	1200
+#define	DFLT_EXP	500
+#define	DFLT_GOLD	3000
 
 #define	MBR_NEAR_R	4
 
 #define	FRIEND_MAX_TURN	(TURN_A_DAY * 1)
 
+#define	LS_MBR_NULL	LS_MBR_MAX_N
+
 static mbr_t	*ls_mbr[LS_MBR_MAX_N + 1];
 static mbr_t	ls_mbr_buf[LS_MBR_MAX_N + 1];
 static party_t	party;
@@ -86,59 +91,69 @@ static party_t	party;
 static long	abl_init[MAIN_RACE_MAX_N][ABL_KIND_MAX_N] = {
 
 	/* HP  MP  DEX STR */
-	/* QUI WIS INT CHA */
+	/* QUI INT WIS CHA */
 	/* FIG MON HUN THI */
 	/* MAG SOR ENC SUM */
 	/* PRI SHA BAR NIN */
 
 	/* MAIN_RACE_HUMAN, */
-	{  2,  2,  2,  2,
-	   2,  2,  2,  2,
-	   0,  0,  0,  0,
-	   0,  0,  0,  0,
-	   0,  0,  0,  0, },
+	{
+		2, 2, 2, 2,
+		2, 2, 2, 2,
+		0, 0, 0, 0,
+		0, 0, 0, 0,
+		0, 0, 0, 0,
+	},
 
 	/* MAIN_RACE_HALF_ELF, */
-	{  2,  2,  2,  1,
-	   3,  2,  2,  2,
-	   0,  0,  1,  0,
-	   0,  0,  0,  0,
-	   0,  0,  0,  0, },
+	{
+		2, 2, 2, 1,
+		3, 2, 2, 2,
+		0, 0, 1, 0,
+		0, 0, 0, 0,
+		0, 0, 0, 0,
+	},
 
 	/* MAIN_RACE_ELF, */
-	{  1,  2,  2,  1,
-	   3,  2,  2,  3,
-	   0,  0,  1,  0,
-	   0,  0,  0,  0,
-	   0,  1,  0,  0, },
+	{
+		1, 2, 2, 1,
+		3, 2, 2, 3,
+		0, 0, 1, 0,
+		0, 0, 0, 0,
+		0, 1, 0, 0,
+	},
 
 	/* MAIN_RACE_DWARF, */
-	{  3,  1,  3,  3,
-	   1,  2,  2,  1,
-	   1,  0,  0,  0,
-	   0,  0,  0,  0,
-	   0,  0,  0,  0, },
+	{
+		3, 1, 3, 3,
+		1, 2, 2, 1,
+		1, 0, 0, 0,
+		0, 0, 0, 0,
+		0, 0, 0, 0,
+	},
 
 	/* MAIN_RACE_VIVID, */
-	{  1,  2,  3,  1,
-	   3,  2,  2,  2,
-	   0,  0,  0,  1,
-	   0,  0,  0,  0,
-	   0,  0,  0,  0, },
+	{
+		1, 2, 3, 1,
+		3, 2, 2, 2,
+		0, 0, 0, 1,
+		0, 0, 0, 0,
+		0, 0, 0, 0,
+	},
 };
 
 static rate_t	resi_init[MAIN_RACE_MAX_N][RESI_KIND_MAX_N] = {
-	/* KNOC,SLAS,STIN,HEAT,COLD,ACID,WIND,ELEC,POIS, */
+	/* KNOC,SLAS,STIN,HEAT,COLD,MIND,ACID,ELEC,POIS, */
 /* MAIN_RACE_HUMAN */
 	{   100, 100, 100, 100, 100, 100, 100, 100, 100 },
 /* MAIN_RACE_HALF_ELF */
 	{    90,  90,  90, 100, 100, 100, 100, 100, 100 },
 /* MAIN_RACE_ELF */
-	{    80,  80,  80, 100, 100, 100, 150, 100, 100 },
+	{    80,  80,  80, 100, 100, 150, 100, 100, 100 },
 /* MAIN_RACE_DWARF */
 	{   150, 150, 150, 100, 100, 100, 100, 100, 200 },
 /* MAIN_RACE_VIVID */
-	{    70,  70,  70, 100, 100, 100, 200, 100, 100 },
+	{    70,  70,  70, 100, 100, 200, 100, 100, 100 },
 };
 
 static stomach_t	ave_stomach[MAIN_RACE_MAX_N] = {
@@ -160,29 +175,29 @@ typedef struct {
 
 /* class */
 
-#include	"class_tab.h"
+#include	"class-tab.h"
 
 /**/
 
 void	init_party( void )
 {
 	long	i, j, max_n;
-	race_t	race;
 
 	for( i = 0; i < LS_MBR_MAX_N + 1; i++ )
 		ls_mbr[i] = &(ls_mbr_buf[i]);
 
 	for( i = 0; i < MBR_MAX_N; i++ )
-		party.mbr[i] = ls_mbr[LS_MBR_MAX_N];
+		party.mbr[i] = ls_mbr[i];
 
 	for( i = 0; i < LS_MBR_MAX_N + 1; i++ ){
+		ls_mbr[i]->stat = FLG_STAT_NOT_EXIST;
 		ls_mbr[i]->mbr_n = MBR_N_NOT_JOIN;
 		ls_mbr[i]->ls_mbr_n = i;
-		ls_mbr[i]->x = -1;
-		ls_mbr[i]->y = -1;
-		ls_mbr[i]->pre_x = -1;
-		ls_mbr[i]->pre_y = -1;
-		ls_mbr[i]->stat = FLG_STAT_NOT_EXIST;
+		ls_mbr[i]->x = MAP_DEL_X;
+		ls_mbr[i]->y = MAP_DEL_Y;
+		ls_mbr[i]->pre_x = MAP_DEL_X;
+		ls_mbr[i]->pre_y = MAP_DEL_Y;
+		ls_mbr[i]->flg_chr |= FLG_CHR_AUTO_FIGHT;
 		ls_mbr[i]->fx.next = &(ls_mbr[i]->fx);
 		ls_mbr[i]->fx.prev = &(ls_mbr[i]->fx);
 	}
@@ -220,13 +235,26 @@ void	init_party( void )
 	party.flg = FLG_NULL;
 	party.flg |= FLG_PARTY_SRCH_AUTO_OFF;
 
-	/**/
+	if( g_flg_debug )
+		init_party_debug();
+	else
+		init_party_dflt();
+}
+
+/**/
+
+void	init_party_dflt( void )
+{
+	race_t	race;
+	item_t	*item;
+	item_kind_dat_t	sub_kind;
+	long	i;
 
 	race.main = MAIN_RACE_HUMAN;
 	race.sub = SUB_RACE_NULL;
 	strncpy( race.name, MSG_MAIN_RACE_HUMAN, RACE_NAME_MAX_LEN );
 	new_mbr( ls_mbr[0], "<///BLADE//}===o", 'B', race,
-			MALE, CLASS_N_NINJA, 0 );
+			MALE, CLASS_N_FIGHTER, 0 );
 
 	race.main = MAIN_RACE_DWARF;
 	race.sub = SUB_RACE_NULL;
@@ -264,21 +292,218 @@ void	init_party( void )
 	new_mbr( ls_mbr[6], "====GOKOU=======", 'G', race,
 			MALE, CLASS_N_MONK, 6 );
 
-	for( i = 0; i < MBR_MAX_N; i++ )
-		join_mbr( i, i );
+	race.main = MAIN_RACE_HUMAN;
+	race.sub = SUB_RACE_CAT;
+	strncpy( race.name, MSG_MAIN_RACE_HUMAN, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[7], "----EYE-------|===", 'Y', race,
+			FEMALE, CLASS_N_SUMMONER, 7 );
+
+	/* item */
+
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		sub_kind.light = LIGHT_KIND_TORCH;
+		init_mbr_item( ls_mbr[i], ITEM_KIND_LIGHT, &sub_kind,
+				TRUE, FALSE, TRUE );
+	}
+
+	/* weapon */
+
+	i = 0;
+	sub_kind.wpn = WPN_KIND_SHORT_SWORD;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 1;
+	sub_kind.wpn = WPN_KIND_SHORT_SWORD;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 2;
+	sub_kind.wpn = WPN_KIND_KNIFE;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, TRUE, TRUE );
+
+	i = 3;
+	sub_kind.wpn = WPN_KIND_LIGHT_MACE;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 4;
+	sub_kind.wpn = WPN_KIND_KNIFE;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 5;
+	sub_kind.wpn = WPN_KIND_WOODEN_CLUB;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 6;
+	sub_kind.wpn = WPN_KIND_QUARTER_STAFF;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 7;
+	sub_kind.wpn = WPN_KIND_KNIFE;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	/* bow */
+
+	i = 2;
+	sub_kind.wpn = WPN_KIND_SHORT_BOW;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 2;
+	sub_kind.wpn = WPN_KIND_ARW;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 3;
+	sub_kind.wpn = WPN_KIND_SLING;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 3;
+	sub_kind.wpn = WPN_KIND_STONE;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 4;
+	sub_kind.wpn = WPN_KIND_DAGGER;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	/* armor */
+
+	i = 0;
+	sub_kind.armor = ARMOR_KIND_SOFT_LEATHER_ARMOR;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 1;
+	sub_kind.armor = ARMOR_KIND_SOFT_LEATHER_ARMOR;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 2;
+	sub_kind.armor = ARMOR_KIND_SOFT_LEATHER_ARMOR;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 3;
+	sub_kind.armor = ARMOR_KIND_SOFT_LEATHER_ARMOR;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 4;
+	sub_kind.armor = ARMOR_KIND_SOFT_LEATHER_ARMOR;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 5;
+	sub_kind.armor = ARMOR_KIND_ROBE;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 6;
+	sub_kind.armor = ARMOR_KIND_SOFT_LEATHER_ARMOR;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 7;
+	sub_kind.armor = ARMOR_KIND_SOFT_LEATHER_ARMOR;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	/* scroll */
+
+	i = 0;
+	sub_kind.scroll = SCROLL_KIND_RECALL;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_SCROLL, &sub_kind,
+			TRUE, FALSE, FALSE );
+
+	/* potion */
+
+	i = 1;
+	sub_kind.potion = POTION_KIND_RUM;
+	item = init_mbr_item( ls_mbr[i], ITEM_KIND_POTION, &sub_kind,
+			TRUE, FALSE, FALSE );
+
+	/* stick */
+
+	/* misc */
+
+	i = 4;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_SPIKE, NULL,
+			TRUE, FALSE, FALSE );
 }
 
 /**/
 
 void	init_party_debug( void )
 {
-	long	i;
+	race_t	race;
 	item_t	*item;
 	item_kind_dat_t	sub_kind;
+	long	i;
 
-	for( i = 0; i < 7; i++ ){
+	race.main = MAIN_RACE_HUMAN;
+	race.sub = SUB_RACE_NULL;
+	strncpy( race.name, MSG_MAIN_RACE_HUMAN, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[0], "<///BLADE//}===o", 'B', race,
+			MALE, CLASS_N_NINJA, 0 );
+
+	race.main = MAIN_RACE_DWARF;
+	race.sub = SUB_RACE_NULL;
+	strncpy( race.name, MSG_MAIN_RACE_DWARF, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[1], ")(==DRANK======>", 'D', race,
+			MALE, CLASS_N_FIGHTER, 1 );
+
+	race.main = MAIN_RACE_HALF_ELF;
+	race.sub = SUB_RACE_NULL;
+	strncpy( race.name, MSG_MAIN_RACE_HALF_ELF, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[2], "<---ELSIE----<<<", 'E', race,
+			MALE, CLASS_N_HUNTER, 2 );
+
+	race.main = MAIN_RACE_HUMAN;
+	race.sub = SUB_RACE_NULL;
+	strncpy( race.name, MSG_MAIN_RACE_HUMAN, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[3], "+-==CECILE=====X", 'C', race,
+			FEMALE, CLASS_N_PRIEST, 3 );
+
+	race.main = MAIN_RACE_VIVID;
+	race.sub = SUB_RACE_NULL;
+	strncpy( race.name, MSG_MAIN_RACE_VIVID, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[4], "m-o HALLE  <<<)=", 'H', race,
+			MALE, CLASS_N_BARD, 4 );
+
+	race.main = MAIN_RACE_ELF;
+	race.sub = SUB_RACE_NULL;
+	strncpy( race.name, MSG_MAIN_RACE_ELF, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[5], "O>==BEAUT====---", 'U', race,
+			FEMALE, CLASS_N_WIZARD, 5 );
+
+	race.main = MAIN_RACE_HUMAN;
+	race.sub = SUB_RACE_MONKEY;
+	strncpy( race.name, MSG_RACE_NAME_VEGE, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[6], "====GOKOU=======", 'G', race,
+			MALE, CLASS_N_MONK, 6 );
+
+	race.main = MAIN_RACE_HUMAN;
+	race.sub = SUB_RACE_CAT;
+	strncpy( race.name, MSG_MAIN_RACE_HUMAN, RACE_NAME_MAX_LEN );
+	new_mbr( ls_mbr[7], "----EYE-------|===", 'Y', race,
+			FEMALE, CLASS_N_SUMMONER, 7 );
+
+	/* ã */
+
+	for( i = 0; i < LS_MBR_MAX_N; i++ ){
 		ls_mbr[i]->dun_max_lev_floor = 0;
 		ls_mbr[i]->dun_max_lev_base = 128;
+
+		ls_mbr[i]->fx_data.friend_turn = 0;
 	}
 
 	/* item */
@@ -308,6 +533,11 @@ void	init_party_debug( void )
 	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
 			TRUE, FALSE, TRUE );
 
+	i = 0;
+	sub_kind.wpn = WPN_KIND_GLAIVE;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
+			TRUE, FALSE, TRUE );
+
 	i = 1;
 	sub_kind.wpn = WPN_KIND_GREAT_AXE;
 	init_mbr_item( ls_mbr[i], ITEM_KIND_WPN, &sub_kind,
@@ -401,6 +631,11 @@ void	init_party_debug( void )
 	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
 			TRUE, FALSE, TRUE );
 
+	i = 2;
+	sub_kind.armor = ARMOR_KIND_SPEED_BOOTS;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			FALSE, FALSE, FALSE );
+
 	i = 3;
 	sub_kind.armor = ARMOR_KIND_ROUND_SHIELD;
 	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
@@ -428,20 +663,20 @@ void	init_party_debug( void )
 	init_mbr_item( ls_mbr[i], ITEM_KIND_SCROLL, &sub_kind,
 			TRUE, FALSE, FALSE );
 
-	i = 3;
+	i = 4;
 	sub_kind.scroll = SCROLL_KIND_TELEPORT;
 	init_mbr_item( ls_mbr[i], ITEM_KIND_SCROLL, &sub_kind,
-			TRUE, FALSE, FALSE );
+			FALSE, FALSE, FALSE );
 
 	i = 4;
 	sub_kind.scroll = SCROLL_KIND_POW_UP_WPN_HIT;
 	init_mbr_item( ls_mbr[i], ITEM_KIND_SCROLL, &sub_kind,
-			TRUE, FALSE, FALSE );
+			FALSE, FALSE, FALSE );
 
 	i = 4;
 	sub_kind.scroll = SCROLL_KIND_POW_UP_WPN_DAM;
 	init_mbr_item( ls_mbr[i], ITEM_KIND_SCROLL, &sub_kind,
-			TRUE, FALSE, FALSE );
+			FALSE, FALSE, FALSE );
 
 	i = 5;
 	sub_kind.scroll = SCROLL_KIND_REMOVE_CURSE;
@@ -481,8 +716,51 @@ void	init_party_debug( void )
 			TRUE, FALSE, FALSE );
 	inc_item( item, +16 );
 
+	i = 5;
+	sub_kind.potion = POTION_KIND_RESI_PHYS;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_POTION, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 5;
+	sub_kind.potion = POTION_KIND_STRENGTH;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_POTION, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 5;
+	sub_kind.potion = POTION_KIND_DULLNESS;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_POTION, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 5;
+	sub_kind.potion = POTION_KIND_POISON;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_POTION, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 5;
+	sub_kind.potion = POTION_KIND_FLY;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_POTION, &sub_kind,
+			FALSE, FALSE, FALSE );
+
 	/* stick */
 
+	i = 5;
+	sub_kind.stick.kind = STICK_KIND_WAND;
+	sub_kind.stick.spell_kind = SPELL_KIND_QUICKNESS;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_STICK, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 5;
+	sub_kind.stick.kind = STICK_KIND_WAND;
+	sub_kind.stick.spell_kind = SPELL_KIND_SLOWNESS;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_STICK, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 5;
+	sub_kind.stick.kind = STICK_KIND_ROD;
+	sub_kind.stick.spell_kind = SPELL_KIND_SLOWNESS;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_STICK, &sub_kind,
+			FALSE, FALSE, FALSE );
+
 	i = 5;
 	sub_kind.stick.kind = STICK_KIND_WAND;
 	sub_kind.stick.spell_kind = SPELL_KIND_SLEEP;
@@ -497,6 +775,47 @@ void	init_party_debug( void )
 
 	/* misc */
 
+	i = 0;
+	sub_kind.armor = ARMOR_KIND_GAUNTLETS_OF_POW_UP_HIT;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ARMOR, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 0;
+	sub_kind.ring = RING_KIND_NULL;
+	sub_kind.amulet = AMULET_KIND_NULL;
+	sub_kind.necklace = NECKLACE_KIND_CURSE_HEAT;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ACCE, &sub_kind,
+			FALSE, TRUE, FALSE );
+
+	i = 0;
+	sub_kind.ring = RING_KIND_DRAIN_HP;
+	sub_kind.amulet = AMULET_KIND_NULL;
+	sub_kind.necklace = NECKLACE_KIND_NULL;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ACCE, &sub_kind,
+			FALSE, FALSE, FALSE );
+
+	i = 5;
+	sub_kind.ring = RING_KIND_CURE_MP;
+	sub_kind.amulet = AMULET_KIND_NULL;
+	sub_kind.necklace = NECKLACE_KIND_NULL;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ACCE, &sub_kind,
+			TRUE, FALSE, TRUE );
+
+	i = 4;
+	sub_kind.inst = INST_KIND_FLUTE;
+	item = init_mbr_item( ls_mbr[i], ITEM_KIND_INST, &sub_kind,
+			TRUE, FALSE, FALSE );
+	if( item != NULL ){
+		item->dat.inst.flg_limit = TRUE;
+		item->dat.inst.spell_kind = SPELL_KIND_SONG_CURE_MP;
+	}
+	set_item_name( item );
+
+	i = 4;
+	sub_kind.anthology = ANTHOLOGY_KIND_WEAKEN_ABL;
+	init_mbr_item( ls_mbr[i], ITEM_KIND_ANTHOLOGY, &sub_kind,
+			FALSE, FALSE, FALSE );
+
 	i = 4;
 	init_mbr_item( ls_mbr[i], ITEM_KIND_SPIKE, NULL,
 			TRUE, FALSE, FALSE );
@@ -516,7 +835,8 @@ void	init_party_debug( void )
 #if	0
 /*@@@*/
 	i = 0;
-	ls_mbr[i]->nicotine.poisoning_rate = _100_PERCENT;
+	ls_mbr[i]->fx_data.nicotine.poisoning_rate = _100_PERCENT;
+	ls_mbr[i]->fx_data.nicotine.lost_rate = _100_PERCENT;
 #endif
 }
 
@@ -527,12 +847,14 @@ item_t	*init_mbr_item(
 	bool_t flg_ident, bool_t curse, bool_t flg_eq
 )
 {
-	item_t	*item;
-	long	tab_n;
 	long	lev;
+	long	tab_n;
+	item_t	*item;
 
 	if( mbr == NULL )
 		return NULL;
+	if( mbr->kind != CHR_KIND_MBR )
+		return NULL;
 
 	lev = -10;
 
@@ -546,10 +868,13 @@ item_t	*init_mbr_item(
 	case ITEM_KIND_SCROLL:
 	case ITEM_KIND_STICK:
 	case ITEM_KIND_ACCE:
+	case ITEM_KIND_INST:
+	case ITEM_KIND_ANTHOLOGY:
 	case ITEM_KIND_LIGHT:
 	case ITEM_KIND_FOOD:
 	case ITEM_KIND_TOBACCO:
 	case ITEM_KIND_PIPE:
+	case ITEM_KIND_RARE_GOODS:
 	case ITEM_KIND_COIN:
 		if( sub_kind == NULL )
 			return NULL;
@@ -561,7 +886,10 @@ item_t	*init_mbr_item(
 		break;
 	}
 
-	tab_n = 0;
+	/**/
+
+	tab_n = 0;	/* -1 ǤϤʤ */
+
 	switch( kind ){
 	case ITEM_KIND_NULL:
 	case ITEM_KIND_MAX_N:
@@ -593,6 +921,14 @@ item_t	*init_mbr_item(
 				sub_kind->amulet,
 				sub_kind->necklace ) );
 		break;
+	case ITEM_KIND_INST:
+		tab_n = get_inst_tab_n( get_inst_tab(
+				sub_kind->inst ) );
+		break;
+	case ITEM_KIND_ANTHOLOGY:
+		tab_n = get_anthology_tab_n( get_anthology_tab(
+				sub_kind->anthology ) );
+		break;
 	case ITEM_KIND_LIGHT:
 		tab_n = get_light_tab_n( get_light_tab(
 				sub_kind->light ) );
@@ -616,6 +952,10 @@ item_t	*init_mbr_item(
 		break;
 	case ITEM_KIND_DISPOSAL:
 		break;
+	case ITEM_KIND_RARE_GOODS:
+		tab_n = get_rare_goods_tab_n( get_rare_goods_tab(
+				sub_kind->rare_goods ) );
+		break;
 	case ITEM_KIND_COIN:
 		tab_n = get_coin_tab_n( get_coin_tab(
 				sub_kind->coin ) );
@@ -623,9 +963,12 @@ item_t	*init_mbr_item(
 	case ITEM_KIND_JUNK:
 		break;
 	}
+
 	if( tab_n < 0 )
 		return NULL;
 
+	/**/
+
 	item = NULL;
 
 	switch( kind ){
@@ -650,6 +993,12 @@ item_t	*init_mbr_item(
 	case ITEM_KIND_ACCE:
 		item = make_item_acce( lev, tab_n );
 		break;
+	case ITEM_KIND_INST:
+		item = make_item_inst( lev, tab_n, curse );
+		break;
+	case ITEM_KIND_ANTHOLOGY:
+		item = make_item_anthology( lev, tab_n );
+		break;
 	case ITEM_KIND_LIGHT:
 		item = make_item_light( lev, tab_n );
 		break;
@@ -660,7 +1009,7 @@ item_t	*init_mbr_item(
 		item = make_item_food( lev, tab_n );
 		break;
 	case ITEM_KIND_CHEST:
-		item = make_item_chest( lev, 4 );
+		item = make_item_chest( lev, 4, TRUE );
 		break;
 	case ITEM_KIND_TOBACCO:
 		item = make_item_tobacco( lev, tab_n );
@@ -671,6 +1020,9 @@ item_t	*init_mbr_item(
 	case ITEM_KIND_DISPOSAL:
 		item = make_item_disposal( lev );
 		break;
+	case ITEM_KIND_RARE_GOODS:
+		item = make_item_rare_goods( lev, tab_n );
+		break;
 	case ITEM_KIND_COIN:
 		item = make_item_coin( lev, tab_n );
 		break;
@@ -682,6 +1034,8 @@ item_t	*init_mbr_item(
 	if( item == NULL )
 		return NULL;
 
+	/**/
+
 	item->flg |= FLG_ITEM_DETECTED;
 	if( flg_ident )
 		item->flg |= FLG_ITEM_IDENTIFIED;
@@ -700,6 +1054,17 @@ item_t	*init_mbr_item(
 
 /**/
 
+void	init_mbr_formation( void )
+{
+	long	i;
+
+	for( i = 0; i < MBR_MAX_N; i++ )
+		if( !chk_flg( ls_mbr[i]->stat, FLG_STAT_NOT_EXIST ) )
+			join_mbr( i, i );
+}
+
+/**/
+
 void	new_mbr_reset( mbr_t *mbr )
 {
 	new_mbr( mbr, mbr->name, mbr->face.mnr, mbr->race, mbr->sex.cur,
@@ -736,6 +1101,8 @@ void	new_mbr(
 	mbr->y = MAP_DEL_Y;
 	mbr->pre_x = MAP_DEL_X;
 	mbr->pre_y = MAP_DEL_Y;
+	mbr->dx = 1;
+	mbr->dy = 1;
 
 	mbr->move.speed.rate = _100_PERCENT;
 	mbr->move.speed.rate_org = _100_PERCENT;
@@ -763,9 +1130,10 @@ void	new_mbr(
 	strncpy( mbr->act.dflt_msg_mnstr,
 			MSG_MENU_MARK_FIGHT, ACT_MSG_MAX_LEN );
 	clr_chr_trgt_act( mbr, TRUE );
+	mbr->pre_act = mbr->act;
 
-	mbr->flg_chr = (FLG_CHR_AUTO_DOOR | FLG_CHR_AVOID_TRAP);
-	mbr->flg_map = FLG_MAP_CHR_FIND;
+	mbr->flg_chr |= (FLG_CHR_AUTO_DOOR | FLG_CHR_AVOID_TRAP);
+	mbr->flg_map |= FLG_MAP_CHR_FIND;
 	mbr->attitude = ATTITUDE_MATE;
 	mbr->owner = NULL;
 	mbr->stat = FLG_NULL;
@@ -799,9 +1167,21 @@ void	new_mbr(
 	mbr->fx.next = &(mbr->fx);
 	mbr->fx.prev = &(mbr->fx);
 
-	mbr->fx_data.armor_def = 0;
-	mbr->fx_data.armor_crtcl = 0;
-	mbr->fx_data.armor_ac = 0;
+	mbr->fx_data.wpn_hit = (rate_t)0;
+	mbr->fx_data.wpn_crtcl = (rate_t)0;
+	mbr->fx_data.wpn_dam = (rate_t)0;
+
+	mbr->fx_data.armor_def = (rate_t)0;
+	mbr->fx_data.armor_crtcl = (rate_t)0;
+	mbr->fx_data.armor_ac = (rate_t)0;
+
+	mbr->fx_data.cure_hp.turn = 0;
+	mbr->fx_data.cure_hp.rate = 0;
+	mbr->fx_data.cure_hp.n = 0;
+
+	mbr->fx_data.cure_mp.turn = 0;
+	mbr->fx_data.cure_mp.rate = 0;
+	mbr->fx_data.cure_mp.n = 0;
 
 	mbr->fx_data.infra_vision = 0;
 
@@ -832,9 +1212,14 @@ void	new_mbr(
 	mbr->work.flg = FLG_NULL;
 	mbr->work.caught = 0;
 	mbr->work.catch_chr = NULL;
+	mbr->work.link_chr = NULL;
+	mbr->work.request = NULL;
 	mbr->work.towner_dx = 0;
 	mbr->work.towner_dy = 0;
 	mbr->work.enter_door = NULL;
+	mbr->work.dismissal_turn = 0;
+
+	set_chr_id( mbr, get_ls_mbr(), LS_MBR_MAX_N );
 
 	/**/
 
@@ -907,24 +1292,28 @@ void	set_mbr_dir( void )
 
 void	appear_party( bool_t flg_clr_map )
 {
+	bool_t	flg_force = TRUE;
 	long	i, n;
-	long	nx, ny;
 
-	for( n = 0; n < MBR_MAX_N; n++ )
+	for( n = 0; n < MBR_MAX_N; n++ ){
+		lost_trgt( party.mbr[n] );
 		clr_chr_trgt_act( party.mbr[n], FALSE );
+	}
 
 	release_all_mbr();
 
 	if( flg_clr_map )
 		clr_map_all_mbr();
 
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		party.square.x = 1 + randm( MAP_MAX_X - 2);
 		party.square.y = 1 + randm( MAP_MAX_Y - 2);
 		party.square.pre_x = party.square.x;
 		party.square.pre_y = party.square.y;
 
 		for( n = 0; n < MBR_MAX_N; n++ ){
+			long	nx, ny;
+
 			if( chk_flg_or( party.mbr[n]->stat,
 					FLG_STAT_NOT_EXIST
 					| FLG_STAT_DEAD ) ){
@@ -941,18 +1330,19 @@ void	appear_party( bool_t flg_clr_map )
 			if( !can_move_chr( party.mbr[n], nx, ny ) )
 				break;	/* ֤˰ưʤ */
 		}
-		if( n >= MBR_MAX_N )
+		if( n >= MBR_MAX_N ){
+			flg_force = FALSE;
 			break;	/* ư褿 */
+		}
 	}
 
-	for( n = 0; n < MBR_MAX_N; n++ ){
-		if( chk_flg_or( party.mbr[n]->stat,
-				FLG_STAT_NOT_EXIST
-				| FLG_STAT_DEAD ) ){
-			continue;
+	if( flg_force ){
+		for( n = 0; n < MBR_MAX_N; n++ ){
+			put_chr( party.mbr[n],
+					MAP_MAX_X / 2,
+					MAP_MAX_Y / 2,
+					FALSE );
 		}
-
-		find_obj( party.mbr[n]->x, party.mbr[n]->y );
 	}
 
 	set_map_all_mbr();
@@ -999,7 +1389,7 @@ void	move_all_mbr_trgt( void )
 
 /**/
 
-void	act_party( void )
+void	move_phase_party( void )
 {
 	long	i;
 
@@ -1009,7 +1399,25 @@ void	act_party( void )
 			continue;
 		}
 
-		act_chr( party.mbr[i] );
+		move_phase_chr( party.mbr[i] );
+	}
+
+	draw_party();
+}
+
+/**/
+
+void	act_phase_party( void )
+{
+	long	i;
+
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		if( chk_flg_or( party.mbr[i]->stat,
+				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
+			continue;
+		}
+
+		act_phase_chr( party.mbr[i] );
 		set_modifier( party.mbr[i] );
 	}
 
@@ -1023,8 +1431,45 @@ void	act_party( void )
 		enter_shop();
 		break;
 	}
+}
 
-	draw_party();
+/**/
+
+void	chk_trap_party( void )
+{
+	long	i;
+	mbr_t	*thief;
+
+	thief = NULL;
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		if( chk_flg_or( party.mbr[i]->stat,
+				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
+			continue;
+		}
+
+		if( thief == NULL ){
+			thief = party.mbr[i];
+		} else {
+			long	thief_lev, mbr_lev;
+
+			thief_lev = get_lev( thief, ABL_KIND_THI );
+			thief_lev += get_lev( thief, ABL_KIND_AGI );
+			mbr_lev = get_lev( party.mbr[i], ABL_KIND_THI );
+			mbr_lev += get_lev( party.mbr[i], ABL_KIND_AGI );
+
+			if( thief_lev < mbr_lev )
+				thief = party.mbr[i];
+		}
+	}
+
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		if( chk_flg_or( party.mbr[i]->stat,
+				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
+			continue;
+		}
+
+		chk_trap( party.mbr[i], thief );
+	}
 }
 
 /**/
@@ -1033,6 +1478,8 @@ bool_t	chk_auto_mnstr( mbr_t *mbr, long x1, long y1, long x2, long y2 )
 {
 	dun_t	*dun = get_dun();
 
+	if( mbr == NULL )
+		return FALSE;
 	if( mbr->kind != CHR_KIND_MBR )
 		return FALSE;
 
@@ -1041,14 +1488,15 @@ bool_t	chk_auto_mnstr( mbr_t *mbr, long x1, long y1, long x2, long y2 )
 
 	if( dun->map.chr.mjr[y1][x1] != FACE_MJR_NULL ){
 		if( dun->map.chr.mjr[y1][x1] != FACE_MJR_MBR ){
-			set_mbr_act_auto_mnstr( mbr, x1, y1 );
-			return TRUE;
+			if( set_mbr_act_auto_mnstr( mbr, x1, y1 ) )
+				return TRUE;
 		}
 	}
+
 	if( dun->map.chr.mjr[y2][x2] != FACE_MJR_NULL ){
 		if( dun->map.chr.mjr[y2][x2] != FACE_MJR_MBR ){
-			set_mbr_act_auto_mnstr( mbr, x2, y2 );
-			return TRUE;
+			if( set_mbr_act_auto_mnstr( mbr, x2, y2 ) )
+				return TRUE;
 		}
 	}
 
@@ -1061,6 +1509,8 @@ bool_t	chk_auto_door_mbr( mbr_t *mbr, long x1, long y1, long x2, long y2 )
 {
 	dun_t	*dun = get_dun();
 
+	if( mbr == NULL )
+		return FALSE;
 	if( mbr->kind != CHR_KIND_MBR )
 		return FALSE;
 
@@ -1081,16 +1531,17 @@ bool_t	chk_auto_door_mbr( mbr_t *mbr, long x1, long y1, long x2, long y2 )
 
 /**/
 
-void	die_mbr( mbr_t *mbr )
+void	die_mbr( mbr_t *mbr, bool_t flg_chk_game_over )
 {
 	long	i;
 
 	if( mbr == NULL )
 		return;
+	if( mbr->kind != CHR_KIND_MBR )
+		return;
 
-	curs_attrset_n( CURS_ATTR_N_FX_MBR_CRTCL );
-	print_msg( FLG_NULL, MSG_MBR_DIE, mbr->name );
-	curs_attrset_dflt();
+	if( !flg_chk_game_over )
+		return;
 
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		if( chk_flg_or( party.mbr[i]->stat,
@@ -1116,6 +1567,9 @@ void	die_mbr( mbr_t *mbr )
 
 bool_t	chk_can_act_mbr( mbr_t *p )
 {
+	if( p == NULL )
+		return FALSE;
+
 	if( chk_flg_or( p->stat,
 			FLG_STAT_NOT_EXIST | FLG_STAT_DEAD
 			| FLG_STAT_STONE | FLG_STAT_PARALYZE
@@ -1128,19 +1582,24 @@ bool_t	chk_can_act_mbr( mbr_t *p )
 
 /**/
 
-void	set_mbr_act_auto_mnstr( mbr_t *mbr, long x, long y )
+bool_t	set_mbr_act_auto_mnstr( mbr_t *mbr, long x, long y )
 {
 	mnstr_t	*mnstr;
 
 	if( mbr == NULL )
-		return;
+		return FALSE;
 
 	mnstr = get_mnstr( x, y );
 	if( mnstr == NULL )
-		return;
+		return FALSE;
+
+	if( mnstr->attitude != ATTITUDE_ENEMY )
+		return FALSE;
 
 	mark_mnstr( mbr, mnstr, DIST_NEAR );
 	set_chr_act( mbr, ACT_KIND_MNSTR_FIGHT, NULL, NULL, 0, 0 );
+
+	return TRUE;
 }
 
 /**/
@@ -1173,7 +1632,7 @@ void	mbr_peep_door( mbr_t *mbr, long dr_n )
 	set_flg_find_all_mnstr();
 	draw_all_mnstr();
 
-	wait_key_ex( TRUE );
+	wait_key_ex( TRUE, TRUE );
 
 	/**/
 
@@ -1193,157 +1652,6 @@ void	mbr_peep_door( mbr_t *mbr, long dr_n )
 
 /**/
 
-void	mbr_srch_door( mbr_t *mbr, long dr_n )
-{
-	dun_t	*dun;
-
-	if( mbr == NULL )
-		return;
-
-	dun = get_dun();
-
-	if( chk_flg( dun->door[dr_n].flg, FLG_DOOR_SHOP ) ){
-		char	*name;
-
-		name = get_shop_name( get_shop_n( dun->door[dr_n].mnr ) );
-		print_msg( FLG_NULL, MSG_SRCH_DOOR_SHOP, name );
-	}
-
-	if( chk_flg( dun->door[dr_n].flg, FLG_DOOR_OPEN ) )
-		print_msg( FLG_NULL, MSG_SRCH_DOOR_OPEN );
-	else
-		print_msg( FLG_NULL, MSG_SRCH_DOOR_CLOSE );
-
-	if( chk_flg( dun->door[dr_n].flg, FLG_DOOR_JAMMED ) )
-		print_msg( FLG_NULL, MSG_SRCH_DOOR_JAMMED );
-
-	if( chk_flg( dun->door[dr_n].flg, FLG_DOOR_BROKEN ) )
-		print_msg( FLG_NULL, MSG_SRCH_DOOR_BROKEN );
-
-	dun->door[dr_n].flg |= FLG_DOOR_SRCHED;
-	dun->door[dr_n].flg &= ~(FLG_DOOR_SECRET);
-
-	set_face_door( dr_n );
-	draw_door( dr_n );
-}
-
-/**/
-
-void	mbr_srch_trap( mbr_t *mbr, trap_t *trap )
-{
-	dun_t	*dun;
-
-	if( mbr == NULL )
-		return;
-	if( trap == NULL )
-		return;
-	if( trap->tab == NULL )
-		return;
-
-	dun = get_dun();
-
-	print_msg( FLG_NULL, MSG_SRCH_TRAP, trap->tab->name );
-}
-
-/**/
-
-void	mbr_srch_item( mbr_t *mbr, item_t *item )
-{
-	print_msg( FLG_NULL, MSG_SRCH_ITEM, item->name );
-}
-
-/**/
-
-void	mbr_srch_obj( mbr_t *mbr )
-{
-	party_t	*pty = get_party();
-	long	dx, dy;
-	long	x, y;
-	bool_t	flg_find;
-
-	if( mbr == NULL )
-		return;
-
-	flg_find = FALSE;
-
-	x = mbr->x;
-	y = mbr->y;
-	for( dy = -1; dy <= +1; dy++ ){
-		if( !clip_y( y + dy ) )
-			continue;
-		for( dx = -1; dx <= +1; dx++ ){
-			if( !clip_x( x + dx ) )
-				continue;
-
-			if( mbr_srch_obj_sub( mbr, x + dx, y + dy ) )
-				flg_find = TRUE;
-		}
-	}
-
-	if( flg_find && chk_flg( pty->flg, FLG_PARTY_SRCH_AUTO_OFF ) ){
-		long	i;
-
-		for( i = 0; i < MBR_MAX_N; i++ ){
-			if( chk_flg_or( pty->mbr[i]->stat,
-					FLG_STAT_NOT_EXIST
-					| FLG_STAT_DEAD ) ){
-				continue;
-			}
-
-			if( pty->mbr[i]->act.kind == ACT_KIND_SRCH ){
-				clr_chr_trgt_act( pty->mbr[i], FALSE );
-			}
-		}
-	}
-}
-
-/**/
-
-bool_t	mbr_srch_obj_sub( mbr_t *mbr, long x, long y )
-{
-	dun_t	*dun = get_dun();
-	bool_t	flg_find;
-	char	mjr, mnr;
-	flg_map_t	flg;
-
-	flg_find = FALSE;
-
-	mjr = dun->map.obj.mjr[y][x];
-	mnr = dun->map.obj.mnr[y][x];
-	flg = dun->map.obj.flg[y][x];
-
-	if( (mjr == FACE_MJR_DOOR_CLOSE) || (mjr == FACE_MJR_DOOR_OPEN) ){
-		door_t	*dr;
-
-		dr = get_door( x, y );
-		if( dr != NULL ){
-			if( chk_flg( dr->flg, FLG_DOOR_SECRET ) ){
-				flg_find = TRUE;
-				print_msg( FLG_NULL, MSG_DOOR_SRCH,
-						mbr->name );
-			}
-			srch_door( dr->n );
-		}
-	}
-	if( mjr == FACE_MJR_TRAP ){
-		trap_t	*trap;
-
-		trap = get_trap( x, y );
-		if( trap != NULL ){
-			if( chk_flg( flg, FLG_MAP_OBJ_LOOK_FLOOR ) ){
-				flg_find = TRUE;
-				print_msg( FLG_NULL, MSG_TRAP_SRCH,
-						mbr->name );
-			}
-			find_trap( trap );
-		}
-	}
-
-	return flg_find;
-}
-
-/**/
-
 void	clr_party_act( void )
 {
 	party.act.kind = ACT_KIND_NULL;
@@ -1353,11 +1661,17 @@ void	clr_party_act( void )
 
 void	join_mbr( long ls_mbr_n, long mbr_n )
 {
+	long	x, y;
+
 	if( mbr_n == MBR_N_NOT_JOIN )
 		return;
 
 	party.mbr[mbr_n] = ls_mbr[ls_mbr_n];
 	ls_mbr[ls_mbr_n]->mbr_n = mbr_n;
+
+	x = party.square.x + party.square.chr_pos[mbr_n].x;
+	y = party.square.y + party.square.chr_pos[mbr_n].y;
+	put_chr( party.mbr[mbr_n], x, y, TRUE );
 }
 
 /**/
@@ -1367,33 +1681,56 @@ void	not_join_mbr( long ls_mbr_n, long mbr_n )
 	if( mbr_n == MBR_N_NOT_JOIN )
 		return;
 
+	clr_map_chr( party.mbr[mbr_n] );
+	draw_chr( party.mbr[mbr_n] );
+
+	party.mbr[mbr_n]->x = MAP_DEL_X;
+	party.mbr[mbr_n]->y = MAP_DEL_Y;
+
 	ls_mbr[ls_mbr_n]->mbr_n = MBR_N_NOT_JOIN;
-	party.mbr[mbr_n] = ls_mbr[LS_MBR_MAX_N];
+	party.mbr[mbr_n] = ls_mbr[LS_MBR_NULL];
 }
 
 /**/
 
 void	collect_gold( mbr_t *mbr )
 {
+	bool_t	flg_force;
 	gold_t	gold;
 	long	i;
 
 	if( mbr == NULL )
 		return;
-	if( mbr->fx_data.friend_turn > 0 ){
-		print_msg( FLG_NULL, MSG_NOT_FRIEND_GOLD,
-				mbr->name );
-		return;
+
+	flg_force = FALSE;
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		if( !chk_flg( party.mbr[i]->stat, FLG_STAT_DEAD ) ){
+			/* ѤGOLDϵݽʤ */
+
+			flg_force = TRUE;
+			break;
+		}
+	}
+
+	if( party.mbr[i]->fx_data.friend_turn > 0 ){
+		if( !flg_force ){
+			print_msg( FLG_NULL, MSG_NOT_FRIEND_GOLD,
+					mbr->name );
+			return;
+		}
 	}
 
 	gold = 0;
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		if( chk_flg( party.mbr[i]->stat, FLG_STAT_NOT_EXIST ) )
 			continue;
+
 		if( party.mbr[i]->fx_data.friend_turn > 0 ){
-			print_msg( FLG_NULL, MSG_NOT_FRIEND_GOLD,
-					party.mbr[i]->name );
-			continue;
+			if( !flg_force ){
+				print_msg( FLG_NULL, MSG_NOT_FRIEND_GOLD,
+						party.mbr[i]->name );
+				continue;
+			}
 		}
 
 		gold += party.mbr[i]->gold;
@@ -1409,49 +1746,100 @@ void	collect_gold( mbr_t *mbr )
 
 void	divide_gold( void )
 {
-	long	sum, gold;
-	long	i, n, n2;
+	bool_t	flg_force;
+	gold_t	sum, gold;
+	long	i, n, mod_n;
 
+	flg_force = FALSE;
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		if( !chk_flg( party.mbr[i]->stat, FLG_STAT_DEAD ) ){
+			/* ѤGOLDϵݽʤ */
+
+			flg_force = TRUE;
+			break;
+		}
+	}
+
+	mod_n = 0;
 	n = 0;
 	sum = 0;
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		if( chk_flg( party.mbr[i]->stat, FLG_STAT_NOT_EXIST ) )
 			continue;
+
 		if( party.mbr[i]->fx_data.friend_turn > 0 ){
-			print_msg( FLG_NULL, MSG_NOT_FRIEND_GOLD,
-					party.mbr[i]->name );
-			continue;
+			if( !flg_force ){
+				print_msg( FLG_NULL, MSG_NOT_FRIEND_GOLD,
+						party.mbr[i]->name );
+				continue;
+			}
 		}
 
 		sum += party.mbr[i]->gold;
+		party.mbr[i]->gold = 0;
 		n++;
+		if( per_randm( n ) )
+			mod_n = i;
 	}
 	if( n <= 0 )
 		return;
 
-	n2 = 0;
 	gold = sum / n;
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		if( chk_flg( party.mbr[i]->stat, FLG_STAT_NOT_EXIST ) )
 			continue;
 		if( party.mbr[i]->fx_data.friend_turn > 0 )
-			continue;
+			if( !flg_force )
+				continue;
 
-		party.mbr[i]->gold = gold;
+		party.mbr[i]->gold += gold;
 		sum -= gold;
-
-		n2++;
-		if( n2 == n ){
-			party.mbr[i]->gold += sum;
-			break;
-		}
 	}
 
+	party.mbr[mod_n]->gold += sum;
+
 	print_msg( FLG_NULL, MSG_DIVIDE_GOLD, gold );
 }
 
 /**/
 
+void	add_party_gold( gold_t gold )
+{
+	gold_t	div_gold;
+	long	i, n, mod_n;
+
+	mod_n = 0;
+	n = 0;
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		if( chk_flg( party.mbr[i]->stat, FLG_STAT_NOT_EXIST ) )
+			continue;
+
+		n++;
+		if( per_randm( n ) )
+			mod_n = i;
+	}
+	if( n <= 0 )
+		return;
+
+	curs_attrset_chr( party.mbr[mod_n], +1, FALSE );
+	print_msg( FLG_NULL, MSG_ADD_PARTY_GOLD, gold );
+	curs_attrset_dflt();
+
+	div_gold = gold / n;
+
+	for( i = 0; i < MBR_MAX_N; i++ ){
+		if( chk_flg( party.mbr[i]->stat, FLG_STAT_NOT_EXIST ) )
+			continue;
+
+		party.mbr[i]->gold += div_gold;
+		gold -= div_gold;
+	}
+
+	party.mbr[mod_n]->gold += gold;
+}
+
+/**/
+
 void	exp_party( long exp, bool_t flg_draw )
 {
 	party_t	*pty = get_party();
@@ -1470,19 +1858,9 @@ void	exp_party( long exp, bool_t flg_draw )
 			continue;
 
 		n++;
-		if( randm( n ) == 0 )
+		if( per_randm( n ) )
 			mod_n = i;
 	}
-	for( i = 0; i < PET_MAX_N; i++ ){
-		if( pet[i] == NULL )
-			continue;
-		if( chk_flg_or( pet[i]->stat, flg ) )
-			continue;
-
-		n++;
-		if( randm( n ) == 0 )
-			mod_n = MBR_MAX_N + i;
-	}
 	if( n <= 0 )
 		return;
 
@@ -1490,11 +1868,14 @@ void	exp_party( long exp, bool_t flg_draw )
 	exp_mod = exp % n;
 
 	if( flg_draw ){
-		curs_attrset_n( CURS_ATTR_N_FX_MBR_PLUS );
+		curs_attrset_chr( get_mbr_randm( MAP_DEL_X, MAP_DEL_Y ),
+				+1, FALSE );
 		print_msg( FLG_NULL, MSG_EXP_PARTY, exp_add );
 		curs_attrset_dflt();
 	}
 
+	/* С */
+
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		if( chk_flg_or( party.mbr[i]->stat, flg ) )
 			continue;
@@ -1507,19 +1888,16 @@ void	exp_party( long exp, bool_t flg_draw )
 					FALSE, TRUE );
 		}
 	}
+
+	/* ڥå */
+
 	for( i = 0; i < PET_MAX_N; i++ ){
 		if( pet[i] == NULL )
 			continue;
 		if( chk_flg_or( pet[i]->stat, flg ) )
 			continue;
 
-		if( mod_n == MBR_MAX_N + i ){
-			exp_chr( pet[i], exp_add + exp_mod,
-					FALSE, TRUE );
-		} else {
-			exp_chr( pet[i], exp_add,
-					FALSE, TRUE );
-		}
+		exp_chr( pet[i], exp_add, FALSE, TRUE );
 	}
 }
 
@@ -1529,7 +1907,7 @@ void	remove_mbr( mbr_t *mbr )
 {
 	long	i;
 	long	mbr_n, ls_mbr_n;
-	mbr_t	*tmp_mbr;
+	mbr_t	*tmp_mbr, *p;
 
 	if( mbr == NULL )
 		return;
@@ -1541,20 +1919,32 @@ void	remove_mbr( mbr_t *mbr )
 
 	remove_mbr_del_item( ls_mbr_n );
 
+	/* ls_mbr */
+
+	p = ls_mbr[ls_mbr_n];
+	clr_map_chr( p );
+	p->x = MAP_DEL_X;
+	p->y = MAP_DEL_Y;
+	draw_mbr( p );
+	release_chr( p );
+	free_fx_all( &(p->fx) );
+	lost_trgt( p );
+	lost_fx_chr( p );
+
 	/* party.mbr */
 
 	if( mbr_n != MBR_N_NOT_JOIN )
-		party.mbr[mbr_n] = ls_mbr[LS_MBR_MAX_N];
+		party.mbr[mbr_n] = ls_mbr[LS_MBR_NULL];
 
-	/* ls_mbr */
+	/* ꥹȤ򤺤餹 */
 
-	free_fx_all( &(ls_mbr[ls_mbr_n]->fx) );
 	tmp_mbr = ls_mbr[ls_mbr_n];
 	for( i = ls_mbr_n; i < LS_MBR_MAX_N - 1; i++ ){
 		ls_mbr[i] = ls_mbr[i + 1];
 		ls_mbr[i]->ls_mbr_n--;
 	}
 	ls_mbr[i] = tmp_mbr;
+
 	ls_mbr[i]->mbr_n = MBR_N_NOT_JOIN;
 	ls_mbr[i]->stat |= FLG_STAT_NOT_EXIST;
 }
@@ -1746,15 +2136,17 @@ mbr_t	*get_mbr_randm( long x, long y )
 			continue;
 		}
 
-		pos1.x = p->x;
-		pos1.y = p->y;
-		pos2.x = x;
-		pos2.y = y;
-		if( !chk_find( &pos1, &pos2 ) )
-			continue;
+		if( clip_pos( x, y ) ){
+			pos1.x = p->x;
+			pos1.y = p->y;
+			pos2.x = x;
+			pos2.y = y;
+			if( !chk_find( &pos1, &pos2 ) )
+				continue;
+		}
 
 		n++;
-		if( randm( n ) == 0 )
+		if( per_randm( n ) )
 			ret_p = p;
 	}
 
@@ -1952,62 +2344,13 @@ char	*get_race_str_sub( race_t *race )
 char	*get_race_name( race_t *race )
 {
 	if( race == NULL )
-		return "";
-
-#if	0
-/*@@@*/
-	if( race->sub == SUB_RACE_NULL ){
-		switch( race->main ){
-		case MAIN_RACE_HUMAN:
-			return MSG_MAIN_RACE_HUMAN;
-		case MAIN_RACE_HALF_ELF:
-			return MSG_MAIN_RACE_HALF_ELF;
-		case MAIN_RACE_ELF:
-			return MSG_MAIN_RACE_ELF;
-		case MAIN_RACE_DWARF:
-			return MSG_MAIN_RACE_DWARF;
-		case MAIN_RACE_VIVID:
-			return MSG_MAIN_RACE_VIVID;
-		case MAIN_RACE_MAX_N:
-			break;
-		}
-	}
-#endif
+		return MSG_NULL;
 
 	return race->name;
 }
 
 /**/
 
-long	add_hp_mbr( mbr_t *mbr, long n )
-{
-	long	pre_hp, add;
-
-	if( mbr == NULL )
-		return 0;
-
-	pre_hp = mbr->abl.hp.n;
-
-	mbr->abl.hp.n += n;
-	if( mbr->abl.hp.n > mbr->abl.hp.max )
-		mbr->abl.hp.n = mbr->abl.hp.max;
-	if( mbr->abl.hp.n < 0 )
-		mbr->abl.hp.n = 0;
-
-	add = mbr->abl.hp.n - pre_hp;
-	if( add != 0 ){
-		vfx_num( mbr, add );
-		draw_mbr_stat( mbr );
-	}
-
-	if( mbr->abl.hp.n <= 0 )
-		die_chr( mbr, FALSE );
-
-	return mbr->abl.hp.n;
-}
-
-/**/
-
 bool_t	chk_not_exist_all_mbr( void )
 {
 	long	i;
@@ -2049,10 +2392,19 @@ long	get_mbr_n( long x, long y )
 				FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) ){
 			continue;
 		}
-		if( (mbr->x == x) && (mbr->y == y) ){
-			return n;
-		}
+
+		if( x < get_chr_left( mbr->x, mbr->dx ) )
+			continue;
+		if( x > get_chr_right( mbr->x, mbr->dx ) )
+			continue;
+		if( y < get_chr_top( mbr->y, mbr->dy ) )
+			continue;
+		if( y > get_chr_bottom( mbr->y, mbr->dy ) )
+			continue;
+
+		return n;
 	}
+
 	return MBR_N_NOT_JOIN;
 }
 
