@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,17 +25,24 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: pet.h,v 1.3 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: pet.h,v 1.7 2004/06/30 13:20:45 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	PET_H
 #define	PET_H
 
+/* ڥåȡȤ⡦ */
+
 #include	"mnstr.h"
 
-#define	PET_MAX_N	MBR_MAX_N
+#define	PET_MAX_N	MBR_MAX_N	/* ѡƥϢƹԤ */
+
+/* ڥåȥåפ¤ƤԤޤ᤿ */
 #define	LS_PET_MAX_N	LS_MBR_MAX_N
 
+/* ڥå̾κʸ */
+#define	PET_NAME_MAX_LEN	MBR_NAME_MAX_LEN
+
 typedef mnstr_t	pet_t;
 typedef mnstr_kind_t	pet_kind_t;
 
