@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: resi.h,v 1.9 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: resi.h,v 1.14 2004/06/30 13:20:45 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	RESI_H
@@ -34,21 +33,27 @@
 
 /*  */
 
+#define	RESI_PERFECT_N	30000	/*  */
+#define	RESI_MAX_N	(RESI_PERFECT_N - 1)	/*  */
+
 typedef enum {
-	RESI_KIND_KNOC,
-	RESI_KIND_SLAS,
-	RESI_KIND_STIN,
-	RESI_KIND_HEAT,
-	RESI_KIND_COLD,
-	RESI_KIND_ACID,
-	RESI_KIND_MIND,
-	RESI_KIND_ELEC,
-	RESI_KIND_POIS,
+	RESI_KIND_KNOC,	/*  */
+	RESI_KIND_SLAS,	/* · */
+	RESI_KIND_STIN,	/*  */
+	RESI_KIND_HEAT,	/* Ǯ */
+	RESI_KIND_COLD,	/*  */
+	RESI_KIND_MIND,	/*  */
+	RESI_KIND_ACID,	/* ڻ */
+	RESI_KIND_ELEC,	/*  */
+	RESI_KIND_POIS,	/*  */
 	RESI_KIND_MAX_N,
 } resi_kind_t;
 
+
 typedef struct {
-	rate_t	n, max, add_n;
+	rate_t	n;	/*  */
+	rate_t	max;	/*  */
+	rate_t	add_n;	/*  */
 } resi_t;
 
 #endif	/* RESI_H */
