@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: spell.h,v 1.17 2003/11/19 13:19:13 bowkenken Exp $
+* $Id: spell.h,v 1.28 2004/07/26 10:02:47 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	SPELL_H
@@ -36,21 +35,27 @@
 #include	"chr.h"
 #include	"msg.h"
 
-#include	"spell_kind.h"
+#include	"spell-kind.h"
+#include	"vfx-kind.h"
 
 /**/
 
+/* ʸλѲġԲĤΥꥹ */
+
 typedef struct {
 	spell_kind_t	kind;
 	bool_t	flg_cast;
 } cast_tab_t;
 
+/* ʸθϰ */
+
 typedef unsigned long	flg_extent_t;
 #define	FLG_EXTENT_NORMAL	(1 << 0)
 #define	FLG_EXTENT_SPREAD	(1 << 1)
 #define	FLG_EXTENT_MASS	(1 << 2)
-#define	FLG_EXTENT_HALF	(1 << 3)
-#define	FLG_EXTENT_ALL	\
+#define	FLG_EXTENT_LIMIT	(1 << 3)
+#define	FLG_EXTENT_HALF	(1 << 4)
+#define	FLG_EXTENT_STD_ALL	\
 		(FLG_EXTENT_NORMAL \
 		| FLG_EXTENT_SPREAD \
 		| FLG_EXTENT_MASS \
@@ -59,18 +64,29 @@ typedef unsigned long	flg_extent_t;
 		(FLG_EXTENT_NORMAL \
 		| FLG_EXTENT_SPREAD \
 		| FLG_EXTENT_MASS)
+#define	FLG_EXTENT_NORMAL_HALF	\
+		(FLG_EXTENT_NORMAL \
+		| FLG_EXTENT_HALF)
+#define	FLG_EXTENT_BAR	\
+		(FLG_EXTENT_NORMAL \
+		| FLG_EXTENT_LIMIT)
+
+/* ʸΥǡ */
 
 typedef struct {
 	spell_kind_t	kind;
 	n_msg_t	n_name;
 	char	*name;
-	long	hp, mp;
-	gold_t	gold;
-	abl_kind_t	abl;
-	flg_extent_t	flg_extent;
-	bool_t	flg_trgt;
-	resi_kind_t	resi;
-	gold_t	price;
+	long	hp, mp;	/*  HP, MP */
+	gold_t	gold;	/*  */
+	rate_t	resi_down;	/*  */
+	long	lev;	/* Ѳǽ٥ */
+	abl_kind_t	abl;	/* ɬǽ */
+	flg_extent_t	flg_extent;	/* ϰ */
+	bool_t	flg_trgt;	/* оݤꤹ롦ʤե饰 */
+	resi_kind_t	resi;	/* ɸ椹¦ */
+	gold_t	add_price;	/* ʤν */
+	vfx_kind_t	vfx_kind;	/* и */
 } spell_tab_t;
 
 #endif	/* SPELL_H */
