@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: tmenu.c,v 1.27 2003/11/23 05:13:28 bowkenken Exp $
+* $Id: tmenu.c,v 1.65 2004/07/26 10:02:47 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -43,32 +42,36 @@
 /*#include	"fight.h"*/
 /*#include	"fx.h"*/
 /*#include	"trap.h"*/
-/*#include	"draw.h"*/
-/*#include	"curs.h"*/
+#include	"draw.h"
+#include	"curs.h"
 #include	"menu.h"
+/*#include	"amenu.h"*/
+/*#include	"request.h"*/
+#include	"gfile.h"
 /*#include	"msg.h"*/
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-/*#include	"turn_prot.h"*/
-#include	"dun_prot.h"
-#include	"town_prot.h"
-#include	"item_prot.h"
-#include	"spell_prot.h"
-#include	"chr_prot.h"
-#include	"party_prot.h"
-#include	"mnstr_prot.h"
-#include	"pet_prot.h"
-/*#include	"fight_prot.h"*/
-#include	"fx_prot.h"
-/*#include	"trap_prot.h"*/
-#include	"draw_prot.h"
-/*#include	"curs_prot.h"*/
-#include	"menu_prot.h"
-/*#include	"tmenu_prot.h"*/
-/*#include	"amenu_prot.h"*/
-#include	"gfile_prot.h"
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+/*#include	"turn-prot.h"*/
+#include	"dun-prot.h"
+#include	"town-prot.h"
+#include	"item-prot.h"
+#include	"spell-prot.h"
+#include	"chr-prot.h"
+#include	"party-prot.h"
+#include	"mnstr-prot.h"
+#include	"pet-prot.h"
+/*#include	"fight-prot.h"*/
+#include	"fx-prot.h"
+/*#include	"trap-prot.h"*/
+#include	"draw-prot.h"
+#include	"curs-prot.h"
+#include	"menu-prot.h"
+/*#include	"tmenu-prot.h"*/
+/*#include	"amenu-prot.h"*/
+/*#include	"request-prot.h"*/
+#include	"gfile-prot.h"
+/*#include	"msg-prot.h"*/
 
 /**/
 
@@ -85,13 +88,13 @@
 		((LS_MBR_MAX_N + LS_MBR_MAX_N_PER_PAGE - 1)\
 		/ LS_MBR_MAX_N_PER_PAGE)
 
-#define	CHARGE_STAY_INN	10
-
 #define	ORDER_MILK_LEV	20
 
+#define	PET_DUN_LEV	10
+
 /**/
 
-#include	"tmenu_tab.h"
+#include	"tmenu-tab.h"
 
 /**/
 
@@ -117,7 +120,7 @@ void	init_tmenu( void )
 
 /**/
 
-char	*menu_shop_sel_mbr( shop_n_t shop_n )
+char	*menu_shop_sel_mbr( shop_n_t shop_n, bool_t flg_pet )
 {
 	bool_t	flg_sel_dead;
 
@@ -137,7 +140,7 @@ char	*menu_shop_sel_mbr( shop_n_t shop_n )
 				get_shop_name( g_shop_n ) );
 
 		str_goto = exec_menu_sel_mbr(
-				&g_sel_mbr, ttl, flg_sel_dead );
+				&g_sel_mbr, ttl, flg_sel_dead, flg_pet );
 		if( str_goto != NULL )
 			return str_goto;
 
@@ -149,18 +152,18 @@ char	*menu_shop_sel_mbr( shop_n_t shop_n )
 			case SHOP_N_BAR:
 			case SHOP_N_RESTAURANT:
 				return STR_MENU_ERR;
+			case SHOP_N_MAGIC:
+				str_goto = exec_menu( ls_menu_magic_guild,
+						NULL, str_goto );
 				break;
 			case SHOP_N_TEMPLE:
 				str_goto = exec_menu( ls_menu_temple,
 						NULL, str_goto );
 				break;
-			case SHOP_N_MAGIC:
-				str_goto = exec_menu( ls_menu_magic_guild,
-						NULL, str_goto );
-				break;
 			case SHOP_N_WPN:
 			case SHOP_N_ARMOR:
 			case SHOP_N_ALCHEMY:
+			case SHOP_N_MUSIC:
 			case SHOP_N_GROCERY:
 			case SHOP_N_TEAROOM:
 			case SHOP_N_TOBACCO:
@@ -198,7 +201,7 @@ char	*cb_menu_temple_init( menu_t **pp, menu_t **dflt )
 		long	tmp_len;
 
 		tmp_len = str_len_std( MSG( p->n_msg ) );
-		len = max( len, tmp_len );
+		len = max_l( len, tmp_len );
 	}
 
 	p = srch_menu( *pp, "Title" );
@@ -237,12 +240,16 @@ char	*cb_menu_temple_init( menu_t **pp, menu_t **dflt )
 char	*cb_menu_temple_remove_curse( menu_t **pp, menu_t **dflt )
 {
 	gold_t	price;
+	chr_t	*chr_pay;
 	ask_t	ask;
 
 	price = get_price_temple_remove_curse( g_sel_mbr );
-	if( price > 0 )
-		if( !chk_pay( price, g_sel_mbr ) )
-			return STR_MENU_CANCEL;
+	if( is_pet( g_sel_mbr ) )
+		chr_pay = g_sel_mbr->owner;
+	else
+		chr_pay = g_sel_mbr;
+	if( !chk_pay( chr_pay, price ) )
+		return STR_MENU_CANCEL;
 
 	ask = exec_menu_ask( MSG_TTL_MENU_TEMPLE_REMOVE_CURSE,
 			ASK_NO, TRUE );
@@ -259,7 +266,7 @@ char	*cb_menu_temple_remove_curse( menu_t **pp, menu_t **dflt )
 	}
 
 	if( remove_curse( NULL, g_sel_mbr, 0 ) ){
-		add_chr_gold( g_sel_mbr, -price );
+		add_chr_gold( chr_pay, -price );
 
 		print_msg( FLG_NULL,
 				MSG_MENU_REMOVE_CURSE,
@@ -278,12 +285,16 @@ char	*cb_menu_temple_remove_curse( menu_t **pp, menu_t **dflt )
 char	*cb_menu_temple_cure_status( menu_t **pp, menu_t **dflt )
 {
 	gold_t	price;
+	chr_t	*chr_pay;
 	ask_t	ask;
 
 	price = get_price_temple_cure_status( g_sel_mbr );
-	if( price > 0 )
-		if( !chk_pay( price, g_sel_mbr ) )
-			return STR_MENU_CANCEL;
+	if( is_pet( g_sel_mbr ) )
+		chr_pay = g_sel_mbr->owner;
+	else
+		chr_pay = g_sel_mbr;
+	if( !chk_pay( chr_pay, price ) )
+		return STR_MENU_CANCEL;
 
 	ask = exec_menu_ask( MSG_TTL_MENU_TEMPLE_CURE_STATUS,
 			ASK_NO, TRUE );
@@ -300,7 +311,7 @@ char	*cb_menu_temple_cure_status( menu_t **pp, menu_t **dflt )
 	}
 
 	if( fx_cure_all_status( g_sel_mbr ) ){
-		add_chr_gold( g_sel_mbr, -price );
+		add_chr_gold( chr_pay, -price );
 
 		print_msg( FLG_NULL,
 				MSG_MENU_CURE_STATUS,
@@ -319,12 +330,28 @@ char	*cb_menu_temple_cure_status( menu_t **pp, menu_t **dflt )
 char	*cb_menu_temple_resurrection( menu_t **pp, menu_t **dflt )
 {
 	gold_t	price;
+	chr_t	*chr_pay;
 	ask_t	ask;
 
+	if( g_sel_mbr == NULL )
+		return STR_MENU_ERR;
+	if( chk_flg( g_sel_mbr->stat, FLG_STAT_NOT_EXIST ) )
+		return STR_MENU_ERR;
+
+	if( !chk_flg( g_sel_mbr->stat, FLG_STAT_DEAD ) ){
+		print_msg( FLG_NULL,
+				MSG_MENU_RESURRECTION_ERR,
+				g_sel_mbr->name );
+		return STR_MENU_CANCEL;
+	}
+
 	price = get_price_temple_resurrection( g_sel_mbr );
-	if( price > 0 )
-		if( !chk_pay( price, g_sel_mbr ) )
-			return STR_MENU_CANCEL;
+	if( is_pet( g_sel_mbr ) )
+		chr_pay = g_sel_mbr->owner;
+	else
+		chr_pay = g_sel_mbr;
+	if( !chk_pay( chr_pay, price ) )
+		return STR_MENU_CANCEL;
 
 	ask = exec_menu_ask( MSG_TTL_MENU_TEMPLE_RESURRECTION,
 			ASK_NO, TRUE );
@@ -340,17 +367,10 @@ char	*cb_menu_temple_resurrection( menu_t **pp, menu_t **dflt )
 		return STR_MENU_ERR;
 	}
 
-	if( resurrection( g_sel_mbr ) ){
-		add_chr_gold( g_sel_mbr, -price );
+	fx_resurrection( g_sel_mbr, FALSE );
 
-		print_msg( FLG_NULL,
-				MSG_MENU_RESURRECTION,
-				g_sel_mbr->name );
-	} else {
-		print_msg( FLG_NULL,
-				MSG_MENU_RESURRECTION_ERR,
-				g_sel_mbr->name );
-	}
+	/* ֤ʤƤʧ */
+	add_chr_gold( chr_pay, -price );
 
 	return "/Temple";
 }
@@ -359,21 +379,27 @@ char	*cb_menu_temple_resurrection( menu_t **pp, menu_t **dflt )
 
 char	*cb_menu_temple_contribution( menu_t **pp, menu_t **dflt )
 {
-	gold_t	price;
+	gold_t	price, spell_price;
+	chr_t	*chr_pay;
 	char	*str_goto;
 	ask_t	ask;
 
 	if( g_sel_mbr->gold <= 0 )
 		return STR_MENU_CANCEL;
 
-	price = g_sel_mbr->gold * 10 / _100_PERCENT;
+	spell_price = get_price_temple_contribution( g_sel_mbr );
+	price = spell_price;
+
 	str_goto = exec_menu_sel_num( 1, g_sel_mbr->gold, &price );
 	if( str_goto != NULL )
 		return str_goto;
 
-	if( price > 0 )
-		if( !chk_pay( price, g_sel_mbr ) )
-			return STR_MENU_CANCEL;
+	if( is_pet( g_sel_mbr ) )
+		chr_pay = g_sel_mbr->owner;
+	else
+		chr_pay = g_sel_mbr;
+	if( !chk_pay( chr_pay, price ) )
+		return STR_MENU_CANCEL;
 
 	ask = exec_menu_ask( MSG_TTL_MENU_TEMPLE_CONTRIBUTION,
 			ASK_NO, TRUE );
@@ -389,11 +415,10 @@ char	*cb_menu_temple_contribution( menu_t **pp, menu_t **dflt )
 		return STR_MENU_ERR;
 	}
 
-	add_chr_gold( g_sel_mbr, -price );
-
-	contribution( g_sel_mbr, price );
+	add_chr_gold( chr_pay, -price );
 	print_msg( FLG_NULL, MSG_MENU_CONTRIBUTION,
 			g_sel_mbr->name, price );
+	contribution( g_sel_mbr, price, spell_price );
 
 	return "/Temple";
 }
@@ -452,8 +477,8 @@ char	*menu_order_sel_mbr( void )
 	party_t	*pty = get_party();
 	mbr_t	**mbr;
 	menu_t	*p, *dflt;
-	long	i, k;
 	long	mbr_n;
+	long	i, k;
 
 	p = ls_menu_order_sel_mbr;
 	dflt = NULL;
@@ -498,7 +523,7 @@ char	*menu_order_sel_mbr( void )
 
 		sprintf( p[k].path, "/Order/%ld", i );
 		strncpy( p[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
-		p[k].key = 'A' + i;
+		p[k].key = (char)('A' + i);
 		p[k].accel_key[0] = '\0';
 		if( g_mbr_order[i] == NULL )
 			name = MSG_MENU_ORDER_NULL;
@@ -557,10 +582,13 @@ char	*cb_menu_order_sel( menu_t **pp, menu_t **dflt )
 char	*cb_menu_order_chose( menu_t **pp, menu_t **dflt )
 {
 	party_t	*pty = get_party();
+	bool_t	flg_rumor;
 	long	i;
 
 	g_sel_mbr = NULL;
 
+	flg_rumor = FALSE;
+
 	for( i = 0; i < MBR_MAX_N; i++ ){
 		item_t	*item, *tmp;
 		gold_t	price;
@@ -568,23 +596,31 @@ char	*cb_menu_order_chose( menu_t **pp, menu_t **dflt )
 		item = g_mbr_order[i];
 		if( item == NULL )
 			continue;
+		else
+			flg_rumor = TRUE;
 
 		price = get_price( g_shop_n, item, DEAL_KIND_BUY );
-		if( !chk_pay( price, pty->mbr[i] ) )
+		if( !chk_pay( pty->mbr[i], price ) )
 			continue;
 
+		add_chr_gold( pty->mbr[i], -price );
+
 		tmp = alloc_item();
 		if( tmp == NULL )
 			return STR_MENU_ERR;
 		copy_item( tmp, item );
 		tmp->stock = 1;
 
+		pty->mbr[i]->x = 1;
+		pty->mbr[i]->y = 1;
 		if( tmp->kind == ITEM_KIND_FOOD ){
 			eat_food( pty->mbr[i], tmp );
 		} else if( tmp->kind == ITEM_KIND_POTION ){
 			quaff_potion( pty->mbr[i], tmp, TRUE );
 			chk_order_milk( pty->mbr[i], g_shop_n, item );
 		}
+		pty->mbr[i]->x = MAP_DEL_X;
+		pty->mbr[i]->y = MAP_DEL_Y;
 
 		item->stock--;
 		if( item->stock <= 0 )
@@ -593,6 +629,9 @@ char	*cb_menu_order_chose( menu_t **pp, menu_t **dflt )
 		g_mbr_order[i] = NULL;
 	}
 
+	if( flg_rumor )
+		hear_rumor();
+
 	return "/Order";
 }
 
@@ -614,7 +653,10 @@ bool_t	chk_order_milk( mbr_t *mbr, shop_n_t shop_n, item_t *item )
 	if( item->dat.potion.kind != POTION_KIND_MILK )
 		return FALSE;
 
-	print_words( MSG_NPC_NAME_BAR, MSG_WORDS_MALE_ORDER_MILK );
+	print_words( MSG_NPC_NAME_BAR_CUSTOMER, MALE,
+			MSG_WORDS_MALE_ORDER_MILK,
+			MSG_WORDS_MALE_ORDER_MILK );
+
 	return TRUE;
 }
 
@@ -705,7 +747,7 @@ char	*menu_shop( shop_n_t shop_n, mbr_t *mbr, char *str_goto )
 		ls_art_n++;
 
 		len = str_len_std( art->art_name );
-		max_len = max( len, max_len );
+		max_len = max_l( len, max_len );
 
 		/* menu */
 
@@ -714,7 +756,7 @@ char	*menu_shop( shop_n_t shop_n, mbr_t *mbr, char *str_goto )
 
 		sprintf( ls[k].path, "/Shop/%ld", key_n );
 		strncpy( ls[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
-		ls[k].key = 'A' + key_n;
+		ls[k].key = (char)('A' + key_n);
 		ls[k].accel_key[0] = '\0';
 		ls[k].msg[0] = '\0';
 		ls[k].call_back = cb_menu_shop;
@@ -1030,8 +1072,12 @@ bool_t	menu_shop_buy(
 	item_t	*tmp;
 	gold_t	price;
 
+	price = get_price( g_shop_n, item, DEAL_KIND_BUY );
+
 	/* ̾ɽ */
 	print_msg( FLG_NULL, MSG_MENU_BUY, item->name );
+	if( !chk_pay( g_sel_mbr, price ) )
+		return FALSE;
 
 	/* 㤦ǧ */
 	ask = exec_menu_ask( MSG_TTL_MENU_BUY, ASK_NO, FALSE );
@@ -1044,9 +1090,7 @@ bool_t	menu_shop_buy(
 		return FALSE;
 	}
 
-	price = get_price( g_shop_n, item, DEAL_KIND_BUY );
-	if( !chk_pay( price, g_sel_mbr ) )
-		return FALSE;
+	add_chr_gold( g_sel_mbr, -price );
 
 	tmp = alloc_item();
 	if( tmp == NULL )
@@ -1133,6 +1177,9 @@ bool_t	menu_shop_sale(
 	case SHOP_N_ALCHEMY:
 		asgn = get_shop_alchemy_item_asgn();
 		break;
+	case SHOP_N_MUSIC:
+		asgn = get_shop_music_item_asgn();
+		break;
 	case SHOP_N_GROCERY:
 		asgn = get_shop_grocery_item_asgn();
 		break;
@@ -1181,12 +1228,6 @@ item_t	*get_begin_article(
 				return NULL;
 			return art_head->next;
 			break;
-		case SHOP_N_TEMPLE:
-			art_head = get_shop_temple_item_asgn();
-			if( art_head == NULL )
-				return NULL;
-			return art_head->next;
-			break;
 		case SHOP_N_WPN:
 			art_head = get_shop_wpn_item_asgn();
 			if( art_head == NULL )
@@ -1201,12 +1242,24 @@ item_t	*get_begin_article(
 			break;
 		case SHOP_N_MAGIC:
 			return NULL;
+		case SHOP_N_TEMPLE:
+			art_head = get_shop_temple_item_asgn();
+			if( art_head == NULL )
+				return NULL;
+			return art_head->next;
+			break;
 		case SHOP_N_ALCHEMY:
 			art_head = get_shop_alchemy_item_asgn();
 			if( art_head == NULL )
 				return NULL;
 			return art_head->next;
 			break;
+		case SHOP_N_MUSIC:
+			art_head = get_shop_music_item_asgn();
+			if( art_head == NULL )
+				return NULL;
+			return art_head->next;
+			break;
 		case SHOP_N_GROCERY:
 			art_head = get_shop_grocery_item_asgn();
 			if( art_head == NULL )
@@ -1304,8 +1357,6 @@ bool_t	chk_deal_article(
 		if( art->kind == ITEM_KIND_FOOD )
 			return TRUE;
 		break;
-	case SHOP_N_TEMPLE:
-		return FALSE;
 	case SHOP_N_WPN:
 		if( art->kind == ITEM_KIND_WPN )
 			return TRUE;
@@ -1316,6 +1367,8 @@ bool_t	chk_deal_article(
 		break;
 	case SHOP_N_MAGIC:
 		return FALSE;
+	case SHOP_N_TEMPLE:
+		return FALSE;
 	case SHOP_N_ALCHEMY:
 		if( art->kind == ITEM_KIND_POTION )
 			return TRUE;
@@ -1326,6 +1379,12 @@ bool_t	chk_deal_article(
 		if( art->kind == ITEM_KIND_ACCE )
 			return TRUE;
 		break;
+	case SHOP_N_MUSIC:
+		if( art->kind == ITEM_KIND_INST )
+			return TRUE;
+		if( art->kind == ITEM_KIND_ANTHOLOGY )
+			return TRUE;
+		break;
 	case SHOP_N_GROCERY:
 		if( art->kind == ITEM_KIND_LIGHT )
 			return TRUE;
@@ -1453,7 +1512,7 @@ char	*cb_menu_bar_form( menu_t **pp, menu_t **dflt )
 			sprintf( p[k].path, "/Bar form/%ld", ls_mbr_n );
 			strncpy( p[k].kind, "<CheckItem>",
 					MENU_KIND_MAX_LEN );
-			p[k].key = 'A' + j;
+			p[k].key = (char)('A' + j);
 			p[k].accel_key[0] = '\0';
 			strncpy( p[k].msg, ls_mbr[ls_mbr_n]->name,
 					MENU_MSG_MAX_LEN );
@@ -1600,7 +1659,7 @@ char	*cb_menu_bar_join( menu_t **pp, menu_t **dflt )
 		sprintf( p[k].path, "/Join/%ld", i );
 		strncpy( p[k].kind, "<Item>",
 				MENU_KIND_MAX_LEN );
-		p[k].key = 'A' + i;
+		p[k].key = (char)('A' + i);
 		p[k].accel_key[0] = '\0';
 		sprintf( p[k].msg, "%ld", i + 1 );
 		p[k].call_back = cb_menu_bar_join_sub;
@@ -1631,7 +1690,7 @@ char	*cb_menu_bar_join_sub_init( menu_t **pp, menu_t **dflt )
 
 	clr_flg_menu( *pp, NULL, FLG_MENU_ALL );
 
-	for( i = 0; i < 4096; i++ ){
+	for( i = 0; i < LOOP_MAX_4000; i++ ){
 		long	mbr_n;
 
 		if( (*pp)[i].path[0] == '\0' )
@@ -1762,7 +1821,7 @@ char	*cb_menu_bar_edit_mbr( menu_t **pp, menu_t **dflt )
 					ls_mbr_n );
 			strncpy( p[k].kind, "<Branch>",
 					MENU_KIND_MAX_LEN );
-			p[k].key = 'A' + j;
+			p[k].key = (char)('A' + j);
 			p[k].accel_key[0] = '\0';
 			strncpy( p[k].msg, ls_mbr[ls_mbr_n]->name,
 					MENU_MSG_MAX_LEN );
@@ -1995,7 +2054,7 @@ char	*cb_menu_bar_remove_mbr( menu_t **pp, menu_t **dflt )
 					ls_mbr_n );
 			strncpy( p[k].kind, "<OpenWinItem>",
 					MENU_KIND_MAX_LEN );
-			p[k].key = 'A' + j;
+			p[k].key = (char)('A' + j);
 			p[k].accel_key[0] = '\0';
 			strncpy( p[k].msg, ls_mbr[ls_mbr_n]->name,
 					MENU_MSG_MAX_LEN );
@@ -2040,19 +2099,30 @@ char	*cb_menu_bar_remove_mbr_sub( menu_t **pp, menu_t **dflt )
 		return STR_MENU_CANCEL;
 
 	lev = get_specialist_lev( mbr );
+
+	curs_attrset_chr( mbr, -1, TRUE );
 	print_msg( FLG_NULL, MSG_MENU_REMOVE_MBR_NAME, mbr->name );
+	curs_attrset_dflt();
 	print_msg( FLG_NULL, MSG_MENU_REMOVE_MBR_LEV, lev );
 	print_msg( FLG_MSG_MORE, MSG_MENU_REMOVE_MBR_GOLD, mbr->gold );
 
 	ask = exec_menu_ask( MSG_TTL_MENU_ASK_REMOVE_MBR, ASK_NO, TRUE );
 	switch( ask ){
 	case ASK_YES:
+		curs_attrset_chr( mbr, -1, TRUE );
+		print_msg( FLG_NULL, MSG_MENU_REMOVE_MBR, mbr->name );
+		curs_attrset_dflt();
+
 		remove_mbr( mbr );
 		break;
 	case ASK_NO:
-		break;
 	case ASK_CANCEL:
-		return STR_MENU_CANCEL;
+		curs_attrset_chr( mbr, +1, TRUE );
+		print_msg( FLG_NULL, MSG_MENU_REMOVE_MBR_CANCEL,
+				mbr->name );
+		curs_attrset_dflt();
+
+		break;
 	case ASK_ERR:
 		return STR_MENU_ERR;
 	}
@@ -2073,6 +2143,7 @@ char	*cb_menu_bar_edit_class( menu_t **pp, menu_t **dflt )
 
 		str_goto = exec_menu( ls_menu_bar_sel_edit_class,
 				NULL, NULL );
+
 		if( str_goto != NULL )
 			return str_goto;
 	}
@@ -2386,12 +2457,12 @@ void	menu_edit_class_inc_abl(
 	case EDIT_CLASS_AGI:
 		abl = ABL_KIND_AGI;
 		break;
-	case EDIT_CLASS_WIS:
-		abl = ABL_KIND_WIS;
-		break;
 	case EDIT_CLASS_INT:
 		abl = ABL_KIND_INT;
 		break;
+	case EDIT_CLASS_WIS:
+		abl = ABL_KIND_WIS;
+		break;
 	case EDIT_CLASS_CHA:
 		abl = ABL_KIND_CHA;
 		break;
@@ -2442,12 +2513,26 @@ void	menu_edit_class_inc_abl(
 
 /**/
 
+char	*cb_menu_bar_load( menu_t **pp, menu_t **dflt )
+{
+	menu_bar_load();
+
+	return STR_MENU_CANCEL;
+}
+
+/**/
+
+void	menu_bar_load( void )
+{
+	if( menu_load( FALSE ) )
+		reset_all( TRUE );
+}
+
+/**/
+
 char	*cb_menu_bar_save( menu_t **pp, menu_t **dflt )
 {
-	save_dun();
-	save_town();
-
-	print_msg( FLG_NULL, MSG_S, MSG_INCOMPLETE );
+	menu_save( FALSE );
 
 	return STR_MENU_CANCEL;
 }
@@ -2461,7 +2546,7 @@ char	*menu_inn( void )
 	p = srch_menu( ls_menu_inn, "Stay" );
 	if( p == NULL )
 		return STR_MENU_ERR;
-	sprintf( p->msg, MSG_MENU_INN_STAY, (long)CHARGE_STAY_INN );
+	sprintf( p->msg, MSG_MENU_INN_STAY, (long)get_price_inn() );
 
 	return exec_menu( ls_menu_inn, NULL, NULL );
 }
@@ -2476,14 +2561,14 @@ char	*cb_menu_inn_stay( menu_t **pp, menu_t **dflt )
 
 	flg_pay = TRUE;
 	for( mbr_n = 0; mbr_n < MBR_MAX_N; mbr_n++ )
-		if( !chk_pay( CHARGE_STAY_INN, pty->mbr[mbr_n] ) )
+		if( !chk_pay( pty->mbr[mbr_n], get_price_inn() ) )
 			flg_pay = FALSE;
 
 	if( !flg_pay )
 		return STR_MENU_CANCEL;
 
 	for( mbr_n = 0; mbr_n < MBR_MAX_N; mbr_n++ )
-		add_chr_gold( pty->mbr[mbr_n], -CHARGE_STAY_INN );
+		add_chr_gold( pty->mbr[mbr_n], -get_price_inn() );
 
 	stay_inn();
 
@@ -2512,8 +2597,9 @@ char	*cb_menu_magic_guild_init( menu_t **pp, menu_t **dflt )
 			g_sel_mbr->name, g_sel_mbr->gold );
 
 	key = 'A';
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		spell_kind_t	kind;
+		spell_tab_t	*tab;
 
 		if( ls[i].path[0] == '\0' )
 			break;
@@ -2535,11 +2621,17 @@ char	*cb_menu_magic_guild_init( menu_t **pp, menu_t **dflt )
 
 		kind = (spell_kind_t)(ls[i].arg);
 
-		sprintf( ls[i].msg, MSG_FMT_MENU_MAGIC_GUILD,
-				get_spell_name( kind ),
-				get_price_spell( kind ) );
+		tab = get_spell_tab( kind );
+		if( tab == NULL ){
+			sprintf( ls[i].msg, MSG_FMT_MENU_MAGIC_GUILD,
+					"", 0, 0 );
+		} else {
+			sprintf( ls[i].msg, MSG_FMT_MENU_MAGIC_GUILD,
+					tab->name, tab->lev,
+					get_price_spell( kind ) );
+		}
 
-		if( chk_can_chr_cast( g_sel_mbr, kind ) ){
+		if( chk_chr_learned_spell( g_sel_mbr, kind ) ){
 			ls[i].flg |= FLG_MENU_HIDDEN;
 			ls[i].key = '\0';
 		} else {
@@ -2603,7 +2695,7 @@ char	*cb_menu_magic_guild( menu_t **pp, menu_t **dflt )
 	if( tab == NULL )
 		return STR_MENU_ERR;
 
-	if( !chk_pay( (gold_t)tab->price, g_sel_mbr ) )
+	if( !chk_pay( g_sel_mbr, get_price_spell( kind ) ) )
 		return STR_MENU_CANCEL;
 
 	/* ̾ɽ */
@@ -2620,8 +2712,8 @@ char	*cb_menu_magic_guild( menu_t **pp, menu_t **dflt )
 		return STR_MENU_CANCEL;
 	}
 
-	add_chr_gold( g_sel_mbr, -(gold_t)tab->price );
-	set_can_chr_cast( g_sel_mbr, kind, TRUE );
+	add_chr_gold( g_sel_mbr, -get_price_spell( kind ) );
+	set_chr_learned_spell( g_sel_mbr, kind, TRUE );
 
 	return "/Magic Guild";
 }
@@ -2702,8 +2794,23 @@ char	*cb_menu_pet_shop_take( menu_t **pp, menu_t **dflt )
 		if( !chk_can_join_pet( ls_pet[i] ) )
 			continue;
 
-		if( (n != 0) && ((n % PET_SHOP_PET_N_PER_PAGE) == 0) ){
-			strncpy( ls[k].path, "/Pet Shop Take/Page Separator",
+		sprintf( ls[k].path, "/Pet Shop Take/%ld", (long)(i + 1) );
+		strncpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
+		ls[k].key = key;
+		ls[k].accel_key[0] = '\0';
+		ls[k].n_msg = N_MSG_NULL,
+		strncpy( ls[k].msg, ls_pet[i]->name, MENU_MSG_MAX_LEN );
+		ls[k].call_back = cb_menu_pet_shop_take_sel;
+		ls[k].arg = (void *)(ls_pet[i]);
+		ls[k].flg = FLG_NULL;
+		k++;
+
+		key++;
+		n++;
+
+		if( (n % PET_SHOP_PET_N_PER_PAGE) == 0 ){
+			strncpy( ls[k].path,
+					"/Pet Shop Take/Page Separator",
 					MENU_PATH_MAX_LEN );
 			strncpy( ls[k].kind, "<PageSeparator>",
 					MENU_KIND_MAX_LEN );
@@ -2718,25 +2825,13 @@ char	*cb_menu_pet_shop_take( menu_t **pp, menu_t **dflt )
 
 			key = 'A';
 		}
-
-		sprintf( ls[k].path, "/Pet Shop Take/%ld", (long)(i + 1) );
-		strncpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
-		ls[k].key = key;
-		ls[k].accel_key[0] = '\0';
-		ls[k].n_msg = N_MSG_NULL,
-		strncpy( ls[k].msg, ls_pet[i]->name, MENU_MSG_MAX_LEN );
-		ls[k].call_back = cb_menu_pet_shop_take_sel;
-		ls[k].arg = (void *)(ls_pet[i]);
-		ls[k].flg = FLG_NULL;
-		k++;
-
-		key++;
-		n++;
 	}
 	ls[k].path[0] = '\0';
 
-	if( n <= 0 )
+	if( n <= 0 ){
+		print_msg( FLG_NULL, MSG_S, MSG_ERR_MENU_PET_SHOP_TAKE );
 		return STR_MENU_CANCEL;
+	}
 
 	*pp = ls;
 	*dflt = NULL;
@@ -2811,8 +2906,23 @@ char	*cb_menu_pet_shop_leave( menu_t **pp, menu_t **dflt )
 		if( !chk_can_not_join_pet( party_pet[i] ) )
 			continue;
 
-		if( (n != 0) && ((n % PET_SHOP_PET_N_PER_PAGE) == 0) ){
-			strncpy( ls[k].path, "/Pet Shop Leave/Page Separator",
+		sprintf( ls[k].path, "/Pet Shop Leave/%ld", (long)(i + 1) );
+		strncpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
+		ls[k].key = key;
+		ls[k].accel_key[0] = '\0';
+		ls[k].n_msg = N_MSG_NULL,
+		strncpy( ls[k].msg, party_pet[i]->name, MENU_MSG_MAX_LEN );
+		ls[k].call_back = cb_menu_pet_shop_leave_sel;
+		ls[k].arg = (void *)(party_pet[i]);
+		ls[k].flg = FLG_NULL;
+		k++;
+
+		key++;
+		n++;
+
+		if( (n % PET_SHOP_PET_N_PER_PAGE) == 0 ){
+			strncpy( ls[k].path,
+					"/Pet Shop Leave/Page Separator",
 					MENU_PATH_MAX_LEN );
 			strncpy( ls[k].kind, "<PageSeparator>",
 					MENU_KIND_MAX_LEN );
@@ -2827,25 +2937,13 @@ char	*cb_menu_pet_shop_leave( menu_t **pp, menu_t **dflt )
 
 			key = 'A';
 		}
-
-		sprintf( ls[k].path, "/Pet Shop Leave/%ld", (long)(i + 1) );
-		strncpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
-		ls[k].key = key;
-		ls[k].accel_key[0] = '\0';
-		ls[k].n_msg = N_MSG_NULL,
-		strncpy( ls[k].msg, party_pet[i]->name, MENU_MSG_MAX_LEN );
-		ls[k].call_back = cb_menu_pet_shop_leave_sel;
-		ls[k].arg = (void *)(party_pet[i]);
-		ls[k].flg = FLG_NULL;
-		k++;
-
-		key++;
-		n++;
 	}
 	ls[k].path[0] = '\0';
 
-	if( n <= 0 )
+	if( n <= 0 ){
+		print_msg( FLG_NULL, MSG_S, MSG_ERR_MENU_PET_SHOP_LEAVE );
 		return STR_MENU_CANCEL;
+	}
 
 	*pp = ls;
 	*dflt = NULL;
@@ -2871,6 +2969,249 @@ char	*cb_menu_pet_shop_leave_sel( menu_t **pp, menu_t **dflt )
 
 /**/
 
+char	*cb_menu_pet_shop_name( menu_t **pp, menu_t **dflt )
+{
+	pet_t	**ls_pet;
+	menu_t	*ls;
+	long	k;
+	long	i, n, max_n;
+	char	key;
+	char	*mnstr_name;
+
+	ls_pet = get_ls_pet();
+
+	ls = ls_menu_pet_shop_name;
+	k = 0;
+	key = 'A';
+	n = 0;
+	max_n = PET_MAX_N;
+
+	strncpy( ls[k].path, "/Pet Shop Name", MENU_PATH_MAX_LEN );
+	strncpy( ls[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
+	ls[k].key = '\0';
+	ls[k].accel_key[0] = '\0';
+	ls[k].n_msg = N_MSG_NULL;
+	ls[k].msg[0] = '\0';
+	ls[k].call_back = NULL;
+	ls[k].arg = NULL;
+	ls[k].flg = FLG_NULL;
+	k++;
+
+	strncpy( ls[k].path, "/Pet Shop Name/Title", MENU_PATH_MAX_LEN );
+	strncpy( ls[k].kind, "<Title>", MENU_KIND_MAX_LEN );
+	ls[k].key = '\0';
+	ls[k].accel_key[0] = '\0';
+	ls[k].n_msg = N_MSG_NULL;
+	sprintf( ls[k].msg, MSG_TTL_MENU_PET_SHOP_NAME,
+			get_shop_name( SHOP_N_PET_SHOP ),
+			g_sel_mbr->name );
+	ls[k].call_back = NULL;
+	ls[k].arg = NULL;
+	ls[k].flg = FLG_NULL;
+	k++;
+
+	for( i = 0; i < max_n; i++ ){
+		if( ls_pet[i] == NULL )
+			continue;
+		if( chk_flg( ls_pet[i]->stat, FLG_STAT_NOT_EXIST ) )
+			continue;
+
+		if( ls_pet[i]->mnstr_tab == NULL )
+			mnstr_name = MSG_MNSTR_UNKNOWN;
+		else
+			mnstr_name = ls_pet[i]->mnstr_tab->name;
+
+		sprintf( ls[k].path, "/Pet Shop Name/%ld", (long)(i + 1) );
+		strncpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
+		ls[k].key = key;
+		ls[k].accel_key[0] = '\0';
+		ls[k].n_msg = N_MSG_NULL,
+		sprintf( ls[k].msg, MSG_MENU_PET_SHOP_NAME_LS,
+				ls_pet[i]->name, mnstr_name );
+		ls[k].call_back = cb_menu_pet_shop_name_sel;
+		ls[k].arg = (void *)(ls_pet[i]);
+		ls[k].flg = FLG_NULL;
+		k++;
+
+		key++;
+		n++;
+
+		if( (n % PET_SHOP_PET_N_PER_PAGE) == 0 ){
+			strncpy( ls[k].path,
+					"/Pet Shop Name/Page Separator",
+					MENU_PATH_MAX_LEN );
+			strncpy( ls[k].kind, "<PageSeparator>",
+					MENU_KIND_MAX_LEN );
+			ls[k].key = '\0';
+			ls[k].accel_key[0] = '\0';
+			ls[k].n_msg = N_MSG_NULL;
+			ls[k].msg[0] = '\0';
+			ls[k].call_back = NULL;
+			ls[k].arg = NULL;
+			ls[k].flg = FLG_NULL;
+			k++;
+
+			key = 'A';
+		}
+	}
+	ls[k].path[0] = '\0';
+
+	if( n <= 0 ){
+		print_msg( FLG_NULL, MSG_S, MSG_ERR_MENU_PET_SHOP_NAME );
+		return STR_MENU_CANCEL;
+	}
+
+	*pp = ls;
+	*dflt = NULL;
+
+	return NULL;
+}
+
+/**/
+
+char	*cb_menu_pet_shop_name_sel( menu_t **pp, menu_t **dflt )
+{
+	pet_t	*pet;
+
+	pet = (pet_t *)((*pp)->arg);
+	if( pet == NULL )
+		return STR_MENU_ERR;
+
+	wipe_all();
+	if( gets_pet_name( pet ) ){
+		print_msg( FLG_NULL, MSG_MENU_PET_SHOP_NAME_SUCCESS,
+				pet->name );
+	}
+	redraw_all();
+
+	return "/Pet Shop";
+}
+
+/**/
+
+char	*cb_menu_pet_shop_remove( menu_t **pp, menu_t **dflt )
+{
+	pet_t	**ls_pet;
+	menu_t	*ls;
+	long	k;
+	long	i, n, max_n;
+	char	key;
+
+	ls_pet = get_ls_pet();
+
+	ls = ls_menu_pet_shop_remove;
+	k = 0;
+	key = 'A';
+	n = 0;
+	max_n = LS_PET_MAX_N;
+
+	strncpy( ls[k].path, "/Pet Shop Remove", MENU_PATH_MAX_LEN );
+	strncpy( ls[k].kind, "<Branch>", MENU_KIND_MAX_LEN );
+	ls[k].key = '\0';
+	ls[k].accel_key[0] = '\0';
+	ls[k].n_msg = N_MSG_NULL;
+	ls[k].msg[0] = '\0';
+	ls[k].call_back = NULL;
+	ls[k].arg = NULL;
+	ls[k].flg = FLG_NULL;
+	k++;
+
+	strncpy( ls[k].path, "/Pet Shop Remove/Title", MENU_PATH_MAX_LEN );
+	strncpy( ls[k].kind, "<Title>", MENU_KIND_MAX_LEN );
+	ls[k].key = '\0';
+	ls[k].accel_key[0] = '\0';
+	ls[k].n_msg = N_MSG_NULL;
+	sprintf( ls[k].msg, MSG_TTL_MENU_PET_SHOP_REMOVE,
+			get_shop_name( SHOP_N_PET_SHOP ),
+			g_sel_mbr->name );
+	ls[k].call_back = NULL;
+	ls[k].arg = NULL;
+	ls[k].flg = FLG_NULL;
+	k++;
+
+	for( i = 0; i < max_n; i++ ){
+		if( ls_pet[i] == NULL )
+			continue;
+		if( chk_flg( ls_pet[i]->stat, FLG_STAT_NOT_EXIST ) )
+			continue;
+
+		sprintf( ls[k].path, "/Pet Shop Remove/%ld", (long)(i + 1) );
+		strncpy( ls[k].kind, "<Item>", MENU_KIND_MAX_LEN );
+		ls[k].key = key;
+		ls[k].accel_key[0] = '\0';
+		ls[k].n_msg = N_MSG_NULL,
+		strncpy( ls[k].msg, ls_pet[i]->name, MENU_MSG_MAX_LEN );
+		ls[k].call_back = cb_menu_pet_shop_remove_sel;
+		ls[k].arg = (void *)(ls_pet[i]);
+		ls[k].flg = FLG_NULL;
+		k++;
+
+		key++;
+		n++;
+
+		if( (n % PET_SHOP_PET_N_PER_PAGE) == 0 ){
+			strncpy( ls[k].path,
+					"/Pet Shop Remove/Page Separator",
+					MENU_PATH_MAX_LEN );
+			strncpy( ls[k].kind, "<PageSeparator>",
+					MENU_KIND_MAX_LEN );
+			ls[k].key = '\0';
+			ls[k].accel_key[0] = '\0';
+			ls[k].n_msg = N_MSG_NULL;
+			ls[k].msg[0] = '\0';
+			ls[k].call_back = NULL;
+			ls[k].arg = NULL;
+			ls[k].flg = FLG_NULL;
+			k++;
+
+			key = 'A';
+		}
+	}
+	ls[k].path[0] = '\0';
+
+	if( n <= 0 ){
+		print_msg( FLG_NULL, MSG_S, MSG_ERR_MENU_PET_SHOP_REMOVE );
+		return STR_MENU_CANCEL;
+	}
+
+	*pp = ls;
+	*dflt = NULL;
+
+	return NULL;
+}
+
+/**/
+
+char	*cb_menu_pet_shop_remove_sel( menu_t **pp, menu_t **dflt )
+{
+	pet_t	*pet;
+	ask_t	ask;
+
+	pet = (pet_t *)((*pp)->arg);
+	if( pet == NULL )
+		return STR_MENU_ERR;
+
+	print_msg( FLG_NULL, MSG_MENU_PET_SHOP_ASK_REMOVE, pet->name );
+
+	ask = exec_menu_ask( MSG_TTL_MENU_PET_SHOP_ASK_REMOVE,
+			ASK_NO, TRUE );
+	switch( ask ){
+	case ASK_YES:
+		break;
+	case ASK_NO:
+	case ASK_CANCEL:
+	case ASK_ERR:
+		return STR_MENU_CANCEL;
+	}
+
+	print_msg( FLG_NULL, MSG_MENU_PET_SHOP_REMOVE_SUCCESS, pet->name );
+	remove_pet( pet );
+
+	return "/Pet Shop";
+}
+
+/**/
+
 char	*cb_menu_pet_shop_buy( menu_t **pp, menu_t **dflt )
 {
 	art_ls_pet_shop_t	*art_ls_pet_shop;
@@ -2915,7 +3256,8 @@ char	*cb_menu_pet_shop_buy( menu_t **pp, menu_t **dflt )
 			break;
 
 		if( (i != 0) && ((i % PET_SHOP_PET_N_PER_PAGE) == 0) ){
-			strncpy( ls[k].path, "/Pet Shop Buy/Page Separator",
+			strncpy( ls[k].path,
+					"/Pet Shop Buy/Page Separator",
 					MENU_PATH_MAX_LEN );
 			strncpy( ls[k].kind, "<PageSeparator>",
 					MENU_KIND_MAX_LEN );
@@ -2976,14 +3318,15 @@ char	*cb_menu_pet_shop_buy_sel( menu_t **pp, menu_t **dflt )
 		return STR_MENU_ERR;
 
 	price = art_ls_pet_shop[n].price;
-	if( !chk_pay( price, g_sel_mbr ) )
+	if( !chk_pay( g_sel_mbr, price ) )
 		return STR_MENU_CANCEL;
 
 	/* ̾ɽ */
 	print_msg( FLG_NULL, MSG_MENU_PET_SHOP_ASK_BUY,
 			get_mnstr_name( kind ) );
 
-	ask = exec_menu_ask( MSG_TTL_MENU_PET_SHOP_ASK_BUY, ASK_NO, FALSE );
+	ask = exec_menu_ask( MSG_TTL_MENU_PET_SHOP_ASK_BUY,
+			ASK_NO, TRUE );
 	switch( ask ){
 	case ASK_YES:
 		break;
@@ -2993,16 +3336,20 @@ char	*cb_menu_pet_shop_buy_sel( menu_t **pp, menu_t **dflt )
 		return STR_MENU_CANCEL;
 	}
 
-	pet = make_pet( 10, kind );
+	pet = make_pet( PET_DUN_LEV, kind, g_sel_mbr );
 	if( pet == NULL ){
 		print_msg( FLG_NULL, MSG_S, MSG_ERR_MENU_PET_SHOP_BUY );
 		return STR_MENU_CANCEL;
 	}
 
-	pet->owner = g_sel_mbr;
+	pet->flg_chr |= FLG_CHR_CAN_DEL;
+	pet->flg_chr &= ~(FLG_CHR_CAN_DISMISSAL);
+
 	join_pet( pet );
 	print_msg( FLG_NULL, MSG_MENU_PET_SHOP_BUY_SUCCESS, pet->name );
 
+	add_chr_gold( g_sel_mbr, (gold_t)(-price) );
+
 	return "/Pet Shop";
 }
 
