@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,24 +25,26 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: town.h,v 1.10 2003/11/23 04:29:50 bowkenken Exp $
+* $Id: town.h,v 1.16 2004/06/30 13:20:45 bowkenken Exp $
 *****************************************************************/
 
 #ifndef	TOWN_H
 #define	TOWN_H
 
 #include	"dun.h"
+#include	"party.h"
 #include	"mnstr.h"
 
 typedef enum {
 	SHOP_N_NULL,
 	SHOP_N_BAR,
 	SHOP_N_INN,
-	SHOP_N_TEMPLE,
 	SHOP_N_WPN,
 	SHOP_N_ARMOR,
 	SHOP_N_MAGIC,
+	SHOP_N_TEMPLE,
 	SHOP_N_ALCHEMY,
+	SHOP_N_MUSIC,
 	SHOP_N_GROCERY,
 	SHOP_N_RESTAURANT,
 	SHOP_N_TEAROOM,
@@ -52,31 +53,29 @@ typedef enum {
 	SHOP_N_MAX_N,
 } shop_n_t;
 
+/*  */
+
+#define	DISCOUNT_QUEUE_MAX_N	(32 + MBR_MAX_N)
+
 typedef struct {
-	mnstr_kind_t	kind;
-	gold_t	price;
-} art_ls_pet_shop_t;
+	shop_n_t	shop_n;
+	door_t	*door;
+	bool_t	flg_sold_out;	/* ڤե饰 */
+	long	n;
+	pos_t	pos[DISCOUNT_QUEUE_MAX_N];	/* οͤΰ */
+	chr_t	*chr[DISCOUNT_QUEUE_MAX_N];	/* ο */
+} discount_queue_t;
 
-#define	FACE_MNR_GATE	'+'
-#define	FACE_MNR_SHOP_INN	'I'
-#define	FACE_MNR_SHOP_BAR	'B'
-#define	FACE_MNR_SHOP_TEMPLE	'T'
-#define	FACE_MNR_SHOP_WPN	'W'
-#define	FACE_MNR_SHOP_ARMOR	'A'
-#define	FACE_MNR_SHOP_MAGIC	'M'
-#define	FACE_MNR_SHOP_ALCHEMY	'L'
-#define	FACE_MNR_SHOP_GROCERY	'G'
-#define	FACE_MNR_SHOP_RESTAURANT	'R'
-#define	FACE_MNR_SHOP_TEAROOM	'C'
-#define	FACE_MNR_SHOP_TOBACCO	'S'
-#define	FACE_MNR_SHOP_PET_SHOP	'P'
-
-#define	FACE_MNR_STREETLAMP	'*'
+/**/
 
 #define	TOWN_PTN_COL_MAX_N	128
 
+/* ξȤ餹ϰ */
+
 #define	STREETLAMP_R	2
 
+/* Υޥåסǡ */
+
 typedef struct {
 	long	col_str_len;
 	long	col_n;
@@ -91,4 +90,11 @@ typedef struct {
 	char	mjr, mnr;
 } town_ptn_col_tab_t;
 
+/* ڥåȡåפξʥꥹ */
+
+typedef struct {
+	mnstr_kind_t	kind;
+	gold_t	price;
+} art_ls_pet_shop_t;
+
 #endif	/* TOWN_H */
