@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,7 +25,7 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: trap.c,v 1.22 2003/11/19 13:19:14 bowkenken Exp $
+* $Id: trap.c,v 1.44 2004/07/30 15:15:28 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
@@ -39,40 +38,45 @@
 /*#include	"chr.h"*/
 /*#include	"party.h"*/
 /*#include	"mnstr.h"*/
-/*#include	"pet.h"*/
+#include	"pet.h"
 #include	"fight.h"
 #include	"fx.h"
 #include	"trap.h"
 #include	"draw.h"
 /*#include	"curs.h"*/
 #include	"menu.h"
+/*#include	"amenu.h"*/
+/*#include	"request.h"*/
+/*#include	"gfile.h"*/
 /*#include	"msg.h"*/
 /*#include	"ver.h"*/
-#include	"gmain_prot.h"
-#include	"misc_prot.h"
-/*#include	"turn_prot.h"*/
-#include	"dun_prot.h"
-/*#include	"town_prot.h"*/
-/*#include	"item_prot.h"*/
-#include	"spell_prot.h"
-#include	"chr_prot.h"
-/*#include	"party_prot.h"*/
-/*#include	"mnstr_prot.h"*/
-/*#include	"pet_prot.h"*/
-#include	"fight_prot.h"
-#include	"fx_prot.h"
-#include	"trap_prot.h"
-#include	"draw_prot.h"
-/*#include	"curs_prot.h"*/
-#include	"menu_prot.h"
-/*#include	"tmenu_prot.h"*/
-/*#include	"amenu_prot.h"*/
-/*#include	"gfile_prot.h"*/
-/*#include	"msg_prot.h"*/
+#include	"gmain-prot.h"
+#include	"misc-prot.h"
+/*#include	"turn-prot.h"*/
+#include	"dun-prot.h"
+/*#include	"town-prot.h"*/
+/*#include	"item-prot.h"*/
+#include	"spell-prot.h"
+#include	"chr-prot.h"
+/*#include	"party-prot.h"*/
+/*#include	"mnstr-prot.h"*/
+#include	"pet-prot.h"
+#include	"fight-prot.h"
+#include	"fx-prot.h"
+#include	"trap-prot.h"
+#include	"draw-prot.h"
+/*#include	"curs-prot.h"*/
+#include	"menu-prot.h"
+/*#include	"tmenu-prot.h"*/
+/*#include	"amenu-prot.h"*/
+/*#include	"request-prot.h"*/
+/*#include	"gfile-prot.h"*/
+/*#include	"msg-prot.h"*/
 
 /**/
 
-#define	TRAP_DIFFICULTY_LEV_RATE	20
+#define	TRAP_DIFFICULTY_LEV_RATE	RATE_LEV_PER_DUN_LEV
+#define	TRAP_DIFFICULTY_LEV_ADD	1
 
 #define	TRAP_MAX_RATE	100
 #define	TRAP_MIN_RATE	50
@@ -80,6 +84,9 @@
 #define	TRAP_ROOM_RATE	1
 #define	TRAP_ROOM_N	50
 
+#define	PIT_DAM_RATE	25
+#define	PIT_DAM_ADD	1
+
 /**/
 
 static trap_t	trap_buf[TRAP_MAX_N];
@@ -90,25 +97,40 @@ static long	trap_tab_max_n;
 
 static trap_tab_t	trap_tab[] = {
 	{ TRAP_KIND_SLEEP, NULL, N_MSG_TRAP_SLEEP,
-		80, 1, DUN_MAX_LEV_BASE, -1 },
+		100, FACE_MNR_TRAP_SLEEP,
+		1, DUN_MAX_LEV_BASE, -1 },
 
 	{ TRAP_KIND_ARW, NULL, N_MSG_TRAP_ARW,
-		80, 2, DUN_MAX_LEV_BASE, -1 },
+		100, FACE_MNR_TRAP_ARW,
+		2, DUN_MAX_LEV_BASE, -1 },
 
 	{ TRAP_KIND_POISON_DARTS, NULL, N_MSG_TRAP_POISON_DARTS,
-		80, 3, DUN_MAX_LEV_BASE, -1 },
+		100, FACE_MNR_TRAP_POISON_DARTS,
+		3, DUN_MAX_LEV_BASE, -1 },
 
 	{ TRAP_KIND_BOMB, NULL, N_MSG_TRAP_BOMB,
-		80, 10, DUN_MAX_LEV_BASE, -1 },
+		100, FACE_MNR_TRAP_BOMB,
+		10, DUN_MAX_LEV_BASE, -1 },
 
 	{ TRAP_KIND_TELEPORT_PARTY, NULL, N_MSG_TRAP_TELEPORT_PARTY,
-		80, 20, DUN_MAX_LEV_BASE, -1 },
+		100, FACE_MNR_TRAP_TELEPORT_PARTY,
+		20, DUN_MAX_LEV_BASE, -1 },
 
 	{ TRAP_KIND_TELEPORT, NULL, N_MSG_TRAP_TELEPORT,
-		80, 30, DUN_MAX_LEV_BASE, -1 },
+		100, FACE_MNR_TRAP_TELEPORT,
+		30, DUN_MAX_LEV_BASE, -1 },
+
+	{ TRAP_KIND_PIT, NULL, N_MSG_TRAP_PIT,
+		100, FACE_MNR_TRAP_PIT,
+		4, DUN_MAX_LEV_BASE, -1 },
+
+	{ TRAP_KIND_CHUTE, NULL, N_MSG_TRAP_CHUTE,
+		80, FACE_MNR_TRAP_CHUTE,
+		10, DUN_MAX_LEV_BASE, -1 },
 
 	{ TRAP_KIND_NULL, NULL, N_MSG_NULL,
-		0, DUN_MAX_LEV_BASE, DUN_MAX_LEV_BASE, 0 },
+		0, FACE_MNR_NULL,
+		DUN_MAX_LEV_BASE, DUN_MAX_LEV_BASE, 0 },
 };
 
 chr_t	g_dmy_chr;
@@ -120,7 +142,7 @@ void	init_trap( void )
 	long	i;
 
 	trap_tab_max_n = 0;
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		if( trap_tab[i].kind == TRAP_KIND_NULL )
 			break;
 		trap_tab[i].name = MSG( trap_tab[i].name_n );
@@ -160,27 +182,38 @@ trap_t	*make_trap( long x, long y, long dun_lev )
 {
 	long	i;
 	dun_t	*dun;
-	trap_t	*trap, dmy;
+	trap_t	*trap;
 	long	n;
 
 	dun = get_dun();
 
-	if( (x != MAP_DEL_X) && (y != MAP_DEL_Y) )
+	if( clip_pos( x, y ) )
 		if( dun->map.obj.mjr[y][x] != FACE_MJR_FLOOR )
 			return NULL;
 
 	n = 0;
-	for( i = 128; i > 0; i-- ){
+	for( i = LOOP_MAX_100; i > 0; i-- ){
 		n = randm( trap_tab_max_n );
+
+#if	defined( DEBUG ) && defined( D_TEST_TRAP_KIND )
+		if( D_TEST_TRAP_KIND != TRAP_KIND_NULL ){
+			if( trap_tab[n].kind == D_TEST_TRAP_KIND ){
+				break;
+			} else {
+				continue;
+			}
+		}
+#endif
+
 		if( trap_tab[n].kind == TRAP_KIND_NULL )
 			continue;
 		if( !rate_randm( trap_tab[n].rate ) )
 			continue;
-		if( abs( dun_lev ) < trap_tab[n].min_lev )
+		if( labs( dun_lev ) < trap_tab[n].min_lev )
 			continue;
-		if( trap_tab[n].max_lev < abs( dun_lev ) )
+		if( labs( dun_lev ) > trap_tab[n].max_lev )
 			continue;
-		if( sgn( dun_lev ) != trap_tab[n].sgn_lev )
+		if( sgn_l( dun_lev ) != trap_tab[n].sgn_lev )
 			continue;
 
 		break;
@@ -188,14 +221,14 @@ trap_t	*make_trap( long x, long y, long dun_lev )
 	if( i <= 0 )
 		return NULL;
 
-	trap = alloc_trap( &dmy );
+	trap = alloc_trap();
 	if( trap == NULL )
 		return NULL;
 
 	do {
 		if( !make_trap_std( dun_lev, trap, n ) )
 			break;
-		if( (x != MAP_DEL_X) && (y != MAP_DEL_Y) )
+		if( clip_pos( x, y ) )
 			if( !set_trap( x, y, trap ) )
 				break;
 
@@ -211,6 +244,8 @@ trap_t	*make_trap( long x, long y, long dun_lev )
 
 bool_t	make_trap_std( long dun_lev, trap_t *trap, long n )
 {
+	long	difficulty;
+
 	if( trap == NULL )
 		return FALSE;
 
@@ -218,11 +253,15 @@ bool_t	make_trap_std( long dun_lev, trap_t *trap, long n )
 	trap->x = MAP_DEL_X;
 	trap->y = MAP_DEL_Y;
 	trap->lev = dun_lev;
-	trap->difficulty = abs( dun_lev );
-	trap->difficulty *= TRAP_DIFFICULTY_LEV_RATE;
-	trap->difficulty /= _100_PERCENT;
-	trap->difficulty += 1;
-	trap->tab = &(trap_tab[n]);
+
+	difficulty = labs( dun_lev );
+	difficulty *= TRAP_DIFFICULTY_LEV_RATE;
+	difficulty /= _100_PERCENT;
+	difficulty += TRAP_DIFFICULTY_LEV_ADD;
+	trap->difficulty = difficulty;
+
+	trap->flg_chked = FALSE;
+	trap->tab = get_trap_tab( trap_tab[n].kind );
 
 	return TRUE;
 }
@@ -231,16 +270,13 @@ bool_t	make_trap_std( long dun_lev, trap_t *trap, long n )
 
 bool_t	set_trap( long x, long y, trap_t *trap )
 {
-	dun_t	*dun;
+	dun_t	*dun = get_dun();
 
 	if( trap == NULL )
 		return FALSE;
 
-	if( (x == MAP_DEL_X) || (y == MAP_DEL_Y) )
+	if( !clip_pos( x, y ) )
 		return FALSE;
-
-	dun = get_dun();
-
 	if( dun->map.obj.mjr[y][x] != FACE_MJR_FLOOR )
 		return FALSE;
 
@@ -252,6 +288,8 @@ bool_t	set_trap( long x, long y, trap_t *trap )
 	dun->map.obj.flg[y][x] |= FLG_MAP_OBJ_PASS;
 	dun->map.obj.flg[y][x] |= FLG_MAP_OBJ_LOOK_FLOOR;
 
+	set_trap_face_mnr( trap );
+
 	ins_ls_trap( &dun_trap_asgn, trap );
 
 	return TRUE;
@@ -259,6 +297,39 @@ bool_t	set_trap( long x, long y, trap_t *trap )
 
 /**/
 
+bool_t	set_trap_face_mnr( trap_t *trap )
+{
+	dun_t	*dun = get_dun();
+	long	x, y;
+
+	if( trap == NULL )
+		return FALSE;
+	if( trap->tab == NULL )
+		return FALSE;
+
+	x = trap->x;
+	y = trap->y;
+
+	if( !clip_pos( x, y ) )
+		return FALSE;
+	if( dun->map.obj.mjr[y][x] != FACE_MJR_TRAP )
+		return FALSE;
+
+	if( chk_flg( dun->map.obj.flg[y][x], FLG_MAP_OBJ_LOOK_FLOOR ) ){
+		dun->map.obj.mnr[y][x] = FACE_MNR_NULL;
+	} else if( trap->flg_chked ){
+		dun->map.obj.mnr[y][x] = trap->tab->mnr;
+	} else {
+		dun->map.obj.mnr[y][x] = FACE_MNR_NULL;
+	}
+
+	draw_trap( trap );
+
+	return TRUE;
+}
+
+/**/
+
 void	make_trap_room( long area_x, long area_y, long lev )
 {
 	dun_t	*dun = get_dun();
@@ -269,7 +340,7 @@ void	make_trap_room( long area_x, long area_y, long lev )
 		return;
 
 	n = 0;
-	for( i = 0; i < 1024; i++ ){
+	for( i = 0; i < LOOP_MAX_1000; i++ ){
 		long	mx, my;
 		trap_t	*trap;
 
@@ -310,10 +381,10 @@ void	chk_trap_chest( item_t *item, chr_t *chr, rate_t rate )
 
 /**/
 
-void	chk_trap( chr_t *chr )
+void	chk_trap( chr_t *chr, chr_t *thief )
 {
 	trap_t	*trap;
-	dun_t	*dun;
+	dun_t	*dun = get_dun();
 
 	if( chr == NULL )
 		return;
@@ -321,8 +392,15 @@ void	chk_trap( chr_t *chr )
 		return;
 	if( chk_flg( chr->stat, FLG_STAT_FLY ) )
 		return;
+	if( !chk_can_act_chr( chr ) )
+		return;
 
-	dun = get_dun();
+	if( chr->x == chr->pre_x )
+		if( chr->y == chr->pre_y )
+			return;
+
+	if( thief == NULL )
+		thief = chr;
 
 	if( dun->map.obj.mjr[chr->y][chr->x] != FACE_MJR_TRAP )
 		return;
@@ -331,9 +409,14 @@ void	chk_trap( chr_t *chr )
 	if( trap == NULL )
 		return;
 
-	find_trap( trap );
+	if( chr->kind == CHR_KIND_MBR ){
+		find_trap( trap );
+	} else {
+		if( chk_flg( chr->flg_map, FLG_MAP_CHR_FIND ) )
+			find_trap( trap );
+	}
 
-	if( chr_roll( chr, ABL_KIND_THI, ABL_KIND_AGI, trap->difficulty ) )
+	if( chr_roll( thief, ABL_KIND_THI, ABL_KIND_AGI, trap->difficulty ) )
 		print_msg_mbr( chr, FLG_NULL, MSG_NOT_TRAPPED, chr->name );
 	else
 		caught_trap( trap, chr );
@@ -348,9 +431,7 @@ void	find_trap( trap_t *trap )
 	if( trap == NULL )
 		return;
 
-	if( !clip_x( trap->x ) )
-		return;
-	if( !clip_y( trap->y ) )
+	if( !clip_pos( trap->x, trap->y ) )
 		return;
 
 	dun->map.obj.flg[trap->y][trap->x] &= ~FLG_MAP_OBJ_LOOK_FLOOR;
@@ -371,6 +452,13 @@ void	caught_trap( trap_t *trap, chr_t *chr )
 	if( chk_flg_or( chr->stat, FLG_STAT_NOT_EXIST | FLG_STAT_DEAD ) )
 		return;
 
+	if( chr->kind == CHR_KIND_MBR ){
+		find_trap( trap );
+	} else {
+		if( chk_flg( chr->flg_map, FLG_MAP_CHR_FIND ) )
+			find_trap( trap );
+	}
+
 	print_msg_mbr( chr, FLG_NULL, MSG_TRAPPED,
 			chr->name, trap->tab->name );
 
@@ -416,6 +504,12 @@ void	caught_trap( trap_t *trap, chr_t *chr )
 	case TRAP_KIND_TELEPORT:
 		teleport_chr( chr );
 		break;
+	case TRAP_KIND_PIT:
+		caught_trap_pit( chr );
+		break;
+	case TRAP_KIND_CHUTE:
+		caught_trap_chute( chr );
+		break;
 	}
 }
 
@@ -458,9 +552,9 @@ void	set_trap_arw_pos( chr_t *chr, long *x, long *y )
 
 	nx = chr->x;
 	ny = chr->y;
-	for( i = max( AREA_MAX_X, AREA_MAX_Y ); i >= 0; i-- ){
-		nx += sgn( mx - chr->x );
-		ny += sgn( my - chr->y );
+	for( i = max_l( AREA_MAX_X, AREA_MAX_Y ); i >= 0; i-- ){
+		nx += sgn_l( mx - chr->x );
+		ny += sgn_l( my - chr->y );
 		if( !clip_pos( nx, ny ) )
 			break;
 		if( !chk_flg( dun->map.obj.flg[ny][nx],
@@ -474,6 +568,38 @@ void	set_trap_arw_pos( chr_t *chr, long *x, long *y )
 
 /**/
 
+void	caught_trap_pit( chr_t *chr )
+{
+	long	dam;
+
+	dam = calc_max_hp( chr ) * PIT_DAM_RATE / _100_PERCENT;
+	dam += PIT_DAM_ADD;
+
+	add_hp( chr, -dam, TRUE, FALSE );
+}
+
+/**/
+
+void	caught_trap_chute( chr_t *chr )
+{
+	bool_t	flg_party;
+
+	if( chr == NULL )
+		return;
+
+	flg_party = FALSE;
+	if( chr->kind == CHR_KIND_MBR )
+		flg_party = TRUE;
+	if( chr->kind == CHR_KIND_MNSTR )
+		if( is_pet( chr ) )
+			flg_party = TRUE;
+
+	if( flg_party )
+		up_dun( -1 );
+}
+
+/**/
+
 void	disarm_trap( trap_t *p )
 {
 	dun_t	*dun;
@@ -519,10 +645,10 @@ void	ins_ls_trap( trap_t *ls, trap_t *p )
 	if( p == NULL )
 		return;
 
-	if( p->next != NULL )
-		p->next->prev = p->prev;
 	if( p->prev != NULL )
 		p->prev->next = p->next;
+	if( p->next != NULL )
+		p->next->prev = p->prev;
 
 	p->prev = ls->prev;
 	p->next = ls;
@@ -534,19 +660,27 @@ void	ins_ls_trap( trap_t *ls, trap_t *p )
 
 /**/
 
-trap_t	*alloc_trap( trap_t *dmy )
+trap_t	*alloc_trap( void )
 {
-	trap_t	*p;
-	/* ŪʥƥꥹȤˤ֤鲼 */
-
-	dmy->prev = dmy;
-	dmy->next = dmy;
+	trap_t	*p, dmy;
 
 	p = get_trap_free_next();
 	if( p == NULL )
 		return NULL;
 
-	ins_ls_trap( dmy, p );
+	p->kind = ITEM_KIND_NULL;
+	p->x = MAP_DEL_X;
+	p->y = MAP_DEL_Y;
+
+	/* ŪʥȥåץꥹȤˤ֤鲼 */
+
+	dmy.prev = &dmy;
+	dmy.next = &dmy;
+
+	ins_ls_trap( &dmy, p );
+
+	p->prev = NULL;
+	p->next = NULL;
 
 	return p;
 }
@@ -589,6 +723,13 @@ trap_t	*get_trap_free_next( void )
 
 /**/
 
+trap_t	*get_dun_trap_asgn( void )
+{
+	return &dun_trap_asgn;
+}
+
+/**/
+
 trap_t	*get_trap( long x, long y )
 {
 	trap_t	*p;
@@ -602,9 +743,16 @@ trap_t	*get_trap( long x, long y )
 
 /**/
 
-trap_t	*get_dun_trap_asgn( void )
+trap_tab_t	*get_trap_tab( trap_kind_t kind )
 {
-	return &dun_trap_asgn;
+	long	i;
+
+	for( i = 0; i < LOOP_MAX_1000; i++ )
+		if( trap_tab[i].kind == kind )
+			return( &(trap_tab[i]) );
+
+	return NULL;
 }
 
 /**/
+
