@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,11 +25,11 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: turn.c,v 1.7 2003/11/23 04:29:50 bowkenken Exp $
+* $Id: turn.c,v 1.28 2004/07/27 21:57:59 bowkenken Exp $
 *****************************************************************/
 
 #include	"gmain.h"
-/*#include	"misc.h"*/
+#include	"misc.h"
 #include	"turn.h"
 /*#include	"dun.h"*/
 #include	"town.h"
@@ -39,36 +38,40 @@
 /*#include	"chr.h"*/
 #include	"party.h"
 #include	"mnstr.h"
-/*#include	"pet.h"*/
+#include	"pet.h"
 /*#include	"fight.h"*/
 #include	"fx.h"
 /*#include	"trap.h"*/
-/*#include	"draw.h"*/
+#include	"draw.h"
 /*#include	"curs.h"*/
 #include	"menu.h"
+/*#include	"amenu.h"*/
+#include	"request.h"
+/*#include	"gfile.h"*/
 #include	"msg.h"
 /*#include	"ver.h"*/
-/*#include	"gmain_prot.h"*/
-/*#include	"misc_prot.h"*/
-#include	"turn_prot.h"
-/*#include	"dun_prot.h"*/
-#include	"town_prot.h"
-#include	"item_prot.h"
-/*#include	"spell_prot.h"*/
-/*#include	"chr_prot.h"*/
-/*#include	"party_prot.h"*/
-#include	"mnstr_prot.h"
-/*#include	"pet_prot.h"*/
-/*#include	"fight_prot.h"*/
-#include	"fx_prot.h"
-/*#include	"trap_prot.h"*/
-/*#include	"draw_prot.h"*/
-/*#include	"curs_prot.h"*/
-/*#include	"menu_prot.h"*/
-/*#include	"tmenu_prot.h"*/
-/*#include	"amenu_prot.h"*/
-/*#include	"gfile_prot.h"*/
-/*#include	"msg_prot.h"*/
+/*#include	"gmain-prot.h"*/
+#include	"misc-prot.h"
+#include	"turn-prot.h"
+/*#include	"dun-prot.h"*/
+#include	"town-prot.h"
+#include	"item-prot.h"
+/*#include	"spell-prot.h"*/
+/*#include	"chr-prot.h"*/
+/*#include	"party-prot.h"*/
+#include	"mnstr-prot.h"
+/*#include	"pet-prot.h"*/
+/*#include	"fight-prot.h"*/
+#include	"fx-prot.h"
+/*#include	"trap-prot.h"*/
+#include	"draw-prot.h"
+/*#include	"curs-prot.h"*/
+/*#include	"menu-prot.h"*/
+/*#include	"tmenu-prot.h"*/
+/*#include	"amenu-prot.h"*/
+#include	"request-prot.h"
+/*#include	"gfile-prot.h"*/
+/*#include	"msg-prot.h"*/
 
 /**/
 
@@ -76,6 +79,76 @@ static long	g_day, g_turn;
 
 /**/
 
+long	init_turn()
+{
+	set_game_date( 999, 4, 1 );
+	reset_turn( 10 * TURN_PER_HOUR );
+
+	return g_turn;
+}
+
+/**/
+
+long	reset_turn( long n )
+{
+	g_turn = n;
+
+	if( g_turn >= 0 )
+		g_day += g_turn / TURN_A_DAY;
+	else
+		g_day -= labs( g_turn ) / TURN_A_DAY + 1;
+
+	g_turn %= TURN_A_DAY;
+	g_turn += TURN_A_DAY;
+	g_turn %= TURN_A_DAY;
+
+	return g_turn;
+}
+
+/**/
+
+long	add_turn( long n )
+{
+	long	ret;
+	long	i;
+
+	ret = -1;
+
+	for( i = 0; i < n; i++ )
+		ret = inc_turn();
+
+	return ret;
+}
+
+/**/
+
+long	inc_turn( void )
+{
+	g_turn++;
+	draw_misc_stat_time( FALSE );
+	reset_day();
+
+	chk_morning_glow( g_turn );
+	chk_evening_glow( g_turn );
+	chk_discount_day();
+	chk_discount_queue();
+
+	inc_turn_all_item();
+	inc_turn_all_fx();
+	inc_turn_all_mnstr();
+
+	return g_turn;
+}
+
+/**/
+
+long	set_turn( long n )
+{
+	return reset_turn( n );
+}
+
+/**/
+
 long	get_turn( void )
 {
 	return g_turn;
@@ -83,35 +156,54 @@ long	get_turn( void )
 
 /**/
 
-long	set_turn( long n )
+long	reset_day( void )
 {
-	g_turn = n;
-	if( g_turn >= TURN_A_DAY ){
-		g_day += g_turn / TURN_A_DAY;
-		g_turn %= TURN_A_DAY;
-	}
+	for( ; g_turn >= TURN_A_DAY; g_turn -= TURN_A_DAY )
+		inc_day();
 
-	return g_turn;
+	return g_day;
 }
 
 /**/
 
-long	inc_turn( void )
+long	inc_day( void )
 {
-	g_turn++;
-	if( g_turn >= TURN_A_DAY ){
-		g_day += g_turn / TURN_A_DAY;
-		g_turn %= TURN_A_DAY;
-	}
-	chk_morning_glow( g_turn );
-	chk_evening_glow( g_turn );
-	chk_discount_day( g_day );
+	g_day++;
 
-	inc_turn_all_item();
-	inc_turn_all_fx();
-	inc_turn_all_mnstr();
+	inc_day_all_fx();
 
-	return g_turn;
+	chk_request_limit();
+	chk_request_contract();
+
+	return g_day;
+}
+
+/**/
+
+long	set_game_date( long year, long month, long day )
+{
+	g_day = 0;
+	g_day += year * 360;
+	g_day += (month - 1) * 30;
+	g_day += day - 1;
+
+	return g_day;
+}
+
+/**/
+
+long	set_game_day( long day )
+{
+	g_day = day;
+
+	return g_day;
+}
+
+/**/
+
+long	get_game_day( void )
+{
+	return g_day;
 }
 
 /**/
@@ -169,13 +261,14 @@ bool_t	chk_night( void )
 
 char	*get_calendar_str( void )
 {
+	static char	s[127 + 1];
 	long	game_day, turn;
 	long	year, month, day, hour, minute;
 	char	*week;
-	static char	s[127 + 1];
 
 	game_day = get_game_day();
 	turn = get_turn();
+
 	year = (game_day / 360);
 	month = (game_day / 30) % 12 + 1;
 	day = game_day % 30 + 1;
@@ -190,21 +283,19 @@ char	*get_calendar_str( void )
 
 /**/
 
-long	get_game_day( void )
+char	*get_stat_time_str( void )
 {
-	return g_day;
-}
+	static char	s[40 + 1];
+	long	turn;
+	long	hour, minute;
 
-/**/
+	turn = get_turn();
+	hour = (turn / TURN_PER_HOUR) % 24;
+	minute = (turn / TURN_PER_MINUTE) % 60;
 
-long	set_game_day( long year, long month, long day )
-{
-	g_day = 0;
-	g_day += year * 360;
-	g_day += (month - 1) * 30;
-	g_day += day - 1;
+	sprintf( s, MSG_STAT_TIME, hour, minute );
 
-	return g_day;
+	return s;
 }
 
 /**/
@@ -220,10 +311,10 @@ char	*get_day_of_week_name( void )
 		return MSG_DAY_OF_WEEK_HEAT;
 	case DAY_OF_WEEK_COLD:
 		return MSG_DAY_OF_WEEK_COLD;
-	case DAY_OF_WEEK_ACID:
-		return MSG_DAY_OF_WEEK_ACID;
 	case DAY_OF_WEEK_WIND:
 		return MSG_DAY_OF_WEEK_MIND;
+	case DAY_OF_WEEK_ACID:
+		return MSG_DAY_OF_WEEK_ACID;
 	case DAY_OF_WEEK_POIS:
 		return MSG_DAY_OF_WEEK_POIS;
 	case DAY_OF_WEEK_MAX_N:
