@@ -1,9 +1,8 @@
 /*****************************************************************
 * L&L - Labyrinths & Legends
-* Copyright (c) 1993-2003 YOSHIMURA Tomohiko All rights reserved.
+* Copyright (c) 1993-2004 YOSHIMURA Tomohiko All rights reserved.
 * 
 * Created by BowKenKen
-*   e-mail: bowkenken@users.sourceforge.jp
 *   URL: https://sourceforge.jp/projects/lnl/
 * 
 * License is GPL
@@ -26,37 +25,44 @@
 *   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 * ؼ񤤤Ƥ
 * 
-* $Id: ver.h,v 1.28 2003/11/19 13:19:14 bowkenken Exp $
+* $Id: ver.h,v 1.36 2004/07/29 07:52:22 bowkenken Exp $
 *****************************************************************/
 
-#ifndef	VER_H
-#define	VER_H
+#ifndef	GVER_H
+#define	GVER_H
 
 #define	STR_FILE_NAME_GAME	"lnl"
 #define	STR_TITLE	"L&L - Labyrinths & Legends :"
 #define	STR_SINCE	"1993"
-#define	STR_LATEST	"2003"
+#define	STR_LATEST	"2004"
 #define	STR_AUTHOR	"YOSHIMURA Tomohiko"
 #define	STR_HANDLE1	"BowKenKen"
 #define	STR_HANDLE2	""
-#define	STR_EMAIL	"bowkenken@users.sourceforge.jp"
-#define	STR_WEB	"https://sourceforge.jp/projects/lnl/"
+#define	STR_WEB	"http://lnl.sourceforge.jp/"
+#define	STR_E_MAIL	"bowkenken@users.sourceforge.jp"
+#define	STR_THANKS	"TeaMiKL"
 
 #define	STR_VER_GAME	VERSION
 
 #define	STR_FMT_COPYRIGHT	\
+		"\n" \
 		"%s version %s\n" \
 		"Copyright (c) %s-%s %s All rights reserved.\n" \
-		"Created by %s, %s\n" \
-		"\t%s\n" \
-		"\t%s\n"
+		"  Created by %s, %s\n" \
+		"  URL   : %s\n" \
+		"  e-mail: %s\n" \
+		"Special thanks %s and you.\n" \
+		"\n" \
+		""
+
 #define	LS_STR_COPYRIGHT	\
 		STR_TITLE, STR_VER_GAME, \
 		STR_SINCE, STR_LATEST, STR_AUTHOR, \
 		STR_HANDLE1, \
 		STR_HANDLE2, \
-		STR_EMAIL, \
-		STR_WEB
+		STR_WEB, \
+		STR_E_MAIL, \
+		STR_THANKS
 
-#endif	/* VER_H */
+#endif	/* GVER_H */
 
